"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _YIUIBase = require("../YIUI-base");

var _exceptions = require("../exceptions");

/**
 * 控件管理类
 * 控件类型管理类
 * @type {YIUI.ComponentMgr}
 */
var _default = _YIUIBase.YIUI.ComponentMgr = function () {
  // 所有控件
  var all = {}; // 所有控件类型

  var types = {};
  var Return = {
    /** 注册控件，以控件id为key */
    register: function register(c) {
      all[c.id] = c;
    },

    /** 取消注册控件，通常在destroy */
    unregister: function unregister(c) {
      all[c.id] = null;
    },

    /**
     * 根据控件id，获取已经注册的控件
     */
    get: function get(id) {
      return all[id];
    },
    all: all,

    /**
    * 检查某控件类型是否已经注册过
     */
    isRegistered: function isRegistered(type) {
      return types[type] !== undefined;
    },

    /**
     * 注册控件类型，控件type为key，控件class为value
     */
    registerType: function registerType(type, cls) {
      var t = type.toLowerCase();
      types[t] = cls;
      cls.type = type;
    },

    /**
     * 遍历所有已注册的所有类型。
     * @param fn 对每个遍历到的类型进行的操作，形如：function(typeName,type)
     */
    eachType: function eachType(fn) {
      if ($.isFunction(fn)) {
        for (var typeName in types) {
          fn(typeName, types[typeName]);
        }
      }
    },

    /**
     * 根据传入的json动态创建控件
     * @param config 传入的json
     * @param defaultType 如果json中未定义type，默认创建的控件
     * @returns {*}
     */
    create: function create(config, defaultType) {
      if (!types[config.tagName.toLowerCase()]) {
        // YIUI.ViewException.throwException(YIUI.ViewException.NO_COMPONENT, config.tagName);
        throw new _exceptions.ComponentMissingException(config.tagName);
      }

      return config.render ? config : new types[(config.tagName || defaultType).toLowerCase()](config);
    }
  };
  return Return;
}();
/**
 * YIUI.ComponentMgr#registerType的简写
 */


exports["default"] = _default;
_YIUIBase.YIUI.reg = _YIUIBase.YIUI.ComponentMgr.registerType;
/**
 * YIUI.ComponentMgr#create的简写
 */

_YIUIBase.YIUI.create = _YIUIBase.YIUI.ComponentMgr.create;
/**
 * YIUI.ComponentMgr#get的简写
 */

_YIUIBase.YIUI.get = _YIUIBase.YIUI.ComponentMgr.get;