/*
 * @Author: gmf
 * @Date:   2016-03-18 10:42:31
 * @Last Modified by:   gmf
 * @Last Modified time: 2016-12-07 11:28:01
 */

import BaseControl from './control';
import { YIUI } from '../../YIUI-base';
import { DataDef } from '../../YIUI-common';
import Svr from '../../YIUI-svr';
import { View } from '../../YIUI-parser';
import { regControlType } from './controlUtil';
import { lodash as $ } from 'yes-common';
import Immutable from 'immutable';
const ComboboxEditor = YIUI.extend(BaseControl, {
    needRebuild: true,
    /**
     * 下拉框数据
     */
    items: null,
    /**
     * 是否多选
     * @type Boolean
     */
    multiselect: false,
    hasText: false,
    sourceType: YIUI.COMBOBOX_SOURCETYPE.ITEMS,
    /**
     * 是否可编辑
     * @type Boolean
     */
    editable: true,
    // handler: YIUI.ComboBoxHandler,
    init(options) {
        this.base(options);
        const items = this.items || this.getMetaObj().items;
        this.getMetaObj().multiSelect = this.type === YIUI.CONTROLTYPE.CHECKLISTBOX;
        this.state = this.state.set('items', Immutable.fromJS(items || []));
    },
    async calculateDisplayValue(v, cxt) {
        try {
            v = v + '';
            await this.resetItems(false, cxt);
            const items = v.split(',');
            const captions = items.map((value) => {
                const tmp = this.state.get('items').find((item) => item.get('value') === value);
                return tmp ? tmp.get('caption') : '';
            });
            return captions.join(',');
        } catch (ex) {
            return '出错';
        }
    },
    async resetItems(force = false, cxt) {
        const items = await this.getComboboxItems(force, cxt);
        this.items = this.clearData(items);
        const state = this.state.mergeDeepIn(['items'], Immutable.fromJS(this.items));
        // this.items = items;
        this.changeState(state);
    },
    clearData(data) {
        const keys = [];
        const result = [];
        data.forEach((item) => {
            if (!keys.includes(item.value)) {
                result.push(item);
                keys.push(item.value);
            }
        });
        return result;
    },
    async getFormulaItems(form, meta, cxt) {
        let items = [];
        if (meta.formula) {
            // const f = '((OA_GetDropItemByBPM()+"xxxx")+OA_GetDropItemByBPM())+"yyyy"';
            // console.log(meta.formula);
            const rs = await form.evalFormula($.trim(meta.formula), cxt, null);
            // var rs = await form.eval(f,cxt,null);
            if ($.isString(rs)) {
                const item_Arr = rs.split(';');
                for (let i = 0, len = item_Arr.length; i < len; i++) {
                    const item_obj = item_Arr[i].split(',');
                    const i1 = item_obj[0];
                    let i2 = '';
                    if (item_obj.length > 1) {
                        i2 = item_obj[1];
                    } else {
                        i2 = i1;
                    }
                    const item = {
                        value: i1,
                        caption: i2,
                    };
                    items.push(item);
                }
            } else if (rs instanceof DataDef.DataTable) {
                rs.beforeFirst();
                if (rs.cols.length == 1) { // 如果查询的是一列,那么认为是查询了value,caption从全局中匹配
                    var globalItems = meta.globalItems;
                    var pItems = await form.evalFormula($.trim(globalItems), cxt, null);
                    while (rs.next()) {
                        var value = YIUI.TypeConvertor.toString(rs.get(0));
                        var caption = '';
                        if (pItems && pItems.length > 0) {
                            for (var i = 0, len = pItems.length; i < len; i++) {
                                var pItem = pItems[i];
                                if (pItem.value == value) {
                                    caption = pItem.caption;
                                    break;
                                }
                            }
                        }
                        caption = caption == '' ? value : caption;
                        var item = {
                            value,
                            caption,
                        };
                        items.push(item);
                    }
                } else if (rs.cols.length > 1) {
                    while (rs.next()) {
                        var value = YIUI.TypeConvertor.toString(rs.get(0));
                        var caption = YIUI.TypeConvertor.toString(rs.get(1));
                        var item = {
                            value,
                            caption,
                        };
                        items.push(item);
                    }
                }
            } else if ($.isArray(rs)) {
                items = items.concat(rs);
            }
        }

        return items;
    },
    async getQueryItems(form, meta, cxt) {
        var queryParas = meta.queryParas || [];
        var data = {}, type;
        data.formKey = form.getFormKey();
        data.fieldKey = meta.key;
        data.typeDefKey = meta.typeDefKey;
        data.queryIndex = 0;
        data.service = 'ComboBoxService';
        data.cmd = 'getQueryItems';

        var sourceType, value, paras = [];
        for (var i = 0, len = queryParas.length; i < len; i++) {
            sourceType = queryParas[i].sourceType;
            value = queryParas[i].value;
            switch (sourceType) {
                case YIUI.COMBOBOX_PARAMETERSOURCETYPE.CONST:
                    break;
                case YIUI.COMBOBOX_PARAMETERSOURCETYPE.FORMULA:
                case YIUI.COMBOBOX_PARAMETERSOURCETYPE.FIELD:
                    value = await form.evalFormula(value, cxt, null, true);
                    break;
            }
            if (value instanceof Date) {
                value = value.getTime();
            }
            paras.push(value);
        }
        data.paras = $.toJSON(paras);

        return await Svr.Request.getData(data);
    },
    async getComboboxItems(force, cxt) {
        const meta = this.getMetaObj();
        let form = null;
        let localCxt = cxt;
        const sourceType = meta.sourceType || this.sourceType || YIUI.COMBOBOX_SOURCETYPE.ITEMS;
        if (this.items && this.items.length > 0 && !force)
            return this.items;
        switch (sourceType) {
            case YIUI.COMBOBOX_SOURCETYPE.ITEMS:
            case YIUI.COMBOBOX_SOURCETYPE.STATUS:
            case YIUI.COMBOBOX_SOURCETYPE.PARAGROUP:
                return meta.items;
            case YIUI.COMBOBOX_SOURCETYPE.FORMULA:
                form = YIUI.FormStack.getForm(this.ofFormID);
                if (localCxt == null) {
                    localCxt = new View.Context(form);
                }
                const data = await this.getFormulaItems(form, meta.formula ? meta : this, localCxt);
                return data;
            case YIUI.COMBOBOX_SOURCETYPE.QUERY:
                form = YIUI.FormStack.getForm(this.ofFormID);
                if (localCxt == null) {
                    localCxt = new View.Context(form);
                }
                return await this.getQueryItems(form, meta, localCxt);
            default:
        }
        return null;
    },
});
YIUI.reg('combobox', ComboboxEditor);
YIUI.reg('checklistbox', ComboboxEditor);
regControlType(YIUI.CONTROLTYPE.COMBOBOX, 'combobox');
regControlType(YIUI.CONTROLTYPE.CHECKLISTBOX, 'checklistbox');
export default ComboboxEditor;
