"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _YIUIBase = require("../YIUI-base");

require("../YIUI-common");

require("../YIUI-parser");

require("../YIUI-svr");

function addCellLocation(adapter, rootCompKey, tableKey, comp) {
  if (comp.items) {
    comp.items.forEach(function (item) {
      addCellLocation(adapter, rootCompKey, tableKey, item);
    });
  } else {
    var cellLocation = {
      key: rootCompKey,
      column: -1,
      row: -1,
      tableKey: tableKey,
      columnKey: comp.metaObj.columnKey
    };
    adapter.addCellLocation(comp.metaObj.key, cellLocation);
  }
}

var _default = _YIUIBase.YIUI.GridLookupUtil = function () {
  var Return = {
    /**
     * 建立单元格查找
     * @param form
     * @param com
     */
    buildCellLookup: function buildCellLookup(form, com) {
      var cellLocation, columnInfo, metaRow, metaCell;

      switch (com.type) {
        case _YIUIBase.YIUI.CONTROLTYPE.GRID:
          var metaRows = com.getMetaObj().rows;

          for (var i = 0, size = metaRows.length; i < size; i++) {
            metaRow = metaRows[i];

            for (var c = 0, length = metaRow.cells.length; c < length; c++) {
              metaCell = metaRow.cells[c];
              cellLocation = {
                key: com.key,
                column: c,
                row: metaRow.rowType === 'Fix' ? i : -1,
                tableKey: com.tableKey,
                columnKey: metaCell.columnKey,
                expand: metaCell.isColExpand
              };
              form.formAdapt.addCellLocation(metaCell.key, cellLocation);
            }
          }

          break;

        case _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW:
          for (var i = 0, len = com.columnInfo.length; i < len; i++) {
            columnInfo = com.columnInfo[i];
            columnInfo.tableKey = com.tableKey;
            cellLocation = {
              key: com.key,
              column: i,
              row: -1,
              tableKey: com.tableKey,
              columnKey: columnInfo.columnKey
            };
            form.formAdapt.addCellLocation(columnInfo.key, cellLocation);
          }

          break;

        case _YIUIBase.YIUI.CONTROLTYPE.TABLEVIEW:
          //处理每一个TableViewRow
          com.items.forEach(function (row) {
            addCellLocation(form.formAdapt, com.key, row.TableKey, row.root);
          });
          break;
      }
    },

    /**
     * 更新固定行位置
     * @param form
     * @param grid
     */
    updateFixPos: function updateFixPos(form, grid) {
      if (!grid.hasFixRow) return;
      var cellKey;

      for (var i = 0, size = grid.getRowCount(); i < size; i++) {
        var rowData = grid.getRowDataAt(i);
        if (rowData.rowType !== "Fix") continue;

        for (var j = 0, length = rowData.cellKeys.length; j < length; j++) {
          cellKey = rowData.cellKeys[j];

          if (cellKey) {
            form.getCellLocation(cellKey).row = i;
          }
        }
      }
    }
  };
  return Return;
}();

exports["default"] = _default;