/*
 * @Author: gmf
 * @Date:   2016-03-15 16:13:00
 * @Last Modified by:   gmf
 * @Last Modified time: 2016-12-29 11:00:13
 */

import BaseControl from './control';
import Immutable from 'immutable';
import empty from 'empty';
import { regControlType, getTagName } from './controlUtil';
import { YIUI } from '../../YIUI-base';
import '../../YIUI-common';
import '../../YIUI-svr';
import { View } from '../../YIUI-parser';

YIUI.ListView = YIUI.extend(BaseControl, {
    handler: YIUI.ListViewHandler,
    init(options) {
        this.base(options);
        this.columnInfo = this.getMetaObj().columnInfo;
        this.state = this.state.set('columnInfo', Immutable.fromJS(this.columnInfo));
        this.state = this.state.set('data', Immutable.fromJS(this.data));
        this.editors = {};
        this.activeRowIndex = -1;
        this._pagination = {
            hidePagination: empty.func,
            setTotalRowCount: empty.func,
        };
    },
    findIndex(colKey) {
        const cols = this.columnInfo;
        let index = -1;
        for (let i = 0, len = cols.length; i < len; i++) {
            if (cols[i].key === colKey) {
                index = i;
                break;
            }
        }
        return index;
    },
    diff(diffJson) {
        const listView = this;
        listView.totalRowCount = diffJson.totalRowCount || listView.totalRowCount;
        const columnInfo = diffJson.columnInfo;
        // let oldState = this.state;
        let state = null;
        if (diffJson.data) {
            state = this.state.set('data', Immutable.fromJS(diffJson.data));
        } else if (columnInfo) {
            state = this.state.mergeDeep('columnInfo', columnInfo);
            state = this.state.set('data', Immutable.fromJS(diffJson.data));
        }
        this.changeState(state);
    },
    /**
     * 加载更多数据
     */
    async loadMoreData() {
        const controlKey = this.key,
            tableKey = this.tableKey;
        const form = YIUI.FormStack.getForm(this.ofFormID);
        const tblFilter = form.getFilterMap().getTblFilter(tableKey);
        const startRow = tblFilter.startRow + tblFilter.maxRows;
        form.getFilterMap().setStartRow(tableKey, startRow);
        let doc = await YIUI.DocService.loadFormData(form, form.getFilterMap().OID, form.getFilterMap(), form.getCondParas());
        doc = YIUI.DataUtil.fromJSONDoc(doc);
        const table = doc.getByKey(tableKey);
        const formDoc = form.getDocument();
        formDoc.getByKey(tableKey).concat(table);
        const rowList = [];
        for (var j = 0, length = table.getRowCount(); j < length; j++) {
            const row = {};
            // listView.data.push(row);
            table.setPos(j);
            // var bkmkRow = new YIUI.DetailRowBkmk();
            // bkmkRow.setBookmark(table.getBkmk());
            row.bkmkRow = startRow + table.getBkmk();
            for (var m = 0, size = this.columnInfo.length; m < size; m++) {
                var column = this.columnInfo[m];
                var columnKey = column.columnKey;
                var value = null;
                if (columnKey) {
                    value = table.getByKey(columnKey);
                } else {
                    if (column.type == YIUI.CONTROLTYPE.HYPERLINK ||
                        column.type == YIUI.CONTROLTYPE.BUTTON ||
                        column.type == YIUI.CONTROLTYPE.LABEL) {
                        value = column.caption;
                    } else if (column.type == YIUI.CONTROLTYPE.HYPERLINK) {
                        value = false;
                    }
                }
                // listView.setValueAt(j, m, value);
                row[columnKey] = value;
            }
            rowList.push(row);
        }
        this.addDataRow(rowList);
    },
    hasMore() {
        const tableKey = this.tableKey;
        const form = YIUI.FormStack.getForm(this.ofFormID);
        const tblFilter = form.getFilterMap().getTblFilter(tableKey);
        if(!tblFilter) {
            return false;
        }
        const pageRowCount = tblFilter.maxRows;
        if (this.getTotalRowCount() <= pageRowCount) {
            return false;
        }
        return true;
    },
    clearDataRow() {
        // this.data = [];
        // this.state = this.state.set('data',Immutable.fromJS(this.data));
    },
    clearAllRows() {
        this.data = [];
        const state = this.state.set('data', Immutable.fromJS(this.data));
        this.changeState(state);
    },
    getValue(rowIndex, colIndex) {
        if (rowIndex == null) {
            return this.data;
        }
        const dbKey = this.columnInfo[colIndex].columnKey;
        return this.getValByKey(rowIndex, dbKey);
    },
    getValByKey(rowIndex, colKey) {
        const colCfg = this.getColumnConfig(colKey);
        return this.state.getIn(['data', rowIndex, colCfg.columnKey]);
    },
    async rowClick(rowIndex) {
        var form = YIUI.FormStack.getForm(this.ofFormID);
        var cxt = new View.Context(form);
        cxt.setRowIndex(rowIndex);
        this.activeRowIndex = rowIndex;
        const formula = this.getMetaObj().rowDblClick;
        if (formula) {
            await form.eval(formula, cxt, null);
        }
    },
    setDisplayValueByKey(rowIndex, colKey, v) {
    },
    getDisplayValueByKey(rowIndex, colKey) {
        return '';
    },
    getRowCount() {
        this.state.get('data').size;
    },
    deleteRow(rowIndex) {
        // this.data.splice(rowIndex,1);
        const state = this.state.deleteIn(['data', rowIndex]);
        this.changeState(state);
    },
    addNewRow(row) {
        if (row) {

        }
        // 新增行，首先计算新增空白行的数据，这个数据需要缓存下来，
        this.data.push(row);
        const newRow = [];
        newRow.push(row);
        this.addDataRow(newRow);
    },
    getEmtpyRowData() {
        return this.emptyRowData || this.calculateEmptyData();
    },
    async calculateEmptyRowData() {
        this.emptyRowData = [];
        const form = YIUI.FormStack.getForm(this.ofFormID);
        for (let column in this.columnInfo) {
            let v = '';
            if (column.defaultValueFormula) {//设置了默认数值公式
                v = await form.evalFormulua(column.defaultValueFormulua);
            }
            if (column.defaultValue) {//设置了默认数值
                v = column.defaultValue;
            }
            this.emptyRowData.push(v);
        }
        return this.emptyRowData();
    },
    setColumnVisible(key, visible) {
        let editor = this.columnInfo.find((item) => {
            return item.key === key;
        });
        editor.visible = visible;
    },
    setColumnEnable(key, enable) {
        let editor = this.columnInfo.find((item) => {
            return item.key === key;
        });
        editor.enable = enable;
    },
    addDataRow(rows) {
        // if (rows) {
        //     this.data = rows;
        // }
        const data = this.state.get('data');
        const state = this.state.set('data', 
            data.concat(Immutable.fromJS(rows))
        );
        this.changeState(state);
        // this.state = this.state.set('data', Immutable.fromJS(this.data));
    },
    resetData(data) {
        this.data = data;
        const state = this.state.set('data', Immutable.fromJS(this.data));
        this.changeState(state);
    },
    setTotalRowCount(v) {
        const state = this.state.set('totalRowCount', v);
        this.changeState(state);
    },
    getTotalRowCount() {
        return this.state.get('totalRowCount');
    },
    getColumnConfig(columnKey) {
        return this.columnInfo.find((item) => {
            return item.key === columnKey || item.columnKey === columnKey;
        });
    },
    /**
     * 获取所有列的控件
     */
    getAllColumnEditors() {
        this.columnInfo.forEach(item => {
            this.getColumnEditor(item.key);
        });
        return Object.values(this.editors);
    },
    getColumnEditor(columnKey) {
        if (!this.editors[columnKey]) {
            const editor = this.columnInfo.find((item) => {
                return item.key === columnKey || item.columnKey === columnKey;
            });
            if (!editor) {
                return null;
            }
            editor.tagName = getTagName(editor.columnType);
            editor.ofFormID = this.ofFormID;
            // editor.metaObj = editor;
            const cmp = YIUI.create(editor);
            this.editors[columnKey] = cmp;
        }
        return this.editors[columnKey];
    },
    getFocusRowIndex() {
        return this.activeRowIndex;
    },
    createColumnHead: empty.func,
    _createHandles: empty.func,
    syncHandleWidths: empty.func,
});
YIUI.reg('listview', YIUI.ListView);
YIUI.reg('rotator', YIUI.ListView);
YIUI.reg('gallery', YIUI.ListView);
regControlType(YIUI.CONTROLTYPE.LISTVIEW, 'listview');
export default YIUI.ListView;
