"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.View = void 0;

var _YIUIBase = require("./YIUI-base");

var _YIUISvr = _interopRequireDefault(require("./YIUI-svr"));

var _conditionparas = require("./component/conditionparas");

var _YIUICommon = require("./YIUI-common");

var _decimal = _interopRequireDefault(require("decimal.js"));

var _yesCommon = require("yes-common");

var _cache = _interopRequireDefault(require("./cache"));

var _dispatchers = _interopRequireDefault(require("./dispatchers"));

var _actions = require("./actions");

var _session = require("./session");

require("./services");

require("./opt");

var _confirm = _interopRequireDefault(require("./utils/confirm"));

var _exceptions = require("./exceptions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var Expr = {};

(function () {
  Expr.rs = null;

  Expr.rules = function () {
    if (this.rs == null) {
      this.rs = new Expr.ParserRules();
    }

    return this.rs;
  };

  Expr.regCluster = function (map, c) {
    for (var p in c) {
      map.put(p, c[p]);
    }
  };

  Expr.eq = function (s1, s2) {
    return s1.toString().toLowerCase() == s2.toString().toLowerCase();
  };

  Expr.ADD = 0;
  Expr.STRCAT = 1;
  Expr.SUB = 2;
  Expr.MUL = 3;
  Expr.DIV = 4;
  Expr.OR = 5;
  Expr.AND = 6;
  Expr.EQ = 7;
  Expr.ASSIGN = 8;
  Expr.NEQ = 9;
  Expr.GT = 10;
  Expr.GT_EQ = 11;
  Expr.LT = 12;
  Expr.LT_EQ = 13;
  Expr.FUN = 14;
  Expr.ID = 15;
  Expr.CONST = 16;
  Expr.LB = 17;
  Expr.RB = 18;
  Expr.COMMA = 19;
  Expr.DOLLAR = 20;
  Expr.IF = 21;
  Expr.ELSE = 22;
  Expr.WHILE = 23;
  Expr.VAR = 24;
  Expr.NOT = 25;
  Expr.SEMI = 26;
  Expr.L_BR = 27;
  Expr.R_BR = 28;
  Expr.RETURN = 29;
  Expr.BREAK = 30;
  Expr.LOOP = 31;
  Expr.RANGE = 32;
  Expr.SWITCH = 33;
  Expr.CASE = 34;
  Expr.COLON = 35;
  Expr.SL = 50;
  Expr.E = 51;
  Expr.IF_H = 52;
  Expr.IF_T = 53;
  Expr.FUN_H = 54;
  Expr.FUN_T = 55;
  Expr.PL = 56;
  Expr.SWITCH_S = 57;
  Expr.CASE_S = 58;
  Expr.CASE_SL = 59;
  Expr.MAX = 60;
  Expr.SELF = 0;
  Expr.PARENT = 1;
  Expr.T_INT = 0;
  Expr.T_NUM = 1;
  Expr.T_STR = 2;
  Expr.T_BOOL = 3;
  Expr.IIF = "iif";
  Expr.IIFS = "iifs";
  Expr.S_SELF = "self";
  Expr.S_PARENT = "parent";
  Expr.F_RETURN = 0;
  Expr.F_BREAK = 1;

  Expr.RefInt = function (v) {
    this.v = v;
  };

  _YIUICommon.Lang.impl(Expr.RefInt, {
    set: function set(v) {
      this.v = v;
    },
    get: function get() {
      return this.v;
    },
    inc: function inc() {
      ++this.v;
    },
    dec: function dec() {
      --this.v;
    }
  });

  Expr.RefBool = function (v) {
    this.v = v;
  };

  _YIUICommon.Lang.impl(Expr.RefBool, {
    set: function set(v) {
      this.v = v;
    },
    get: function get() {
      return this.v;
    }
  });

  Expr.Lexer = function (s) {
    this.s = s;
    this.len = s ? s.length : 0;
    this.li = null;
    this.lv = null;
    this.type = -1;
    this.start = -1;
    this.pos = 0;
    this.b_skip = false;
    this.c_skip = false;
    this.obj = -1;
    this.err = -1;
    this.lastID = -1;
  };

  _YIUICommon.Lang.impl(Expr.Lexer, {
    setContent: function setContent(s) {
      this.s = s;
      this.pos = 0;
      this.len = s.length;
    },
    skipB: function skipB(c) {
      var r = c;
      this.b_skip = false;

      while (c == ' ' || c == '\t' || c == '\n' || c == '\r') {
        this.b_skip = true;

        if (this.pos < this.len) {
          c = this.s.charAt(this.pos);
        } else {
          this.err = -1;
          return r;
        }

        r = c;
        ++this.pos;
        ++this.start;
      }

      return r;
    },
    skipC: function skipC(c) {
      var r = c;
      this.c_skip = false;

      if (this.pos < this.len) {
        var oldPos = this.pos;
        var oldStart = this.start;
        var found = false;

        if (c == '/') {
          if (this.pos < this.len) {
            c = this.s.charAt(this.pos);
            ++this.pos;
            ++this.start;

            if (c == '/') {
              if (this.pos < this.len) {
                c = this.s.charAt(this.pos);
                ++this.pos;
                ++this.start;

                while (c != '\n' && this.pos < this.len) {
                  if (this.pos < this.len) {
                    c = this.s.charAt(this.pos);
                    ++this.pos;
                    ++this.start;
                  }
                }

                if (this.pos < this.len) {
                  c = this.s.charAt(this.pos);
                  ++this.pos;
                  ++this.start;
                }

                r = c;
                found = true;
              }
            } else if (c == '*') {
              if (this.pos < this.len) {
                c = this.s.charAt(this.pos);
                ++this.pos;
                ++this.start;

                while (c != '*' && this.pos < this.len) {
                  if (this.pos < this.len) {
                    c = this.s.charAt(this.pos);
                    ++this.pos;
                    ++this.start;
                  }
                }

                ++this.pos;
                ++this.start;

                if (this.pos < this.len) {
                  c = this.s.charAt(this.pos);
                  ++this.pos;
                  ++this.start;
                }

                r = c;
                found = true;
              }
            }
          }
        }

        if (!found) {
          this.pos = oldPos;
          this.start = oldStart;
        } else {
          this.c_skip = true;
        }
      }

      return r;
    },
    lookSemi: function lookSemi() {
      var id = 26;
      var oldPos = this.pos;
      var c = ' ';

      if (this.pos < this.len) {
        c = this.s.charAt(this.pos);
        ++this.pos;

        do {
          c = this.skipB(c);

          if (this.err == -1) {
            return -1;
          }

          c = this.skipC(c);
          c = this.skipB(c);

          if (this.err == -1) {
            return -1;
          }
        } while (this.b_skip || this.c_skip);

        if (c == '}') {
          id = 28;
          this.li = "}";
          this.lv = "}";
          return id;
        } else {
          this.pos = oldPos;
        }
      } else {
        id = -1;
      }

      return id;
    },
    lookahead: function lookahead(ahc) {
      var oldPos = this.pos;
      var c = ' ';

      if (this.pos < this.len) {
        c = this.s.charAt(this.pos);
        ++this.pos;

        do {
          c = this.skipB(c);

          if (this.err == -1) {
            return false;
          }

          c = this.skipC(c);
          c = this.skipB(c);

          if (this.err == -1) {
            return false;
          }
        } while (this.b_skip || this.c_skip);

        if (c == ahc) {
          this.pos = oldPos;
          return true;
        } else {
          this.pos = oldPos;
        }
      }

      this.pos = oldPos;
      return false;
    },
    neg: function neg() {
      return this.lastID === -1 || this.lastID >= 0 && this.lastID <= 13 || this.lastID === 17 || this.lastID === 19;
    },
    resolveConst: function resolveConst(t) {
      if (Expr.eq(t.toLowerCase(), "true")) {
        this.lv = "true";
        this.type = Expr.T_BOOL;
        return 16;
      } else if (Expr.eq(t.toLowerCase(), "false")) {
        this.lv = "false";
        this.type = Expr.T_BOOL;
        return 16;
      }

      return -1;
    },
    resolveID: function resolveID(t) {
      var idx = t.indexOf('.');

      if (idx > 0) {
        var first = t.substring(0, idx);
        var second = t.substring(idx + 1);
        t = second;
        this.obj = first;
        this.lv = second;
      } else {
        this.obj = null;
        this.lv = t;
      }

      var id = -1;

      if (Expr.eq(t, "if")) {
        id = 21;
      } else if (Expr.eq(t, "else")) {
        id = 22;
      } else if (Expr.eq(t, "while")) {
        id = 23;
      } else if (Expr.eq(t, "var")) {
        id = 24;
      } else if (Expr.eq(t, "return")) {
        id = 29;
      } else if (Expr.eq(t, "break")) {
        id = 30;
      } else if (Expr.eq(t, "loop")) {
        id = 31;
      } else if (Expr.eq(t, "switch")) {
        id = 33;
      } else if (Expr.eq(t, "case")) {
        id = 34;
      }

      return id;
    },
    next: function next(slAsString) {
      this.err = 0;
      this.obj = null;
      var oldPos = this.pos;
      var id = -1;
      var c;
      this.start = this.pos;
      if (this.pos >= this.len) return -1;
      c = this.s.charAt(this.pos);
      ++this.pos;

      do {
        c = this.skipB(c);

        if (this.err == -1) {
          return -1;
        }

        c = this.skipC(c);
        c = this.skipB(c);

        if (this.err == -1) {
          return -1;
        }
      } while (this.b_skip || this.c_skip);

      var cnt = 1;

      switch (c) {
        case '+':
          id = 0;
          this.li = "+";
          this.lv = "+";
          break;

        case '-':
          if (this.neg()) {
            var tmpC = ' ';

            if (this.pos < this.len) {
              tmpC = this.s.charAt(this.pos);
            }

            if (tmpC >= '0' && tmpC <= '9') {
              var isInt = true;

              if (this.pos < this.len) {
                c = this.s.charAt(this.pos);
              }

              while (c >= '0' && c <= '9' || c == '.') {
                if (c == '.') isInt = false;
                ++cnt;
                ++this.pos;

                if (this.pos >= this.len) {
                  break;
                }

                c = this.s.charAt(this.pos);
              }

              id = 16;
              this.li = "const";
              this.lv = this.s.substring(this.start, this.start + cnt);
              if (isInt) {
                if (this.lv.length > 10) {
                  this.type = Expr.T_NUM;
                } else {
                  this.type = Expr.T_INT;
                }
              } else this.type = Expr.T_NUM;
            } else {
              id = 2;
              this.li = "-";
              this.lv = "-";
            }
          } else {
            id = 2;
            this.li = "-";
            this.lv = "-";
          }

          break;

        case '*':
          id = 3;
          this.li = "*";
          this.lv = "*";
          break;

        case '/':
          id = 4;
          this.li = "/";
          this.lv = "/";
          break;

        case '(':
          id = 17;
          this.li = "(";
          this.lv = "(";
          break;

        case '{':
          if (slAsString) {
            var deep = 1;

            do {
              if (this.pos < this.len) {
                c = this.s.charAt(this.pos);

                if (c == '{') {
                  ++deep;
                } else if (c == '}') {
                  --deep;
                }

                ++this.pos;
                ++cnt;
              } else {
                this.err = 1;
                return -1;
              }
            } while (c != '}' || deep != 0);

            id = 16;
            this.type = Expr.T_STR;
            this.li = "const";
            this.lv = this.s.substring(this.start + 1, this.start + cnt - 1);
          } else {
            id = 27;
            this.li = "{";
            this.lv = "{";
          }

          break;

        case ')':
          id = 18;
          this.li = ")";
          this.lv = ")";
          break;

        case '}':
          id = 28;
          this.li = "}";
          this.lv = "}";
          break;

        case ',':
          id = 19;
          this.li = ",";
          this.lv = ",";
          break;

        case ';':
          id = 26;
          this.li = ";";
          this.lv = ";";
          id = this.lookSemi();
          break;

        case '!':
          if (this.pos < this.len) {
            c = this.s.charAt(this.pos);

            if (c == '=') {
              id = 9;
              this.li = "<>";
              this.lv = "<>";
              ++this.pos;
            } else {
              id = 25;
              this.li = "!";
              this.lv = "!";
            }
          } else {
            id = 25;
            this.li = "!";
            this.lv = "!";
          }

          break;

        case '=':
          if (this.pos < this.len) {
            c = this.s.charAt(this.pos);

            if (c == '=') {
              id = 7;
              this.li = "==";
              this.lv = "==";
              ++this.pos;
            } else {
              id = 8;
              this.li = "=";
              this.lv = "=";
            }
          } else {
            id = 8;
            this.li = "=";
            this.lv = "=";
          }

          break;

        case '&':
          if (this.pos < this.len) {
            c = this.s.charAt(this.pos);

            if (c == '&') {
              id = 6;
              this.li = "&&";
              this.lv = "&&";
              ++this.pos;
            } else {
              id = 1;
              this.li = "&";
              this.lv = "&";
            }
          } else {
            id = 1;
            this.li = "&";
            this.lv = "&";
          }

          break;

        case '|':
          id = -1;

          if (this.pos < this.len) {
            c = this.s.charAt(this.pos);

            if (c == '|') {
              id = 5;
              this.li = "||";
              this.lv = "||";
              ++this.pos;
            }
          }

          break;

        case '>':
          if (this.pos < this.len) {
            c = this.s.charAt(this.pos);

            if (c == '=') {
              id = 11;
              this.li = ">=";
              this.lv = ">=";
              ++this.pos;
            } else {
              id = 10;
              this.li = ">";
              this.lv = ">";
            }
          } else {
            id = 10;
            this.li = ">";
            this.lv = ">";
          }

          break;

        case '<':
          if (this.pos < this.len) {
            c = this.s.charAt(this.pos);

            if (c == '=') {
              id = 13;
              this.li = "<=";
              this.lv = "<=";
              ++this.pos;
            } else if (c == '>') {
              id = 9;
              this.li = "<>";
              this.lv = "<>";
              ++this.pos;
            } else {
              id = 12;
              this.li = "<";
              this.lv = "<";
            }
          } else {
            id = 12;
            this.li = "<";
            this.lv = "<";
          }

          break;

        case '$':
          id = 20;
          this.li = "$";
          this.lv = "$";
          break;

        case '\'':
          do {
            if (this.pos < this.len) {
              c = this.s.charAt(this.pos);
              ++this.pos;
              ++cnt;
            } else {
              this.err = 1;
              return -1;
            }
          } while (c != '\'');

          id = 16;
          this.li = "const";
          this.type = Expr.T_STR;
          this.lv = this.s.substring(this.start + 1, this.start + cnt - 1);
          break;

        case '"':
          do {
            if (this.pos < this.len) {
              c = this.s.charAt(this.pos);
              ++this.pos;
              ++cnt;
            } else {
              this.err = 1;
              return -1;
            }
          } while (c != '"');

          id = 16;
          this.type = Expr.T_STR;
          this.li = "const";
          this.lv = this.s.substring(this.start + 1, this.start + cnt - 1);
          break;

        case ':':
          id = 35;
          this.li = ":";
          this.lv = ":";
          break;

        default:
          if (c >= 'A' && c <= 'Z' || c >= 'a' && c <= 'z') {
            var hasColon = false;

            if (this.pos < this.len) {
              c = this.s.charAt(this.pos);

              while (c >= 'A' && c <= 'Z' || c >= 'a' && c <= 'z' || c >= '0' && c <= '9' || c == '.' || c == '_' || c == ':') {
                if (c == ':') {
                  hasColon = true;
                }

                ++cnt;
                ++this.pos;

                if (this.pos >= this.len) {
                  break;
                }

                c = this.s.charAt(this.pos);
              }
            }

            if (hasColon) {
              id = 32;
              this.li = "range";
              this.lv = this.s.substring(this.start, this.start + cnt);
            } else {
              id = 15;
              this.li = "id";
              this.lv = this.s.substring(this.start, this.start + cnt);
              var nID = -1;
              nID = this.resolveConst(this.lv);

              if (nID != -1) {
                id = nID;
                this.li = "const";
              } else {
                nID = this.resolveID(this.lv);

                if (nID != -1) {
                  id = nID;
                  this.li = this.lv;
                } else {
                  if (this.lookahead('(')) {
                    id = 14;
                    this.li = "fun";
                  }
                }
              }
            }
          } else if (c >= '0' && c <= '9') {
            var isInteger = true;

            if (this.pos < this.len) {
              c = this.s.charAt(this.pos);

              while (c >= '0' && c <= '9' || c == '.') {
                if (c == '.') isInteger = false;
                ++cnt;
                ++this.pos;

                if (this.pos >= this.len) {
                  break;
                }

                c = this.s.charAt(this.pos);
              }
            }

            id = 16;

            if (isInteger) {
              this.type = Expr.T_INT;
            } else {
              this.type = Expr.T_NUM;
            }

            this.li = "const";
            this.lv = this.s.substring(this.start, this.start + cnt);
          }

          break;
      } // 如果上次的符号是}，那么如果符号不是(-1)、(;)、(else)、(})，则需要补充(;)，此时需要回退输出缓冲区


      if (this.lastID == Expr.R_BR) {
        if (id != -1 && id != Expr.SEMI && id != Expr.ELSE && id != Expr.R_BR) {
          id = Expr.SEMI;
          this.li = ";";
          this.lv = ";";
          this.pos = oldPos;
        }
      }

      this.lastID = id;
      return id;
    },
    getLexID: function getLexID() {
      return this.li;
    },
    getLexValue: function getLexValue() {
      return this.lv;
    },
    getType: function getType() {
      return this.type;
    },
    getErr: function getErr() {
      return this.err;
    },
    getObj: function getObj() {
      return this.obj;
    }
  });

  Expr.Factor = function (id, name, ter) {
    this.id = id;
    this.name = name;
    this.ter = ter;
  };

  _YIUICommon.Lang.impl(Expr.Factor, {
    get: function get() {
      return this.id;
    },
    isTer: function isTer() {
      return this.ter;
    },
    getName: function getName() {
      return this.name;
    }
  });

  Expr.Lookup = function (fid) {
    this.fid = fid;
    this.inMap = null;
    this.exID = -1;
    this.exRule = null;
  };

  _YIUICommon.Lang.impl(Expr.Lookup, {
    getIn: function getIn(ffID) {
      return this.inMap == null ? null : this.inMap[ffID];
    },
    addIn: function addIn(ffID, r) {
      this.inMap = this.inMap || new Array(Expr.MAX);
      this.inMap[ffID] = r;
    },
    getEx: function getEx(ffID) {
      if (this.exID == -1) {
        return null;
      }

      return this.exID == ffID ? null : this.exRule;
    },
    addEx: function addEx(ffID, r) {
      this.exID = ffID;
      this.exRule = r;
    }
  });

  Expr.Rule = function (idx, l, rs, closeRule) {
    this.idx = idx;
    this.l = l;
    this.rs = rs;
    this.closeRule = closeRule;
    this.foMap = null;
    this.flag = 0;
    this.slAsString = true;
  };

  _YIUICommon.Lang.impl(Expr.Rule, {
    index: function index() {
      return this.idx;
    },
    left: function left() {
      return this.l;
    },
    right: function right() {
      return this.rs;
    },
    rightAt: function rightAt(idx) {
      return this.rs[idx];
    },
    rightLen: function rightLen() {
      return this.rs.length;
    },
    isClose: function isClose() {
      return this.closeRule;
    },
    setFlag: function setFlag(flag) {
      this.flag = flag;
    },
    getFlag: function getFlag() {
      return this.flag;
    },
    setSLAsString: function setSLAsString(slAsString) {
      this.slAsString = slAsString;
    },
    isSLAsString: function isSLAsString() {
      return this.slAsString;
    }
  });

  Expr.ParserRules = function () {
    this.fMap = new Array(Expr.MAX);
    this.rs = new Array( // SL -> E
    this.RULE(0, true, true, this.SL, [this.E]), // SL -> E ; SL
    this.RULE(1, false, true, this.SL, [this.E, this.SEMI, this.SL]), // E -> E + E
    this.RULE(2, false, true, this.E, [this.E, this.ADD, this.E]), // E -> E - E
    this.RULE(3, false, true, this.E, [this.E, this.SUB, this.E]), // E -> E * E
    this.RULE(4, false, true, this.E, [this.E, this.MUL, this.E]), // E -> E / E
    this.RULE(5, false, true, this.E, [this.E, this.DIV, this.E]), // E -> E & E
    this.RULE(6, false, true, this.E, [this.E, this.STRCAT, this.E]), // E -> ( E )
    this.RULE(7, false, true, this.E, [this.LB, this.E, this.RB]), // E -> ! E
    this.RULE(8, false, true, this.E, [this.NOT, this.E]), // E -> E || E
    this.RULE(9, false, true, this.E, [this.E, this.OR, this.E]), // E -> E && E
    this.RULE(10, false, true, this.E, [this.E, this.AND, this.E]), // E -> E == E
    this.RULE(11, false, true, this.E, [this.E, this.EQ, this.E]), // E -> E <> E
    this.RULE(12, false, true, this.E, [this.E, this.NEQ, this.E]), // E -> E > E
    this.RULE(13, false, true, this.E, [this.E, this.GT, this.E]), // E -> E >= E
    this.RULE(14, false, true, this.E, [this.E, this.GT_EQ, this.E]), // E -> E < E
    this.RULE(15, false, true, this.E, [this.E, this.LT, this.E]), // E -> E <= E
    this.RULE(16, false, true, this.E, [this.E, this.LT_EQ, this.E]), // E -> const
    this.RULE(17, false, true, this.E, [this.CONST]), // E -> id
    this.RULE(18, false, true, this.E, [this.ID]), // E -> FUNC_HEAD FUNC_TAIL
    this.RULE(19, false, true, this.E, [this.FUN_H, this.FUN_T]), // FUNC_HEAD -> function (
    this.RULE(20, false, true, this.FUN_H, [this.FUN, this.LB]), // FUNC_TAIL -> )
    this.RULE(21, false, true, this.FUN_T, [this.RB]), // FUNC_TAIL -> PL )
    this.RULE(22, false, true, this.FUN_T, [this.PL, this.RB]), // PL -> E
    this.RULE(23, true, true, this.PL, [this.E]), // PL -> E , PL
    this.RULE(24, false, true, this.PL, [this.E, this.COMMA, this.PL]), // E -> var id = E
    this.RULE(25, false, true, this.E, [this.VAR, this.ID, this.ASSIGN, this.E]), // E -> IF_HEAD
    this.RULE(26, false, true, this.E, [this.IF_HEAD]), // E -> IF_HEAD IF_TAIL
    this.RULE(27, false, true, this.E, [this.IF_HEAD, this.IF_TAIL]), // IF_HEAD -> if ( E ) { SL }
    this.RULE(28, false, false, this.IF_HEAD, [this.IF, this.LB, this.E, this.RB, this.L_BRACE, this.SL, this.R_BRACE]), // IF_TAIL -> else { SL }
    this.RULE(29, false, false, this.IF_TAIL, [this.ELSE, this.L_BRACE, this.SL, this.R_BRACE]), // E -> while ( E ) { SL }
    this.RULE(30, false, false, this.E, [this.WHILE, this.LB, this.E, this.RB, this.L_BRACE, this.SL, this.R_BRACE]), // E -> E = E
    this.RULE(31, false, true, this.E, [this.E, this.ASSIGN, this.E]), // E -> return E
    this.RULE(32, false, true, this.E, [this.RETURN, this.E]), // E -> break
    this.RULE(33, false, true, this.E, [this.BREAK]), // E -> loop E ( E ) { SL }
    this.RULE(34, false, false, this.E, [this.LOOP, this.E, this.LB, this.E, this.RB, this.L_BRACE, this.SL, this.R_BRACE]), // E -> RANGE
    this.RULE(35, false, true, this.E, [this.RANGE]), // E -> switch ( E ) { CASE_SL }
    this.RULE(36, false, false, this.E, [this.SWITCH, this.LB, this.E, this.RB, this.L_BRACE, this.CASE_SL, this.R_BRACE]), // CASE_SL -> CASE_S
    this.RULE(37, false, true, this.CASE_SL, [this.CASE_S]), // CASE_SL -> CASE_S ; CASE_SL
    this.RULE(38, false, true, this.CASE_SL, [this.CASE_S, this.SEMI, this.CASE_SL]), // CASE_S -> case E : { SL }
    this.RULE(39, false, false, this.CASE_S, [this.CASE, this.E, this.COLON, this.L_BRACE, this.SL, this.R_BRACE]));
  };

  _YIUICommon.Lang.impl(Expr.ParserRules, {
    first: function first(id) {
      return this.fMap[id];
    },
    at: function at(idx) {
      return this.rs[idx];
    },
    RULE: function RULE(idx, closeRule, slAsString, l, rs) {
      var r = new Expr.Rule(idx, l, rs, closeRule);
      r.setSLAsString(slAsString);
      var len = rs.length;

      for (var i = 0; i < len; ++i) {
        var rrf = rs[i];

        if (i == 0 && rrf.isTer()) {
          this.fMap[rrf.get()] = r;
        }
      }

      return r;
    },
    SL: new Expr.Factor(Expr.SL, "SL", false),
    E: new Expr.Factor(Expr.E, "E", false),
    IF_HEAD: new Expr.Factor(Expr.IF_H, "IF_HEAD", false),
    IF_TAIL: new Expr.Factor(Expr.IF_T, "IF_TAIL", false),
    FUN_H: new Expr.Factor(Expr.FUN_H, "FUNC_HEAD", false),
    FUN_T: new Expr.Factor(Expr.FUN_T, "FUNC_TAIL", false),
    PL: new Expr.Factor(Expr.PL, "PL", false),
    ADD: new Expr.Factor(0, "+", true),
    STRCAT: new Expr.Factor(1, "&", true),
    SUB: new Expr.Factor(2, "-", true),
    MUL: new Expr.Factor(3, "*", true),
    DIV: new Expr.Factor(4, "/", true),
    OR: new Expr.Factor(5, "||", true),
    AND: new Expr.Factor(6, "&&", true),
    EQ: new Expr.Factor(7, "==", true),
    ASSIGN: new Expr.Factor(8, "=", true),
    NEQ: new Expr.Factor(9, "<>", true),
    GT: new Expr.Factor(10, ">", true),
    GT_EQ: new Expr.Factor(11, ">=", true),
    LT: new Expr.Factor(12, "<", true),
    LT_EQ: new Expr.Factor(13, "<=", true),
    FUN: new Expr.Factor(14, "fun", true),
    ID: new Expr.Factor(15, "id", true),
    CONST: new Expr.Factor(16, "const", true),
    LB: new Expr.Factor(17, "(", true),
    RB: new Expr.Factor(18, ")", true),
    COMMA: new Expr.Factor(19, ",", true),
    IF: new Expr.Factor(21, "if", true),
    ELSE: new Expr.Factor(22, "else", true),
    WHILE: new Expr.Factor(23, "while", true),
    VAR: new Expr.Factor(24, "var", true),
    NOT: new Expr.Factor(25, "!", true),
    SEMI: new Expr.Factor(26, ";", true),
    L_BRACE: new Expr.Factor(27, "{", true),
    R_BRACE: new Expr.Factor(28, "}", true),
    RETURN: new Expr.Factor(29, "return", true),
    BREAK: new Expr.Factor(30, "break", true),
    LOOP: new Expr.Factor(31, "loop", true),
    RANGE: new Expr.Factor(32, "range", true),
    SWITCH: new Expr.Factor(33, "switch", true),
    CASE: new Expr.Factor(34, "case", true),
    COLON: new Expr.Factor(35, ":", true),
    CASE_S: new Expr.Factor(58, "CASE_S", false),
    CASE_SL: new Expr.Factor(59, "CASE_SL", false)
  });

  Expr.Heap = function () {
    this.varMap = new _YIUICommon.HashMap();
  };

  _YIUICommon.Lang.impl(Expr.Heap, {
    put: function put(name, value) {
      this.varMap.put(name, value);
    },
    get: function get(name) {
      return this.varMap.get(name);
    },
    contains: function contains(name) {
      return this.varMap.containsKey(name);
    }
  });

  Expr.Scope = function (parent) {
    this.parent = parent;
    this.paras = null;
    this.heap = new Expr.Heap();
  };

  _YIUICommon.Lang.impl(Expr.Scope, {
    getHeap: function getHeap() {
      return this.heap;
    },
    getParent: function getParent() {
      return this.parent;
    },
    addPara: function addPara(key, value) {
      if (this.paras == null) {
        this.paras = new _YIUICommon.HashMap();
      }

      this.paras.put(key, value);
    },
    getPara: function getPara(key) {
      return this.paras != null ? this.paras.get(key) : null;
    }
  });

  Expr.Item = function (f, r) {
    this.f = f;
    this.r = r;
    this.cl = null;
    this.obj = null;
    this.lv = null;
    this.value = null;
    this.ctrl = false;
  };

  _YIUICommon.Lang.impl(Expr.Item, {
    rule: function rule() {
      return this.r;
    },
    factor: function factor() {
      return this.f;
    },
    add: function add(factor) {
      this.cl = this.cl || new Array();
      this.cl.push(factor);
    },
    all: function all() {
      return this.cl;
    },
    size: function size() {
      return this.cl == null ? 0 : this.cl.length;
    },
    item: function item(idx) {
      return this.cl[idx];
    },
    set: function set(value) {
      this.value = value;
    },
    get: function get() {
      return this.value;
    },
    setLex: function setLex(lv) {
      this.lv = lv;
    },
    getLex: function getLex() {
      return this.lv;
    },
    setCtrl: function setCtrl(ctrl) {
      this.ctrl = ctrl;
    },
    isCtrl: function isCtrl() {
      return this.ctrl;
    },
    setObj: function setObj(obj) {
      this.obj = obj;
    },
    getObj: function getObj() {
      return this.obj;
    },
    setChild: function setChild(idx, value) {
      this.cl[idx].set(value);
    },
    setChildLex: function setChildLex(idx, obj, lv) {
      var child = this.cl[idx];
      child.setObj(obj);
      child.setLex(lv);
    },
    setChildCtrl: function setChildCtrl(idx, ctrl) {
      this.cl[idx].setCtrl(ctrl);
    },
    merge: function merge(childFactor) {
      this.r = childFactor.rule();
      this.cl = childFactor.all();
    },
    replace: function replace(idx, factorList) {
      this.cl.splice(idx, 1);
      var len = factorList.length;

      for (var i = 0; i < len; ++i) {
        this.cl.splice(idx + i, 0, factorList[i]);
      }
    }
  });

  Expr.Tree = function () {
    this.stack = new _YIUICommon.Stack();
    this.root = null;
  };

  _YIUICommon.Lang.impl(Expr.Tree, {
    get: function get() {
      return this.root;
    },
    push: function push(factor) {
      this.stack.push(factor);
    },
    peek: function peek() {
      return this.stack.peek();
    },
    pop: function pop() {
      return this.stack.pop();
    },
    extract: function extract() {
      if (!this.stack.empty()) {
        this.root = this.stack.pop();
      }
    },
    opti1: function opti1(p) {
      var SL = p.item(2);
      p.replace(2, SL.all());
    },
    opti19: function opti19(p) {
      var FT = p.item(1);
      p.replace(1, FT.all());
      var FH = p.item(0);
      p.replace(0, FH.all());
    },
    opti22: function opti22(p) {
      var PL = p.item(0);
      p.replace(0, PL.all());
    },
    opti23: function opti23(p) {
      var PL = p.item(2);
      p.replace(2, PL.all());
    },
    opti26: function opti26(p) {
      var IH = p.item(0);
      p.replace(0, IH.all());
    },
    opti27: function opti27(p) {
      var IT = p.item(1);
      p.replace(1, IT.all());
      var IH = p.item(0);
      p.replace(0, IH.all());
    },
    opti36: function opti36(p) {
      var CASE_SL = p.item(5);
      p.replace(5, CASE_SL.all());
    },
    opti38: function opti38(p) {
      var CASE_SL = p.item(2);
      p.replace(2, CASE_SL.all());
    },
    optiItem: function optiItem(p) {
      var r = p.rule();

      if (r == null) {
        return;
      }

      var childList = p.all();

      if (childList != null) {
        var childFactor = null;
        var len = childList.length;

        for (var i = 0; i < len; ++i) {
          childFactor = childList[i];
          this.optiItem(childFactor);
        }
      } // 优化主要针对参数列表


      var ruleIndex = r.index();

      switch (ruleIndex) {
        case 1:
          this.opti1(p);
          break;

        case 19:
          this.opti19(p);
          break;

        case 22:
          this.opti22(p);
          break;

        case 24:
          this.opti23(p);
          break;

        case 26:
          this.opti26(p);
          break;

        case 27:
          this.opti27(p);
          break;

        case 36:
          this.opti36(p);
          break;

        case 38:
          this.opti38(p);
          break;
      }
    },
    opti: function opti() {
      this.optiItem(this.root);
    }
  });

  Expr.RuleTrace = function (r) {
    this.r = r;
    this.p = -1;
  };

  _YIUICommon.Lang.impl(Expr.RuleTrace, {
    rule: function rule() {
      return this.r;
    },
    pos: function pos() {
      return this.p;
    },
    matched: function matched() {
      return this.p == this.r.rightLen() - 1;
    },
    consum: function consum() {
      ++this.p;
      return this.p;
    },
    match: function match(li) {
      var rf = this.r.rightAt(this.pos() + 1);
      return rf.get() == li;
    },
    left: function left() {
      return this.r.left().get();
    },
    isClose: function isClose() {
      return this.r.isClose();
    },
    print: function print() {
      var left = this.r.left();
      var leftString = left.getName();
      var rightString = null;
      var vRight = this.r.right();
      var length = vRight.length;

      for (var i = 0; i < length; ++i) {
        if (rightString == null) {
          rightString = vRight[i].getName();
        } else {
          rightString += " " + vRight[i].getName();
        }

        if (this.p == i) {
          rightString += " ^ ";
        }
      }

      var result = null;

      if (this.p == -1) {
        result = leftString + " -> ^ " + rightString;
      } else {
        result = leftString + " -> " + rightString;
      } //console.log(result);

    }
  });

  Expr.IEvalContext = function () {};
  /**
   * EvalEnv函数成员说明
   * 1.getValue(cxt, id)，函数的目的是为了取得id所表示的值
   *    参数：
   *    cxt 上下文
   *    id 标识符
   *    返回值:
   *    返回id所表示的标识值的值，在客户端为id对应组件(或者单元格)的值
   * 2.setValue(cxt, id, value)，函数的目的是为了给标识符id赋值
   *    参数：
   *    cxt 上下文
   *    id 标识符
   *    value 值
   *    返回值：无
   * 3.checkMacro(cxt, name),函数的目的是为了检查name是否是宏公式
   *    参数：
   *    cxt 上下文
   *    name 名称
   *    返回值：
   *    如果name定义为宏，返回宏定义，否则返回null作为判断是否为宏的依据
   * 4.evalMacro(cxt, scope, name, macro, args)，解析宏公式
   *    参数：
   *    cxt 上下文
   *    scope 作用域
   *    name 宏名称
   *    macro 宏定义
   *    args 参数集
   *    返回值：宏公式的返回值
   * 5.getLoop(cxt, name, obj)，根据函数名称和对象取得循环对象
   *    参数：
   *    cxt 上下文
   *    name 函数名称
   *    obj 循环对象标识
   *    返回值：IObjectLoop接口的循环对象
   * 6.resolveObject(self, scope, obj)，解析对象上下文
   *    参数：
   *    self 当前上下文
   *    scope 作用域
   *    obj 对象标识
   *    返回值：根据obj返回的新的上下文
   * 7.evalObject(cxt, obj, name, args)，执行对象操作，在js上不能使用
   *    参数：
   *    cxt 上下文
   *    obj 对象标识
   *    name 对象名称
   *    args 参数列表
   *    返回值：执行结果
   **/


  Expr.IEvalEnv = function () {};
  /**
   * IObjectLoop成员函数说明：
   * 1.hasNext(),返回是否还有下一个对象
   * 2.next(),移到下一个对象
   * 3.clean(),清理环境
   */


  Expr.IObjectLoop = function () {};

  Expr.InnerFuns = function () {
    var funs = {};

    funs.InList = function (name, cxt, args) {
      var r = false;
      var v = args[0];
      var isStr = typeof v == 'string';
      var l = args.length;

      for (var i = 1; i < l; ++i) {
        var tv = args[i];

        if (isStr) {
          if (v == tv.toString()) {
            r = true;
            break;
          }
        } else {
          if (v instanceof _decimal["default"] && tv instanceof _decimal["default"]) {
            if (v.equals(tv)) {
              r = true;
              break;
            }
          } else if (v == tv) {
            r = true;
            break;
          }
        }
      }

      return r;
    };

    funs.Left = function (name, cxt, args) {
      var s = args[0];
      var l = args[1];
      var r = s.substring(0, l);
      return r;
    };

    funs.Right = function (name, cxt, args) {
      var s = args[0];
      var l = args[1];
      var r = s.substring(s.length - l);
      return r;
    };

    funs.Mid = function (name, cxt, args) {
      var s = args[0];
      var p = args[1];
      var l = args[2];
      var r = s.substring(p, p + l);
      return r;
    };

    funs.Length = function (name, cxt, args) {
      var s = args[0] || "";
      return s.length;
    };

    funs.IndexOf = function (name, cxt, args) {
      var fs = args[0];
      var ss = args[1];
      return fs.indexOf(ss);
    };

    funs.ToInt = function (name, cxt, args) {
      var v = args[0];

      var r = _YIUIBase.YIUI.TypeConvertor.toInt(v == null ? 0 : v);

      return r;
    };

    funs.ToDecimal = function (name, cxt, args) {
      var v = _YIUIBase.YIUI.TypeConvertor.toDecimal(args[0]);

      var r = new _decimal["default"](v == null ? 0 : v);
      return r;
    };

    funs.ToLong = function (name, cxt, args) {
      var v = args[0];

      var r = _YIUIBase.YIUI.TypeConvertor.toLong(v == null ? 0 : v);

      return r;
    };

    funs.ToString = function (name, cxt, args) {
      var v = args[0];

      var r = _YIUIBase.YIUI.TypeConvertor.toString(v);

      return r;
    };

    funs.IIF = function (name, cxt, args) {
      var cv = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);

      var v = null;

      if (cv) {
        v = args[1];
      } else if (args.length > 2) {
        v = args[2];
      }

      return v;
    };

    funs.IIFS = function (name, cxt, args) {
      var v = null;
      var l = args.length;

      var cv = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);

      for (var i = 0; i < l; ++i) {
        if (i % 2 == 0) {
          cv = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[i]);
        } else if (cv) {
          v = args[i];
          break;
        }
      }

      return v;
    };

    funs.OUT = function (name, cxt, args) {
      if (window.console && args[0] && args[0] !== null) {
        console.log(args[0].toString());
      }
    };

    funs.IsNull = function (name, cxt, args) {
      return args[0] == null || args[0] == undefined;
    };

    funs.ToUpper = function (name, cxt, args) {
      var value = args[0];
      var result = value.toUpperCase();
      return result;
    };

    funs.ToLower = function (name, cxt, args) {
      var value = args[0];
      var result = value.toLowerCase();
      return result;
    };

    funs.Trim = function (name, cxt, args) {
      var value = args[0];
      var result = value.trim();
      return result;
    };

    funs.ToBool = function (name, cxt, args) {
      return _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);
    };

    funs.Format = function (name, cxt, args) {
      var form = cxt.form;
      var value = args[0];
      var format = null;

      if (args.length > 1) {
        format = args[1];
      }

      var s = "",
          date = null;

      if (value != null) {
        if (value instanceof _decimal["default"]) {//TODO 如何格式化
        } else if (_yesCommon.lodash.isNumeric(value) || typeof value == "boolean") {
          s = value.toString();
        } else if (value instanceof Date) {
          date = value;

          if (format == null) {
            s = date.Format("yyyy-MM-dd HH:mm:ss");
          } else {
            s = date.Format(format);
          }
        }
      }

      return s;
    };

    funs.DateAdd = function (name, cxt, args) {
      var interval = args[1].toString().toLowerCase(),
          number = parseInt(args[2]),
          date = args[0];
      date = new Date(date);

      switch (interval) {
        case "d":
        case "dd":
          date.setDate(date.getDate() + number);
          break;

        case "m":
        case "mm":
          date.setMonth(date.getMonth() + number);
          break;

        case "yyyy":
          date.setYear(date.getFullYear() + number);
          break;

        case "h":
          date.setHours(date.getHours() + number);
          break;

        case "n":
          date.setMinutes(date.getMinutes() + number);
          break;

        case "s":
          date.setSeconds(date.getSeconds() + number);
          break;

        case "q":
          date.setMonth(date.getMonth() + number * 3);
          break;

        case "ww":
          date.setDate(date.getDate() + number * 7);
          break;

        default:
          alert("DateAdd " + interval + " not impl");
          break;
      }

      return date;
    };

    funs.DaysOfMonth = function (name, cxt, args) {
      var date;

      if (args.length == 0 || args[0] == null || args[0] == undefined || args[0].toString().length == 0) {
        date = new Date();
      } else {
        date = new Date(args[0].toString());
      }

      return new Date(date.getFullYear(), date.getMonth() + 1, 0).getDate();
    };

    funs.FirstDayOfMonth = function (name, cxt, args) {
      var date;

      if (args.length == 0 || args[0] == null || args[0] == undefined || args[0].toString().length == 0) {
        date = new Date();
      } else {
        date = new Date(args[0].toString());
      }

      return date.setDate(1);
    };

    funs.LastDayOfMonth = function (name, cxt, args) {
      var date;

      if (args.length == 0 || args[0] == null || args[0] == undefined || args[0].toString().length == 0) {
        date = new Date();
      } else {
        date = new Date(args[0].toString());
      }

      date.setMonth(date.getMonth() + 1);
      date.setDate(0);
      return date;
    };

    funs.DateDiff = function (name, cxt, args) {
      var startDate = _YIUIBase.YIUI.TypeConvertor.toDate(args[0]),
          endDate = _YIUIBase.YIUI.TypeConvertor.toDate(args[1]),
          unit = args[2].toLowerCase(),
          value = -1;

      if (!unit || !startDate || !endDate) {
        _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.Date_Diff_Param_Error);
      }

      switch (unit) {
        case "yyyy":
          value = endDate.getFullYear() - startDate.getFullYear();
          break;

        case "m":
          var date1 = startDate.getFullYear() * 12 + startDate.getMonth();
          var date2 = endDate.getFullYear() * 12 + endDate.getMonth();
          value = date2 - date1;
          break;

        case "d":
          value = (endDate - startDate) / (24 * 60 * 60 * 1000);
          break;

        case "h":
          value = (endDate - startDate) / (60 * 60 * 1000);
          break;

        case "n":
          value = (endDate - startDate) / (60 * 1000);
          break;

        case "s":
          value = (endDate - startDate) / 1000;
          break;
      }

      return Math.floor(value);
    };

    funs.DayOfWeek = function (name, cxt, args) {
      var date = new Date(args[0]);
      return date.getDay() + 1;
    };

    funs.ToDate = function (name, cxt, args) {
      var s = args[0];
      s = s.replace(/-/g, "/");
      var format = "yyyy-MM-dd";

      if (args.length > 1) {
        format = args[1];
      }

      return new Date(s);
    };

    funs.Replace = function (name, cxt, args) {
      var string = args[0].toString(),
          placeholder = args[1].toString(),
          replacement = args[2].toString();
      return string.replace(placeholder, replacement);
    };

    funs.Round = function (name, cxt, args) {
      var bd = _YIUIBase.YIUI.TypeConvertor.toDecimal(args[0]),
          dp = args[1] == undefined ? 0 : parseInt(args[1]),
          rm = _YIUIBase.YIUI.NUMBEREDITOR_ROUNDINGMODE.parseStr(args[2]);

      return bd.toDecimalPlaces(dp, rm);
    };

    funs.GetAmountInWords = function (name, cxt, args) {
      var bd = _YIUIBase.YIUI.TypeConvertor.toDecimal(args[0]);

      _YIUIBase.YIUI.NumericUtil.getAmountInWords(bd);

      return;
    };

    funs.ReplaceByPos = function (name, cxt, args) {
      var format = args[0];
      return _YIUIBase.YIUI.UIUtil.format(format, args, 1);
    };

    funs.RaiseErr = function (name, cxt, args) {
      var info = args[0]; // $.error(info);

      throw new _exceptions.YESException('', '', info);
    };

    return funs;
  }();

  Expr.Evaluator = function (implMap, evalEnv, tree, scope) {
    this.implMap = implMap;
    this.evalEnv = evalEnv;
    this.tree = tree;
    this.scope = scope;
    this.ctrlFlow = -1;
  };

  _YIUICommon.Lang.impl(Expr.Evaluator, {
    eval: function () {
      var _eval2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(cxt) {
        var v;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                v = null;
                _context.next = 3;
                return this.exec(cxt, this.tree.get());

              case 3:
                v = this.tree.get().get();
                return _context.abrupt("return", v);

              case 5:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function _eval(_x) {
        return _eval2.apply(this, arguments);
      }

      return _eval;
    }(),
    exec: function () {
      var _exec = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2(cxt, p) {
        var r, rIdx;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                r = p.rule();
                rIdx = -1;

                if (!(r != null)) {
                  _context2.next = 127;
                  break;
                }

                rIdx = r.index();
                _context2.t0 = rIdx;
                _context2.next = _context2.t0 === 0 ? 7 : _context2.t0 === 1 ? 10 : _context2.t0 === 2 ? 13 : _context2.t0 === 3 ? 16 : _context2.t0 === 4 ? 19 : _context2.t0 === 5 ? 22 : _context2.t0 === 6 ? 25 : _context2.t0 === 7 ? 28 : _context2.t0 === 8 ? 31 : _context2.t0 === 9 ? 34 : _context2.t0 === 10 ? 37 : _context2.t0 === 11 ? 40 : _context2.t0 === 12 ? 43 : _context2.t0 === 13 ? 46 : _context2.t0 === 14 ? 49 : _context2.t0 === 15 ? 52 : _context2.t0 === 16 ? 55 : _context2.t0 === 17 ? 58 : _context2.t0 === 18 ? 61 : _context2.t0 === 19 ? 64 : _context2.t0 === 20 ? 67 : _context2.t0 === 21 ? 70 : _context2.t0 === 22 ? 73 : _context2.t0 === 23 ? 76 : _context2.t0 === 24 ? 79 : _context2.t0 === 25 ? 82 : _context2.t0 === 26 ? 85 : _context2.t0 === 27 ? 88 : _context2.t0 === 28 ? 91 : _context2.t0 === 29 ? 94 : _context2.t0 === 30 ? 97 : _context2.t0 === 31 ? 100 : _context2.t0 === 32 ? 103 : _context2.t0 === 33 ? 106 : _context2.t0 === 34 ? 109 : _context2.t0 === 35 ? 112 : _context2.t0 === 36 ? 115 : _context2.t0 === 37 ? 118 : _context2.t0 === 38 ? 121 : _context2.t0 === 39 ? 124 : 127;
                break;

              case 7:
                _context2.next = 9;
                return this.exec0(cxt, p);

              case 9:
                return _context2.abrupt("break", 127);

              case 10:
                _context2.next = 12;
                return this.exec1(cxt, p);

              case 12:
                return _context2.abrupt("break", 127);

              case 13:
                _context2.next = 15;
                return this.exec2(cxt, p);

              case 15:
                return _context2.abrupt("break", 127);

              case 16:
                _context2.next = 18;
                return this.exec3(cxt, p);

              case 18:
                return _context2.abrupt("break", 127);

              case 19:
                _context2.next = 21;
                return this.exec4(cxt, p);

              case 21:
                return _context2.abrupt("break", 127);

              case 22:
                _context2.next = 24;
                return this.exec5(cxt, p);

              case 24:
                return _context2.abrupt("break", 127);

              case 25:
                _context2.next = 27;
                return this.exec6(cxt, p);

              case 27:
                return _context2.abrupt("break", 127);

              case 28:
                _context2.next = 30;
                return this.exec7(cxt, p);

              case 30:
                return _context2.abrupt("break", 127);

              case 31:
                _context2.next = 33;
                return this.exec8(cxt, p);

              case 33:
                return _context2.abrupt("break", 127);

              case 34:
                _context2.next = 36;
                return this.exec9(cxt, p);

              case 36:
                return _context2.abrupt("break", 127);

              case 37:
                _context2.next = 39;
                return this.exec10(cxt, p);

              case 39:
                return _context2.abrupt("break", 127);

              case 40:
                _context2.next = 42;
                return this.exec11(cxt, p);

              case 42:
                return _context2.abrupt("break", 127);

              case 43:
                _context2.next = 45;
                return this.exec12(cxt, p);

              case 45:
                return _context2.abrupt("break", 127);

              case 46:
                _context2.next = 48;
                return this.exec13(cxt, p);

              case 48:
                return _context2.abrupt("break", 127);

              case 49:
                _context2.next = 51;
                return this.exec14(cxt, p);

              case 51:
                return _context2.abrupt("break", 127);

              case 52:
                _context2.next = 54;
                return this.exec15(cxt, p);

              case 54:
                return _context2.abrupt("break", 127);

              case 55:
                _context2.next = 57;
                return this.exec16(cxt, p);

              case 57:
                return _context2.abrupt("break", 127);

              case 58:
                _context2.next = 60;
                return this.exec17(cxt, p);

              case 60:
                return _context2.abrupt("break", 127);

              case 61:
                _context2.next = 63;
                return this.exec18(cxt, p);

              case 63:
                return _context2.abrupt("break", 127);

              case 64:
                _context2.next = 66;
                return this.exec19(cxt, p);

              case 66:
                return _context2.abrupt("break", 127);

              case 67:
                _context2.next = 69;
                return this.exec20(cxt, p);

              case 69:
                return _context2.abrupt("break", 127);

              case 70:
                _context2.next = 72;
                return this.exec21(cxt, p);

              case 72:
                return _context2.abrupt("break", 127);

              case 73:
                _context2.next = 75;
                return this.exec22(cxt, p);

              case 75:
                return _context2.abrupt("break", 127);

              case 76:
                _context2.next = 78;
                return this.exec23(cxt, p);

              case 78:
                return _context2.abrupt("break", 127);

              case 79:
                _context2.next = 81;
                return this.exec24(cxt, p);

              case 81:
                return _context2.abrupt("break", 127);

              case 82:
                _context2.next = 84;
                return this.exec25(cxt, p);

              case 84:
                return _context2.abrupt("break", 127);

              case 85:
                _context2.next = 87;
                return this.exec26(cxt, p);

              case 87:
                return _context2.abrupt("break", 127);

              case 88:
                _context2.next = 90;
                return this.exec27(cxt, p);

              case 90:
                return _context2.abrupt("break", 127);

              case 91:
                _context2.next = 93;
                return this.exec28(cxt, p);

              case 93:
                return _context2.abrupt("break", 127);

              case 94:
                _context2.next = 96;
                return this.exec29(cxt, p);

              case 96:
                return _context2.abrupt("break", 127);

              case 97:
                _context2.next = 99;
                return this.exec30(cxt, p);

              case 99:
                return _context2.abrupt("break", 127);

              case 100:
                _context2.next = 102;
                return this.exec31(cxt, p);

              case 102:
                return _context2.abrupt("break", 127);

              case 103:
                _context2.next = 105;
                return this.exec32(cxt, p);

              case 105:
                return _context2.abrupt("break", 127);

              case 106:
                _context2.next = 108;
                return this.exec33(cxt, p);

              case 108:
                return _context2.abrupt("break", 127);

              case 109:
                _context2.next = 111;
                return this.exec34(cxt, p);

              case 111:
                return _context2.abrupt("break", 127);

              case 112:
                _context2.next = 114;
                return this.exec35(cxt, p);

              case 114:
                return _context2.abrupt("break", 127);

              case 115:
                _context2.next = 117;
                return this.exec36(cxt, p);

              case 117:
                return _context2.abrupt("break", 127);

              case 118:
                _context2.next = 120;
                return this.exec37(cxt, p);

              case 120:
                return _context2.abrupt("break", 127);

              case 121:
                _context2.next = 123;
                return this.exec38(cxt, p);

              case 123:
                return _context2.abrupt("break", 127);

              case 124:
                _context2.next = 126;
                return this.exec39(cxt, p);

              case 126:
                return _context2.abrupt("break", 127);

              case 127:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function exec(_x2, _x3) {
        return _exec.apply(this, arguments);
      }

      return exec;
    }(),
    isNull: function isNull(v) {
      return v == undefined || v == null;
    },
    getCpType: function getCpType(v1, v2) {
      var type = -1;

      if (this.isNull(v1)) {
        var temp = v2;
        v2 = v1;
        v1 = temp;
      }

      if (this.isNull(v1)) {
        type = _YIUICommon.DataType.NUMERIC;
      } else if (typeof v1 == "string") {
        if (_yesCommon.lodash.isNumeric(v1)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else {
          type = _YIUICommon.DataType.STRING;
        }
      } else if (typeof v1 == "number") {
        if (this.isNull(v2)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "string") {
          type = _YIUICommon.DataType.STRING;
        } else {
          type = _YIUICommon.DataType.NUMERIC;
        }
      } else if (typeof v1 == "boolean") {
        if (this.isNull(v2)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "number" || v2 instanceof _decimal["default"]) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "string") {
          type = _YIUICommon.DataType.STRING;
        } else if (typeof v2 == "boolean") {
          type = _YIUICommon.DataType.BOOLEAN;
        }
      } else if (v1 instanceof _decimal["default"]) {
        if (this.isNull(v2)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "number" || v2 instanceof _decimal["default"]) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "string") {
          type = _YIUICommon.DataType.STRING;
        } else if (typeof v2 == "boolean") {
          type = _YIUICommon.DataType.BOOLEAN;
        }
      }

      return type;
    },
    // SL -> E
    exec0: function () {
      var _exec2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee3(cxt, p) {
        var E;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                E = p.item(0);
                _context3.next = 3;
                return this.exec(cxt, E);

              case 3:
                if (this.ctrlFlow != Expr.F_RETURN) {
                  p.set(E.get());
                }

              case 4:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function exec0(_x4, _x5) {
        return _exec2.apply(this, arguments);
      }

      return exec0;
    }(),
    // SL -> E ; SL
    exec1: function () {
      var _exec3 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee4(cxt, p) {
        var l, v, i, E;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                l = (p.size() + 1) / 2;
                v = null;
                i = 0;

              case 3:
                if (!(i < l)) {
                  _context4.next = 13;
                  break;
                }

                E = p.item(i * 2);
                _context4.next = 7;
                return this.exec(cxt, E);

              case 7:
                if (!(this.ctrlFlow != -1)) {
                  _context4.next = 9;
                  break;
                }

                return _context4.abrupt("break", 13);

              case 9:
                v = E.get();

              case 10:
                ++i;
                _context4.next = 3;
                break;

              case 13:
                if (this.ctrlFlow != Expr.F_RETURN) {
                  p.set(v);
                }

              case 14:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function exec1(_x6, _x7) {
        return _exec3.apply(this, arguments);
      }

      return exec1;
    }(),
    // E -> E + E
    exec2: function () {
      var _exec4 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5(cxt, p) {
        var E1, E2, v1, v2, type, v;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context5.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context5.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = E1.get(), v2 = E2.get(), type = this.getCpType(v1, v2);
                _context5.t0 = type;
                _context5.next = _context5.t0 === _YIUICommon.DataType.STRING ? 10 : _context5.t0 === _YIUICommon.DataType.NUMERIC ? 12 : 14;
                break;

              case 10:
                v = _YIUIBase.YIUI.TypeConvertor.toString(v1) + _YIUIBase.YIUI.TypeConvertor.toString(v2);
                return _context5.abrupt("break", 14);

              case 12:
                v = _YIUIBase.YIUI.TypeConvertor.toDecimal(v1).plus(_YIUIBase.YIUI.TypeConvertor.toDecimal(v2));
                return _context5.abrupt("break", 14);

              case 14:
                p.set(v);

              case 15:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function exec2(_x8, _x9) {
        return _exec4.apply(this, arguments);
      }

      return exec2;
    }(),
    // E -> E - E
    exec3: function () {
      var _exec5 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6(cxt, p) {
        var E1, E2, v1, v2;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context6.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context6.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = _YIUIBase.YIUI.TypeConvertor.toDecimal(E1.get()), v2 = _YIUIBase.YIUI.TypeConvertor.toDecimal(E2.get());
                p.set(v1.minus(v2));

              case 8:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function exec3(_x10, _x11) {
        return _exec5.apply(this, arguments);
      }

      return exec3;
    }(),
    // E -> E * E
    exec4: function () {
      var _exec6 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee7(cxt, p) {
        var E1, E2, v1, v2;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context7.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context7.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = _YIUIBase.YIUI.TypeConvertor.toDecimal(E1.get()), v2 = _YIUIBase.YIUI.TypeConvertor.toDecimal(E2.get());
                p.set(v1.times(v2));

              case 8:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function exec4(_x12, _x13) {
        return _exec6.apply(this, arguments);
      }

      return exec4;
    }(),
    // E -> E / E
    exec5: function () {
      var _exec7 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee8(cxt, p) {
        var E1, E2, v1, v2;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context8.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context8.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = _YIUIBase.YIUI.TypeConvertor.toDecimal(E1.get()), v2 = _YIUIBase.YIUI.TypeConvertor.toDecimal(E2.get());
                p.set(v1.dividedBy(v2));

              case 8:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function exec5(_x14, _x15) {
        return _exec7.apply(this, arguments);
      }

      return exec5;
    }(),
    // E -> E & E
    exec6: function () {
      var _exec8 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee9(cxt, p) {
        var E1, E2, v1, v2;
        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context9.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context9.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = _YIUIBase.YIUI.TypeConvertor.toString(E1.get()), v2 = _YIUIBase.YIUI.TypeConvertor.toString(E2.get()); // console.log(`aaaa,${v1},${v2}`);

                p.set(v1 + v2);

              case 8:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function exec6(_x16, _x17) {
        return _exec8.apply(this, arguments);
      }

      return exec6;
    }(),
    // E -> ( E )
    exec7: function () {
      var _exec9 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee10(cxt, p) {
        var E;
        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                E = p.item(1);
                _context10.next = 3;
                return this.exec(cxt, E);

              case 3:
                p.set(E.get());

              case 4:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      function exec7(_x18, _x19) {
        return _exec9.apply(this, arguments);
      }

      return exec7;
    }(),
    // E -> ! E
    exec8: function () {
      var _exec10 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee11(cxt, p) {
        var E, value;
        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                E = p.item(1);
                _context11.next = 3;
                return this.exec(cxt, E);

              case 3:
                value = _YIUIBase.YIUI.TypeConvertor.toBoolean(E.get());
                p.set(!value);

              case 5:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));

      function exec8(_x20, _x21) {
        return _exec10.apply(this, arguments);
      }

      return exec8;
    }(),
    // E -> E || E
    exec9: function () {
      var _exec11 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee12(cxt, p) {
        var E1, b1, E2, b2;
        return regeneratorRuntime.wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                E1 = p.item(0);
                _context12.next = 3;
                return this.exec(cxt, E1);

              case 3:
                b1 = _YIUIBase.YIUI.TypeConvertor.toBoolean(E1.get());

                if (!b1) {
                  _context12.next = 8;
                  break;
                }

                p.set(true);
                _context12.next = 13;
                break;

              case 8:
                E2 = p.item(2);
                _context12.next = 11;
                return this.exec(cxt, E2);

              case 11:
                b2 = _YIUIBase.YIUI.TypeConvertor.toBoolean(E2.get());
                p.set(b2);

              case 13:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this);
      }));

      function exec9(_x22, _x23) {
        return _exec11.apply(this, arguments);
      }

      return exec9;
    }(),
    // E -> E && E
    exec10: function () {
      var _exec12 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee13(cxt, p) {
        var E1, b1, E2, b2;
        return regeneratorRuntime.wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                E1 = p.item(0);
                _context13.next = 3;
                return this.exec(cxt, E1);

              case 3:
                b1 = _YIUIBase.YIUI.TypeConvertor.toBoolean(E1.get());

                if (b1) {
                  _context13.next = 8;
                  break;
                }

                p.set(false);
                _context13.next = 13;
                break;

              case 8:
                E2 = p.item(2);
                _context13.next = 11;
                return this.exec(cxt, E2);

              case 11:
                b2 = _YIUIBase.YIUI.TypeConvertor.toBoolean(E2.get());
                p.set(b2);

              case 13:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      function exec10(_x24, _x25) {
        return _exec12.apply(this, arguments);
      }

      return exec10;
    }(),
    // E -> E == E
    exec11: function () {
      var _exec13 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee14(cxt, p) {
        var E1, E2, v1, v2, type, v, s1, s2;
        return regeneratorRuntime.wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context14.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context14.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = E1.get(), v2 = E2.get(), type = this.getCpType(v1, v2);
                _context14.t0 = type;
                _context14.next = _context14.t0 === _YIUICommon.DataType.NUMERIC ? 10 : _context14.t0 === _YIUICommon.DataType.STRING ? 12 : 16;
                break;

              case 10:
                if (!_yesCommon.lodash.isNumeric(v2)) {
                  v = false;
                } else {
                  v = _YIUIBase.YIUI.TypeConvertor.toDecimal(v1).equals(_YIUIBase.YIUI.TypeConvertor.toDecimal(v2));
                }

                return _context14.abrupt("break", 18);

              case 12:
                s1 = _YIUIBase.YIUI.TypeConvertor.toString(v1);
                s2 = _YIUIBase.YIUI.TypeConvertor.toString(v2);
                v = Expr.eq(s1, s2);
                return _context14.abrupt("break", 18);

              case 16:
                v = v1 == v2;
                return _context14.abrupt("break", 18);

              case 18:
                p.set(v);

              case 19:
              case "end":
                return _context14.stop();
            }
          }
        }, _callee14, this);
      }));

      function exec11(_x26, _x27) {
        return _exec13.apply(this, arguments);
      }

      return exec11;
    }(),
    // E -> E <> E
    exec12: function () {
      var _exec14 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee15(cxt, p) {
        var E1, E2, v1, v2, type, v, s1, s2;
        return regeneratorRuntime.wrap(function _callee15$(_context15) {
          while (1) {
            switch (_context15.prev = _context15.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context15.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context15.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = E1.get(), v2 = E2.get(), type = this.getCpType(v1, v2);
                _context15.t0 = type;
                _context15.next = _context15.t0 === _YIUICommon.DataType.NUMERIC ? 10 : _context15.t0 === _YIUICommon.DataType.STRING ? 12 : 16;
                break;

              case 10:
                v = !_YIUIBase.YIUI.TypeConvertor.toDecimal(v1).equals(_YIUIBase.YIUI.TypeConvertor.toDecimal(v2));
                return _context15.abrupt("break", 18);

              case 12:
                s1 = _YIUIBase.YIUI.TypeConvertor.toString(v1);
                s2 = _YIUIBase.YIUI.TypeConvertor.toString(v2);
                v = !Expr.eq(s1, s2);
                return _context15.abrupt("break", 18);

              case 16:
                v = v1 != v2;
                return _context15.abrupt("break", 18);

              case 18:
                p.set(v);

              case 19:
              case "end":
                return _context15.stop();
            }
          }
        }, _callee15, this);
      }));

      function exec12(_x28, _x29) {
        return _exec14.apply(this, arguments);
      }

      return exec12;
    }(),
    // E -> E > E
    exec13: function () {
      var _exec15 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee16(cxt, p) {
        var E1, E2, v1, v2, type, v, s1, s2, d1, d2;
        return regeneratorRuntime.wrap(function _callee16$(_context16) {
          while (1) {
            switch (_context16.prev = _context16.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context16.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context16.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = E1.get(), v2 = E2.get(), type = this.getCpType(v1, v2);
                _context16.t0 = type;
                _context16.next = _context16.t0 === _YIUICommon.DataType.NUMERIC ? 10 : _context16.t0 === _YIUICommon.DataType.STRING ? 12 : _context16.t0 === _YIUICommon.DataType.DATETIME ? 16 : 20;
                break;

              case 10:
                v = _YIUIBase.YIUI.TypeConvertor.toDecimal(v1).greaterThan(_YIUIBase.YIUI.TypeConvertor.toDecimal(v2));
                return _context16.abrupt("break", 22);

              case 12:
                s1 = _YIUIBase.YIUI.TypeConvertor.toString(v1).toLowerCase();
                s2 = _YIUIBase.YIUI.TypeConvertor.toString(v2).toLowerCase();
                v = s1.localeCompare(s2) == 1;
                return _context16.abrupt("break", 22);

              case 16:
                d1 = _YIUIBase.YIUI.TypeConvertor.toDate(v1);
                d2 = _YIUIBase.YIUI.TypeConvertor.toDate(v2);

                if (d1 == null) {
                  v = false;
                } else if (d2 == null) {
                  v = true;
                } else {
                  v = d1.getTime() > d2.getTime();
                }

                return _context16.abrupt("break", 22);

              case 20:
                v = v1 > v2;
                return _context16.abrupt("break", 22);

              case 22:
                p.set(v);

              case 23:
              case "end":
                return _context16.stop();
            }
          }
        }, _callee16, this);
      }));

      function exec13(_x30, _x31) {
        return _exec15.apply(this, arguments);
      }

      return exec13;
    }(),
    // E -> E >= E
    exec14: function () {
      var _exec16 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee17(cxt, p) {
        var E1, E2, v1, v2, type, v, s1, s2, d1, d2;
        return regeneratorRuntime.wrap(function _callee17$(_context17) {
          while (1) {
            switch (_context17.prev = _context17.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context17.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context17.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = E1.get(), v2 = E2.get(), type = this.getCpType(v1, v2);
                _context17.t0 = type;
                _context17.next = _context17.t0 === _YIUICommon.DataType.NUMERIC ? 10 : _context17.t0 === _YIUICommon.DataType.STRING ? 12 : _context17.t0 === _YIUICommon.DataType.DATETIME ? 16 : 20;
                break;

              case 10:
                v = _YIUIBase.YIUI.TypeConvertor.toDecimal(v1).greaterThanOrEqualTo(_YIUIBase.YIUI.TypeConvertor.toDecimal(v2));
                return _context17.abrupt("break", 22);

              case 12:
                s1 = _YIUIBase.YIUI.TypeConvertor.toString(v1).toLowerCase();
                s2 = _YIUIBase.YIUI.TypeConvertor.toString(v2).toLowerCase();
                v = s1.localeCompare(s2) == 0 || s1.localeCompare(s2) == 1;
                return _context17.abrupt("break", 22);

              case 16:
                d1 = _YIUIBase.YIUI.TypeConvertor.toDate(v1);
                d2 = _YIUIBase.YIUI.TypeConvertor.toDate(v2);

                if (d1 == null) {
                  v = d2 == null ? true : false;
                } else if (d2 == null) {
                  v = true;
                } else {
                  v = d1.getTime() >= d2.getTime();
                }

                return _context17.abrupt("break", 22);

              case 20:
                v = v1 >= v2;
                return _context17.abrupt("break", 22);

              case 22:
                p.set(v);

              case 23:
              case "end":
                return _context17.stop();
            }
          }
        }, _callee17, this);
      }));

      function exec14(_x32, _x33) {
        return _exec16.apply(this, arguments);
      }

      return exec14;
    }(),
    // E -> E < E
    exec15: function () {
      var _exec17 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee18(cxt, p) {
        var E1, E2, v1, v2, type, v, s1, s2, d1, d2;
        return regeneratorRuntime.wrap(function _callee18$(_context18) {
          while (1) {
            switch (_context18.prev = _context18.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context18.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context18.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = E1.get(), v2 = E2.get(), type = this.getCpType(v1, v2);
                _context18.t0 = type;
                _context18.next = _context18.t0 === _YIUICommon.DataType.NUMERIC ? 10 : _context18.t0 === _YIUICommon.DataType.STRING ? 12 : _context18.t0 === _YIUICommon.DataType.DATETIME ? 16 : 20;
                break;

              case 10:
                v = _YIUIBase.YIUI.TypeConvertor.toDecimal(v1).lessThan(_YIUIBase.YIUI.TypeConvertor.toDecimal(v2));
                return _context18.abrupt("break", 22);

              case 12:
                s1 = _YIUIBase.YIUI.TypeConvertor.toString(v1).toLowerCase();
                s2 = _YIUIBase.YIUI.TypeConvertor.toString(v2).toLowerCase();
                v = s1.localeCompare(s2) == -1;
                return _context18.abrupt("break", 22);

              case 16:
                d1 = _YIUIBase.YIUI.TypeConvertor.toDate(v1);
                d2 = _YIUIBase.YIUI.TypeConvertor.toDate(v2);

                if (d1 == null) {
                  v == (d2 == null ? false : true);
                } else if (d2 == null) {
                  v = false;
                } else {
                  v = d1.getTime() < d2.getTime();
                }

                return _context18.abrupt("break", 22);

              case 20:
                v = v1 < v2;
                return _context18.abrupt("break", 22);

              case 22:
                p.set(v);

              case 23:
              case "end":
                return _context18.stop();
            }
          }
        }, _callee18, this);
      }));

      function exec15(_x34, _x35) {
        return _exec17.apply(this, arguments);
      }

      return exec15;
    }(),
    // E -> E <= E
    exec16: function () {
      var _exec18 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee19(cxt, p) {
        var E1, E2, v1, v2, type, v, s1, s2, d1, d2;
        return regeneratorRuntime.wrap(function _callee19$(_context19) {
          while (1) {
            switch (_context19.prev = _context19.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                _context19.next = 4;
                return this.exec(cxt, E1);

              case 4:
                _context19.next = 6;
                return this.exec(cxt, E2);

              case 6:
                v1 = E1.get(), v2 = E2.get(), type = this.getCpType(v1, v2);
                _context19.t0 = type;
                _context19.next = _context19.t0 === _YIUICommon.DataType.NUMERIC ? 10 : _context19.t0 === _YIUICommon.DataType.STRING ? 12 : _context19.t0 === _YIUICommon.DataType.DATETIME ? 16 : 20;
                break;

              case 10:
                v = _YIUIBase.YIUI.TypeConvertor.toDecimal(v1).lessThanOrEqualTo(_YIUIBase.YIUI.TypeConvertor.toDecimal(v2));
                return _context19.abrupt("break", 22);

              case 12:
                s1 = _YIUIBase.YIUI.TypeConvertor.toString(v1).toLowerCase();
                s2 = _YIUIBase.YIUI.TypeConvertor.toString(v2).toLowerCase();
                v = s1.localeCompare(s2) == 0 || s1.localeCompare(s2) == -1;
                return _context19.abrupt("break", 22);

              case 16:
                d1 = _YIUIBase.YIUI.TypeConvertor.toDate(v1);
                d2 = _YIUIBase.YIUI.TypeConvertor.toDate(v2);

                if (d1 == null) {
                  v = true;
                } else if (d2 == null) {
                  v = false;
                } else {
                  v = d1.getTime() <= d2.getTime();
                }

                return _context19.abrupt("break", 22);

              case 20:
                v = v1 <= v2;
                return _context19.abrupt("break", 22);

              case 22:
                p.set(v);

              case 23:
              case "end":
                return _context19.stop();
            }
          }
        }, _callee19, this);
      }));

      function exec16(_x36, _x37) {
        return _exec18.apply(this, arguments);
      }

      return exec16;
    }(),
    // E -> const
    exec17: function () {
      var _exec19 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee20(cxt, p) {
        var cf;
        return regeneratorRuntime.wrap(function _callee20$(_context20) {
          while (1) {
            switch (_context20.prev = _context20.next) {
              case 0:
                cf = p.item(0);
                p.set(cf.get());

              case 2:
              case "end":
                return _context20.stop();
            }
          }
        }, _callee20);
      }));

      function exec17(_x38, _x39) {
        return _exec19.apply(this, arguments);
      }

      return exec17;
    }(),
    // E -> id
    exec18: function () {
      var _exec20 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee21(cxt, p) {
        var idf, id, obj, found, value, heap;
        return regeneratorRuntime.wrap(function _callee21$(_context21) {
          while (1) {
            switch (_context21.prev = _context21.next) {
              case 0:
                idf = p.item(0);
                id = idf.getLex();
                obj = idf.getObj();
                found = false;
                value = null;
                heap = this.scope.getHeap();

                if (heap.contains(id)) {
                  value = heap.get(id);
                  found = true;
                }

                if (!found) {
                  value = this.evalEnv.get(cxt, id, this.scope, obj);
                } // if(__DEV__) {


                console.log("".concat(id, "=").concat(value)); // }

                p.set(value);

              case 10:
              case "end":
                return _context21.stop();
            }
          }
        }, _callee21, this);
      }));

      function exec18(_x40, _x41) {
        return _exec20.apply(this, arguments);
      }

      return exec18;
    }(),
    // E -> FUNC_HEAD FUNC_TAIL
    exec19: function () {
      var _exec21 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee22(cxt, p) {
        return regeneratorRuntime.wrap(function _callee22$(_context22) {
          while (1) {
            switch (_context22.prev = _context22.next) {
              case 0:
                _context22.next = 2;
                return this.execFun(cxt, p, 0, null);

              case 2:
              case "end":
                return _context22.stop();
            }
          }
        }, _callee22, this);
      }));

      function exec19(_x42, _x43) {
        return _exec21.apply(this, arguments);
      }

      return exec19;
    }(),
    // FUNC_HEAD -> function (
    exec20: function () {
      var _exec22 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee23(cxt, p) {
        return regeneratorRuntime.wrap(function _callee23$(_context23) {
          while (1) {
            switch (_context23.prev = _context23.next) {
              case 0:
              case "end":
                return _context23.stop();
            }
          }
        }, _callee23);
      }));

      function exec20(_x44, _x45) {
        return _exec22.apply(this, arguments);
      }

      return exec20;
    }(),
    // FUNC_TAIL -> )
    exec21: function () {
      var _exec23 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee24(cxt, p) {
        return regeneratorRuntime.wrap(function _callee24$(_context24) {
          while (1) {
            switch (_context24.prev = _context24.next) {
              case 0:
              case "end":
                return _context24.stop();
            }
          }
        }, _callee24);
      }));

      function exec21(_x46, _x47) {
        return _exec23.apply(this, arguments);
      }

      return exec21;
    }(),
    // FUNC_TAIL -> PL )
    exec22: function () {
      var _exec24 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee25(cxt, p) {
        return regeneratorRuntime.wrap(function _callee25$(_context25) {
          while (1) {
            switch (_context25.prev = _context25.next) {
              case 0:
              case "end":
                return _context25.stop();
            }
          }
        }, _callee25);
      }));

      function exec22(_x48, _x49) {
        return _exec24.apply(this, arguments);
      }

      return exec22;
    }(),
    // PL -> E
    exec23: function () {
      var _exec25 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee26(cxt, p) {
        return regeneratorRuntime.wrap(function _callee26$(_context26) {
          while (1) {
            switch (_context26.prev = _context26.next) {
              case 0:
              case "end":
                return _context26.stop();
            }
          }
        }, _callee26);
      }));

      function exec23(_x50, _x51) {
        return _exec25.apply(this, arguments);
      }

      return exec23;
    }(),
    // PL -> E , PL
    exec24: function () {
      var _exec26 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee27(cxt, p) {
        return regeneratorRuntime.wrap(function _callee27$(_context27) {
          while (1) {
            switch (_context27.prev = _context27.next) {
              case 0:
              case "end":
                return _context27.stop();
            }
          }
        }, _callee27);
      }));

      function exec24(_x52, _x53) {
        return _exec26.apply(this, arguments);
      }

      return exec24;
    }(),
    // E -> var id = E
    exec25: function () {
      var _exec27 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee28(cxt, p) {
        var id, E;
        return regeneratorRuntime.wrap(function _callee28$(_context28) {
          while (1) {
            switch (_context28.prev = _context28.next) {
              case 0:
                id = p.item(1);
                E = p.item(3);
                _context28.next = 4;
                return this.exec(cxt, E);

              case 4:
                this.scope.getHeap().put(id.getLex(), E.get());

              case 5:
              case "end":
                return _context28.stop();
            }
          }
        }, _callee28, this);
      }));

      function exec25(_x54, _x55) {
        return _exec27.apply(this, arguments);
      }

      return exec25;
    }(),
    // E -> IF_HEAD
    exec26: function () {
      var _exec28 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee29(cxt, p) {
        var E, SL;
        return regeneratorRuntime.wrap(function _callee29$(_context29) {
          while (1) {
            switch (_context29.prev = _context29.next) {
              case 0:
                // if ( E ) { SL }
                E = p.item(2);
                _context29.next = 3;
                return this.exec(cxt, E);

              case 3:
                if (!E.get()) {
                  _context29.next = 8;
                  break;
                }

                SL = p.item(5);
                _context29.next = 7;
                return this.exec(cxt, SL);

              case 7:
                if (this.ctrlFlow != Expr.F_RETURN) {
                  p.set(SL.get());
                }

              case 8:
              case "end":
                return _context29.stop();
            }
          }
        }, _callee29, this);
      }));

      function exec26(_x56, _x57) {
        return _exec28.apply(this, arguments);
      }

      return exec26;
    }(),
    // E -> IF_HEAD IF_TAIL
    exec27: function () {
      var _exec29 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee30(cxt, p) {
        var E, SL1, SL2;
        return regeneratorRuntime.wrap(function _callee30$(_context30) {
          while (1) {
            switch (_context30.prev = _context30.next) {
              case 0:
                // if ( E } { SL1 } else { SL2 }
                E = p.item(2);
                _context30.next = 3;
                return this.exec(cxt, E);

              case 3:
                if (!E.get()) {
                  _context30.next = 10;
                  break;
                }

                SL1 = p.item(5);
                _context30.next = 7;
                return this.exec(cxt, SL1);

              case 7:
                if (this.ctrlFlow != Expr.F_RETURN) {
                  p.set(SL1.get());
                }

                _context30.next = 14;
                break;

              case 10:
                SL2 = p.item(9);
                _context30.next = 13;
                return this.exec(cxt, SL2);

              case 13:
                if (this.ctrlFlow != Expr.F_RETURN) {
                  p.set(SL2.get());
                }

              case 14:
              case "end":
                return _context30.stop();
            }
          }
        }, _callee30, this);
      }));

      function exec27(_x58, _x59) {
        return _exec29.apply(this, arguments);
      }

      return exec27;
    }(),
    // IF_HEAD -> if ( E ) { SL }
    exec28: function () {
      var _exec30 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee31(cxt, p) {
        var E, SL;
        return regeneratorRuntime.wrap(function _callee31$(_context31) {
          while (1) {
            switch (_context31.prev = _context31.next) {
              case 0:
                E = p.item(2);
                _context31.next = 3;
                return this.exec(cxt, E);

              case 3:
                if (!E.get()) {
                  _context31.next = 8;
                  break;
                }

                SL = p.item(5);
                _context31.next = 7;
                return this.exec(cxt, SL);

              case 7:
                if (this.ctrlFlow != Expr.F_RETURN) {
                  p.set(SL.get());
                }

              case 8:
              case "end":
                return _context31.stop();
            }
          }
        }, _callee31, this);
      }));

      function exec28(_x60, _x61) {
        return _exec30.apply(this, arguments);
      }

      return exec28;
    }(),
    // IF_TAIL -> else { SL }
    exec29: function () {
      var _exec31 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee32(cxt, p) {
        return regeneratorRuntime.wrap(function _callee32$(_context32) {
          while (1) {
            switch (_context32.prev = _context32.next) {
              case 0:
              case "end":
                return _context32.stop();
            }
          }
        }, _callee32);
      }));

      function exec29(_x62, _x63) {
        return _exec31.apply(this, arguments);
      }

      return exec29;
    }(),
    // E -> while ( E ) { SL }
    exec30: function () {
      var _exec32 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee33(cxt, p) {
        var E, SL;
        return regeneratorRuntime.wrap(function _callee33$(_context33) {
          while (1) {
            switch (_context33.prev = _context33.next) {
              case 0:
                E = p.item(2);
                SL = p.item(5);
                _context33.next = 4;
                return this.exec(cxt, E);

              case 4:
                if (!E.get()) {
                  _context33.next = 13;
                  break;
                }

                _context33.next = 7;
                return this.exec(cxt, SL);

              case 7:
                if (!(this.ctrlFlow != -1)) {
                  _context33.next = 9;
                  break;
                }

                return _context33.abrupt("break", 13);

              case 9:
                _context33.next = 11;
                return this.exec(cxt, E);

              case 11:
                _context33.next = 4;
                break;

              case 13:
                if (this.ctrlFlow == Expr.F_BREAK) {
                  this.ctrlFlow = -1;
                }

              case 14:
              case "end":
                return _context33.stop();
            }
          }
        }, _callee33, this);
      }));

      function exec30(_x64, _x65) {
        return _exec32.apply(this, arguments);
      }

      return exec30;
    }(),
    // E -> E = E
    exec31: function () {
      var _exec33 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee34(cxt, p) {
        var E1, E2, idf, id, obj, heap;
        return regeneratorRuntime.wrap(function _callee34$(_context34) {
          while (1) {
            switch (_context34.prev = _context34.next) {
              case 0:
                E1 = p.item(0);
                E2 = p.item(2);
                idf = E1.item(0);
                id = "";

                if (!(idf.factor().get() == 15)) {
                  _context34.next = 8;
                  break;
                }

                id = idf.getLex();
                _context34.next = 11;
                break;

              case 8:
                _context34.next = 10;
                return this.exec(cxt, E1);

              case 10:
                id = E1.get();

              case 11:
                obj = idf.getObj();
                _context34.next = 14;
                return this.exec(cxt, E2);

              case 14:
                heap = this.scope.getHeap();

                if (heap.contains(id)) {
                  heap.put(id, E2.get());
                } else {
                  this.evalEnv.set(cxt, id, E2.get(), this.scope, obj);
                }

              case 16:
              case "end":
                return _context34.stop();
            }
          }
        }, _callee34, this);
      }));

      function exec31(_x66, _x67) {
        return _exec33.apply(this, arguments);
      }

      return exec31;
    }(),
    // E -> return E
    exec32: function () {
      var _exec34 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee35(cxt, p) {
        var E;
        return regeneratorRuntime.wrap(function _callee35$(_context35) {
          while (1) {
            switch (_context35.prev = _context35.next) {
              case 0:
                E = p.item(1);
                _context35.next = 3;
                return this.exec(cxt, E);

              case 3:
                this.tree.get().set(E.get());
                this.ctrlFlow = Expr.F_RETURN;

              case 5:
              case "end":
                return _context35.stop();
            }
          }
        }, _callee35, this);
      }));

      function exec32(_x68, _x69) {
        return _exec34.apply(this, arguments);
      }

      return exec32;
    }(),
    // E -> break
    exec33: function () {
      var _exec35 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee36(cxt, p) {
        return regeneratorRuntime.wrap(function _callee36$(_context36) {
          while (1) {
            switch (_context36.prev = _context36.next) {
              case 0:
                this.ctrlFlow = Expr.F_BREAK;

              case 1:
              case "end":
                return _context36.stop();
            }
          }
        }, _callee36, this);
      }));

      function exec33(_x70, _x71) {
        return _exec35.apply(this, arguments);
      }

      return exec33;
    }(),
    // E -> loop E ( E ) { SL }
    exec34: function () {
      var _exec36 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee37(cxt, p) {
        var fun, name, OE, CE, SL, obj, loopObject;
        return regeneratorRuntime.wrap(function _callee37$(_context37) {
          while (1) {
            switch (_context37.prev = _context37.next) {
              case 0:
                fun = p.item(0);
                name = fun.getLex();
                OE = p.item(1);
                _context37.next = 5;
                return this.exec(cxt, OE);

              case 5:
                CE = p.item(3);
                SL = p.item(6);
                obj = OE.get();
                loopObject = this.evalEnv.getLoop(cxt, name, 0, obj);

              case 9:
                if (!loopObject.hasNext()) {
                  _context37.next = 20;
                  break;
                }

                loopObject.next();
                _context37.next = 13;
                return this.exec(cxt, CE);

              case 13:
                if (!CE.get()) {
                  _context37.next = 16;
                  break;
                }

                _context37.next = 16;
                return this.exec(cxt, SL);

              case 16:
                if (!(this.ctrlFlow != -1)) {
                  _context37.next = 18;
                  break;
                }

                return _context37.abrupt("break", 20);

              case 18:
                _context37.next = 9;
                break;

              case 20:
                loopObject.clean(); // 恢复break状态

                if (this.ctrlFlow == 1) {
                  this.ctrlFlow = -1;
                }

              case 22:
              case "end":
                return _context37.stop();
            }
          }
        }, _callee37, this);
      }));

      function exec34(_x72, _x73) {
        return _exec36.apply(this, arguments);
      }

      return exec34;
    }(),
    // E -> RANGE
    exec35: function () {
      var _exec37 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee38(cxt, p) {
        var r;
        return regeneratorRuntime.wrap(function _callee38$(_context38) {
          while (1) {
            switch (_context38.prev = _context38.next) {
              case 0:
                r = p.item(0);
                p.set(r.get());

              case 2:
              case "end":
                return _context38.stop();
            }
          }
        }, _callee38);
      }));

      function exec35(_x74, _x75) {
        return _exec37.apply(this, arguments);
      }

      return exec35;
    }(),
    // E -> switch ( E ) { CASE_SL }
    exec36: function () {
      var _exec38 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee39(cxt, p) {
        var E, condition, childCount, length, i, caseIndex, CASE_S;
        return regeneratorRuntime.wrap(function _callee39$(_context39) {
          while (1) {
            switch (_context39.prev = _context39.next) {
              case 0:
                E = p.item(2);
                _context39.next = 3;
                return this.exec(cxt, E);

              case 3:
                condition = E.get();
                childCount = p.size(); // 减去固定位置(不包括最后一个}，这个用于跟case ; case 中的;配对)，剩下CASE_SL位置的数量

                childCount -= 5; // 求case分支的数量

                length = childCount / 2; // 语法树结构如下：
                // switch ( E ) { case 1; case 2; case 3...}

                i = 0;

              case 8:
                if (!(i < length)) {
                  _context39.next = 19;
                  break;
                }

                caseIndex = 5 + i * 2;
                CASE_S = p.item(caseIndex);
                _context39.next = 13;
                return this.execCase(cxt, condition, CASE_S);

              case 13:
                if (!_context39.sent) {
                  _context39.next = 16;
                  break;
                }

                p.set(CASE_S.get());
                return _context39.abrupt("break", 19);

              case 16:
                ++i;
                _context39.next = 8;
                break;

              case 19:
              case "end":
                return _context39.stop();
            }
          }
        }, _callee39, this);
      }));

      function exec36(_x76, _x77) {
        return _exec38.apply(this, arguments);
      }

      return exec36;
    }(),
    execCase: function () {
      var _execCase = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee40(cxt, condition, factor) {
        var E, value, matched, type, SL;
        return regeneratorRuntime.wrap(function _callee40$(_context40) {
          while (1) {
            switch (_context40.prev = _context40.next) {
              case 0:
                // case E : { SL }
                E = factor.item(1);
                _context40.next = 3;
                return this.exec(cxt, E);

              case 3:
                value = E.get();
                matched = false;
                type = this.getCpType(condition, value);
                _context40.t0 = type;
                _context40.next = _context40.t0 === _YIUICommon.DataType.NUMERIC ? 9 : _context40.t0 === _YIUICommon.DataType.STRING ? 11 : 13;
                break;

              case 9:
                matched = _YIUIBase.YIUI.TypeConvertor.toDecimal(condition).equals(_YIUIBase.YIUI.TypeConvertor.toDecimal(value));
                return _context40.abrupt("break", 15);

              case 11:
                matched = Expr.eq(condition.toString(), value.toString());
                return _context40.abrupt("break", 15);

              case 13:
                matched = condition == value;
                return _context40.abrupt("break", 15);

              case 15:
                if (!matched) {
                  _context40.next = 20;
                  break;
                }

                SL = factor.item(4);
                _context40.next = 19;
                return this.exec(cxt, SL);

              case 19:
                factor.set(SL.get());

              case 20:
                return _context40.abrupt("return", matched);

              case 21:
              case "end":
                return _context40.stop();
            }
          }
        }, _callee40, this);
      }));

      function execCase(_x78, _x79, _x80) {
        return _execCase.apply(this, arguments);
      }

      return execCase;
    }(),
    // CASE_SL -> CASE_S
    exec37: function () {
      var _exec39 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee41(cxt, p) {
        return regeneratorRuntime.wrap(function _callee41$(_context41) {
          while (1) {
            switch (_context41.prev = _context41.next) {
              case 0:
              case "end":
                return _context41.stop();
            }
          }
        }, _callee41);
      }));

      function exec37(_x81, _x82) {
        return _exec39.apply(this, arguments);
      }

      return exec37;
    }(),
    // CASE_SL -> CASE_S ; CASE_SL
    exec38: function () {
      var _exec40 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee42(cxt, p) {
        return regeneratorRuntime.wrap(function _callee42$(_context42) {
          while (1) {
            switch (_context42.prev = _context42.next) {
              case 0:
              case "end":
                return _context42.stop();
            }
          }
        }, _callee42);
      }));

      function exec38(_x83, _x84) {
        return _exec40.apply(this, arguments);
      }

      return exec38;
    }(),
    // CASE_S -> case E : { SL }
    exec39: function () {
      var _exec41 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee43(cxt, p) {
        return regeneratorRuntime.wrap(function _callee43$(_context43) {
          while (1) {
            switch (_context43.prev = _context43.next) {
              case 0:
              case "end":
                return _context43.stop();
            }
          }
        }, _callee43);
      }));

      function exec39(_x85, _x86) {
        return _exec41.apply(this, arguments);
      }

      return exec39;
    }(),
    execFun: function () {
      var _execFun = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee44(cxt, p, postfixCount, extInfo) {
        var fun, name, obj, cv, impl, newCxt, l, args, i, P, r, macro;
        return regeneratorRuntime.wrap(function _callee44$(_context44) {
          while (1) {
            switch (_context44.prev = _context44.next) {
              case 0:
                fun = p.item(0); // 转小写

                name = fun.getLex();
                obj = fun.getObj();
                cv = false;
                impl = null;
                newCxt = null;

                if (obj != null) {
                  newCxt = this.evalEnv.resolveObject(cxt, this.scope, obj);
                  impl = this.evalEnv.evalFuncImpl(cxt, this.scope, obj, name);

                  if (impl == null) {
                    impl = this.implMap.get(name);
                  }
                } else {
                  newCxt = cxt;
                  impl = this.implMap.get(name);
                } // if ( 1, 2, 3, 4 )


                l = (p.size() - 2) / 2;
                l = Math.floor(l);
                args = new Array(l);
                i = 0;

              case 11:
                if (!(i < l)) {
                  _context44.next = 69;
                  break;
                }

                P = p.item(2 + i * 2);

                if (!fun.isCtrl()) {
                  _context44.next = 61;
                  break;
                }

                if (!Expr.eq(Expr.IIF, name)) {
                  _context44.next = 41;
                  break;
                }

                if (!(i == 0)) {
                  _context44.next = 24;
                  break;
                }

                _context44.next = 18;
                return this.exec(cxt, P);

              case 18:
                _context44.next = 20;
                return P.get();

              case 20:
                cv = _context44.sent;
                args[i] = cv;
                _context44.next = 39;
                break;

              case 24:
                if (!cv) {
                  _context44.next = 33;
                  break;
                }

                if (!(i == 1)) {
                  _context44.next = 31;
                  break;
                }

                _context44.next = 28;
                return this.exec(cxt, P);

              case 28:
                _context44.next = 30;
                return P.get();

              case 30:
                args[i] = _context44.sent;

              case 31:
                _context44.next = 39;
                break;

              case 33:
                if (!(i == 2)) {
                  _context44.next = 39;
                  break;
                }

                _context44.next = 36;
                return this.exec(cxt, P);

              case 36:
                _context44.next = 38;
                return P.get();

              case 38:
                args[i] = _context44.sent;

              case 39:
                _context44.next = 59;
                break;

              case 41:
                if (!(i % 2 == 0)) {
                  _context44.next = 52;
                  break;
                }

                _context44.next = 44;
                return this.exec(cxt, P);

              case 44:
                _context44.t0 = _YIUIBase.YIUI.TypeConvertor;
                _context44.next = 47;
                return P.get();

              case 47:
                _context44.t1 = _context44.sent;
                cv = _context44.t0.toBoolean.call(_context44.t0, _context44.t1);
                args[i] = cv;
                _context44.next = 59;
                break;

              case 52:
                if (!cv) {
                  _context44.next = 59;
                  break;
                }

                _context44.next = 55;
                return this.exec(cxt, P);

              case 55:
                _context44.next = 57;
                return P.get();

              case 57:
                args[i] = _context44.sent;
                return _context44.abrupt("break", 69);

              case 59:
                _context44.next = 66;
                break;

              case 61:
                _context44.next = 63;
                return this.exec(cxt, P);

              case 63:
                _context44.next = 65;
                return P.get();

              case 65:
                args[i] = _context44.sent;

              case 66:
                ++i;
                _context44.next = 11;
                break;

              case 69:
                r = null;

                if (!(impl != null)) {
                  _context44.next = 76;
                  break;
                }

                _context44.next = 73;
                return impl(name, newCxt, args);

              case 73:
                r = _context44.sent;
                _context44.next = 92;
                break;

              case 76:
                _context44.next = 78;
                return this.evalEnv.checkMacro(newCxt || cxt, name);

              case 78:
                macro = _context44.sent;

                if (!(macro != null)) {
                  _context44.next = 85;
                  break;
                }

                _context44.next = 82;
                return this.evalEnv.evalMacro(newCxt || cxt, this.scope, name, macro, args);

              case 82:
                r = _context44.sent;
                _context44.next = 92;
                break;

              case 85:
                if (!(obj != null)) {
                  _context44.next = 91;
                  break;
                }

                _context44.next = 88;
                return this.evalEnv.evalObject(cxt, obj, name, args);

              case 88:
                r = _context44.sent;
                _context44.next = 92;
                break;

              case 91:
                alert("未实现表达式： " + name);

              case 92:
                // if (__DEV__) {
                // console.log(`${name} = ${r}`);
                // }
                p.set(r);

              case 93:
              case "end":
                return _context44.stop();
            }
          }
        }, _callee44, this);
      }));

      function execFun(_x87, _x88, _x89, _x90) {
        return _execFun.apply(this, arguments);
      }

      return execFun;
    }()
  });

  Expr.Parser = function (implMap) {
    this.implMap = implMap;
  };

  _YIUICommon.Lang.impl(Expr.Parser, {
    eval: function () {
      var _eval3 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee45(env, script, tree, cxt, scope) {
        var r, eval1;
        return regeneratorRuntime.wrap(function _callee45$(_context45) {
          while (1) {
            switch (_context45.prev = _context45.next) {
              case 0:
                r = null;

                if (!this.parse(script, tree)) {
                  _context45.next = 9;
                  break;
                }

                tree.opti();
                eval1 = new Expr.Evaluator(this.implMap, env, tree, scope);
                _context45.next = 6;
                return eval1.eval(cxt);

              case 6:
                _context45.next = 8;
                return tree.get().get();

              case 8:
                r = _context45.sent;

              case 9:
                return _context45.abrupt("return", r);

              case 10:
              case "end":
                return _context45.stop();
            }
          }
        }, _callee45, this);
      }));

      function _eval(_x91, _x92, _x93, _x94, _x95) {
        return _eval3.apply(this, arguments);
      }

      return _eval;
    }(),
    evalByTree: function () {
      var _evalByTree = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee46(env, tree, cxt, scope) {
        var eval1;
        return regeneratorRuntime.wrap(function _callee46$(_context46) {
          while (1) {
            switch (_context46.prev = _context46.next) {
              case 0:
                eval1 = new Expr.Evaluator(this.implMap, env, tree, scope);
                _context46.next = 3;
                return eval1.eval(cxt);

              case 3:
                return _context46.abrupt("return", tree.get().get());

              case 4:
              case "end":
                return _context46.stop();
            }
          }
        }, _callee46, this);
      }));

      function evalByTree(_x96, _x97, _x98, _x99) {
        return _evalByTree.apply(this, arguments);
      }

      return evalByTree;
    }(),
    slAsString: function slAsString(top) {
      return top != null ? top.rule().isSLAsString() : true;
    },
    printStack: function printStack(stack) {
      if (true) {
        return;
      }

      var v = stack.values();
      var length = v.length;

      for (var i = 0; i < length; ++i) {
        var trace = v[i];
        trace.print();
      } //console.log("    ");
      //console.log("    ");

    },
    parse: function parse(script, tree) {
      var lex = new Expr.Lexer(script);
      var rules = Expr.rules();
      var obj = null;
      var lv = null;
      var li = lex.next(true);
      var oli = -1;
      obj = lex.getObj();
      lv = lex.getLexValue();
      var dataType = lex.getType();
      var topTrace = null;
      var delayTrace = null;
      var curTrace = null;
      var successed = false;
      var refLexID = new Expr.RefInt(-1);
      var delayID = -1;
      var consumPos = -1;
      var handled = false;
      var finish = li == -1;
      var stack = new _YIUICommon.Stack();

      while (!finish) {
        handled = false;

        if (delayTrace != null) {
          if (this.pre(rules, stack, tree, delayTrace, delayID)) {
            delayTrace = null;
            handled = true;
          }
        }

        topTrace = stack.empty() ? null : stack.peek();
        this.printStack(stack);
        refLexID.set(li);
        oli = li;
        var needPre = this.checkPre(stack, topTrace, refLexID, tree, lv);
        li = refLexID.get(); // 被消耗

        if (li == -1) {
          li = lex.next(this.slAsString(topTrace));
          obj = lex.getObj();
          lv = lex.getLexValue();
          dataType = lex.getType();
          delayID = li;
          finish = li == -1;

          if (oli != -1 || li != -1) {
            handled = true;
          }
        }

        if (needPre) {
          var predictRule = rules.first(li);

          if (predictRule != null) {
            var preTrace = new Expr.RuleTrace(predictRule);
            stack.push(preTrace);
            topTrace = preTrace;
            consumPos = preTrace.consum(li);
            var terFactor = this.putToTree(tree, preTrace);
            this.dealLex(li, terFactor, consumPos, dataType, obj, lv);
            li = lex.next(this.slAsString(topTrace));
            obj = lex.getObj();
            lv = lex.getLexValue();
            dataType = lex.getType();
            finish = li == -1;
            this.printStack(stack); // 因为有新的元素加入，之前需要预测的不再有效

            delayTrace = null;
            handled = true;
          } else {
            if (topTrace != null && topTrace.matched()) {
              delayTrace = topTrace;
              delayID = li;
              this.printStack(stack);
            }
          }
        } // 如果栈顶成功匹配，那么将其出栈，并且让上一级规则来消费匹配的语法因子


        if (topTrace.matched()) {
          curTrace = topTrace;
          stack.pop();

          if (li != -1 && this.needMore(rules, stack, topTrace, li)) {
            stack.push(curTrace);
            delayTrace = curTrace;
            delayID = li;
            handled = true;
            this.printStack(stack);
          } else {
            var fullMatched = false;
            var consumed = false;

            if (!stack.empty()) {
              topTrace = stack.peek();

              if (!topTrace.matched() && topTrace.match(curTrace.left())) {
                consumed = true;
                this.consum(tree, topTrace, curTrace);
                curTrace = topTrace;
                stack.pop();
                this.printStack(stack);

                while (curTrace.matched()) {
                  if (li != -1 && this.needMore(rules, stack, curTrace, li)) {
                    delayTrace = curTrace;
                    delayID = li;
                    handled = true;
                    this.printStack(stack);
                    break;
                  }

                  if (!stack.empty()) {
                    topTrace = stack.peek();

                    if (li != -1 && this.needMore(rules, stack, curTrace, li)) {
                      stack.push(curTrace);
                      break;
                    }

                    if (topTrace.match(curTrace.left())) {
                      this.consum(tree, topTrace, curTrace);
                      curTrace = topTrace;
                      stack.pop();
                      this.printStack(stack);
                    } else {
                      break;
                    }
                  } else {
                    fullMatched = true;
                    stack.push(curTrace);
                    break;
                  }
                }
              }
            }

            if (!consumed) {
              if (delayTrace != null && !handled) {
                break;
              }

              stack.push(curTrace);
              delayTrace = curTrace;
              delayID = li;
              this.printStack(stack);
            } else {
              if (!fullMatched) {
                stack.push(curTrace);
                delayTrace = curTrace;
                delayID = li;
              } else {
                successed = true;

                if (li == -1) {
                  li = lex.next(this.slAsString(topTrace));
                  lv = lex.getLexValue();
                  dataType = lex.getType();
                }

                if (li != -1) {
                  topTrace = stack.pop();

                  if (this.needMore(rules, stack, topTrace, li)) {
                    stack.push(topTrace);
                    delayTrace = topTrace;
                    delayID = li;
                    this.printStack(stack);
                  } else {
                    successed = false;
                    this.moreInput(rules, stack, li, lv);
                    break;
                  }
                } else {
                  break;
                }
              }
            }
          }
        }

        if (li == -1) {
          li = lex.next(this.slAsString(topTrace));
          obj = lex.getObj();
          lv = lex.getLexValue();
          dataType = lex.getType();
          delayID = li;
          finish = li == -1;

          if (li == -1) {
            if (!stack.empty()) {
              topTrace = stack.peek();

              if (!topTrace.matched()) {
                this.notMatch(rules, stack);
                break;
              } else {
                if (stack.size() == 1) {
                  successed = true;
                  break;
                } else if (delayTrace != null) {
                  finish = false;
                }
              }
            }
          }
        }
      }

      if (successed) {
        tree.extract();
      }

      return successed;
    },
    dealLex: function dealLex(li, factor, pos, dataType, obj, lv) {
      factor.setChildLex(pos, obj, lv);

      if (li == 14) {
        if (Expr.eq(Expr.IIF, lv) || Expr.eq(Expr.IIFS, lv)) {
          factor.setChildCtrl(pos, true);
        }
      }

      switch (li) {
        case 16:
          {
            var value = null;

            switch (dataType) {
              case Expr.T_BOOL:
                {
                  value = Expr.eq("true", lv);
                }
                break;

              case Expr.T_INT:
                {
                  value = parseInt(lv == null ? 0 : lv);
                }
                break;

              case Expr.T_NUM:
                {
                  value = new _decimal["default"](lv == null ? 0 : lv);
                }
                break;

              case Expr.T_STR:
                {
                  value = lv;
                }
                break;
            }

            factor.setChild(pos, value);
          }
          break;

        case 32:
          {
            factor.setChild(pos, lv);
          }
          break;
      }
    },
    putToTree: function putToTree(tree, ruleTrace) {
      var r = ruleTrace.rule();
      var factor = new Expr.Item(r.left(), r);
      var rs = r.right();
      var length = rs.length;

      for (var i = 0; i < length; ++i) {
        var childFactor = new Expr.Item(rs[i]);
        factor.add(childFactor);
      }

      tree.push(factor);
      return factor;
    },
    pre: function pre(rules, stack, tree, delayTrace, fid) {
      var result = false;
      stack.pop();
      var topFactor = tree.pop();
      var delayRule = this.resolveDelay(rules, stack, delayTrace, fid);

      if (delayRule != null) {
        var ruleTrace = new Expr.RuleTrace(delayRule);
        stack.push(ruleTrace);
        this.putToTree(tree, ruleTrace);
        tree.push(topFactor);
        this.consum(tree, ruleTrace, delayTrace);
        result = true;
      } else {
        stack.push(delayTrace);
        tree.push(topFactor);
        result = false;
      }

      return result;
    },
    notMatch: function notMatch(rules, stack) {},
    moreInput: function moreInput(rules, stack, li, lv) {},
    checkPre: function checkPre(stack, topTrace, li, tree, lv) {
      var b = true;

      if (topTrace != null) {
        if (!topTrace.matched()) {
          if (topTrace.match(li.get())) {
            var cpos = topTrace.consum(li.get());
            var ti = tree.peek();
            ti.setChildLex(cpos, null, lv);
            li.set(-1);
            b = false;
          }
        } else {
          if (topTrace.isClose() || li.get() == -1) {
            b = false;
          }
        }
      }

      return b;
    },
    resolveDelay: function resolveDelay(rules, stack, delayRule, li) {
      var factorID = delayRule.left();
      var rule = null;

      if (delayRule.matched()) {
        switch (factorID) {
          case Expr.E:
            {
              switch (li) {
                case Expr.SEMI:
                  // rule 1: SL -> E ; SL
                  rule = rules.at(1);
                  break;
                // rule 2: E -> E + E

                case Expr.ADD:
                  rule = rules.at(2);
                  break;
                // rule 3: E -> E - E

                case Expr.SUB:
                  rule = rules.at(3);
                  break;
                // rule 4: E -> E * E

                case Expr.MUL:
                  rule = rules.at(4);
                  break;
                // rule 5: E -> E / E

                case Expr.DIV:
                  rule = rules.at(5);
                  break;
                // rule 6: E -> E & E

                case Expr.STRCAT:
                  rule = rules.at(6);
                  break;
                // rule 9: E -> E || E

                case Expr.OR:
                  rule = rules.at(9);
                  break;
                // rule 10: E -> E && E

                case Expr.AND:
                  rule = rules.at(10);
                  break;
                // rule 11: E -> E == E

                case Expr.EQ:
                  rule = rules.at(11);
                  break;
                // rule 12: E -> E <> E

                case Expr.NEQ:
                  rule = rules.at(12);
                  break;
                // rule 13: E -> E > E

                case Expr.GT:
                  rule = rules.at(13);
                  break;
                // rule 14: E -> E >= E

                case Expr.GT_EQ:
                  rule = rules.at(14);
                  break;
                // rule 15: E -> E < E

                case Expr.LT:
                  rule = rules.at(15);
                  break;
                // rule 16: E -> E <= E

                case Expr.LT_EQ:
                  rule = rules.at(16);
                  break;
                // rule 24: PL -> E , PL

                case Expr.COMMA:
                  rule = rules.at(24);
                  break;
                // rule 31: E -> E = E

                case Expr.ASSIGN:
                  rule = rules.at(31);
                  break;
                // )

                case Expr.RB:
                  {
                    if (delayRule.rule().index() == 7) {
                      // rule 7: E -> ( E )
                      rule = null;
                    } else {
                      // rule 23: PL -> E
                      rule = rules.at(23);
                    }
                  }
                  break;

                default:
                  // rule 0: SL -> E
                  rule = rules.at(0);
                  break;
              }
            }
            break;

          case Expr.FUN_H:
            {
              // rule 19: E -> FUNC_HEAD FUNC_TAIL
              rule = rules.at(19);
            }
            break;

          case Expr.PL:
            {
              // rule 22: FUNC_TAIL -> PL )
              if (li == Expr.RB) {
                rule = rules.at(22);
              }
            }
            break;

          case Expr.IF_H:
            // rule 27: E -> IF_HEAD IF_TAIL
            if (li == Expr.ELSE) {
              rule = rules.at(27);
            } else {
              // rule 26: E -> IF_HEAD
              rule = rules.at(26);
            }

            break;

          case Expr.CASE_S:
            // rule 38: CASE_SL -> CASE_S ; CASE_SL
            if (li == Expr.SEMI) {
              rule = rules.at(38);
            } else {
              // rule 37: CASE_SL -> CASE_S
              rule = rules.at(37);
            }

            break;
        }
      }

      return rule;
    },
    needMore: function needMore(rules, stack, r, li) {
      var tr = null;

      if (!stack.empty()) {
        tr = stack.peek().rule();
      }

      var has = false;

      switch (r.left()) {
        case Expr.E:
          switch (li) {
            // rule 1: SL -> E ; SL
            case 26:
              has = true;

              if (tr != null) {
                switch (tr.index()) {
                  // rule 2: E -> E + E
                  // rule 3: E -> E - E
                  // rule 4: E -> E * E
                  // rule 5: E -> E / E
                  // rule 6: E -> E & E
                  // rule 9: E -> E || E
                  // rule 10: E -> E && E
                  // rule 11: E -> E == E
                  // rule 12: E -> E <> E
                  // rule 13: E -> E > E
                  // rule 14: E -> E >= E
                  // rule 15: E -> E < E
                  // rule 16: E -> E <= E
                  // rule 24: PL -> E , PL
                  // rule 25: E -> var id = E
                  // rule 31: E -> E = E
                  case 2:
                  case 3:
                  case 4:
                  case 5:
                  case 6:
                  case 9:
                  case 10:
                  case 11:
                  case 12:
                  case 13:
                  case 14:
                  case 15:
                  case 16:
                  case 24:
                  case 25:
                  case 31:
                    has = false;
                    break;
                }
              }

              break;
            // +(加),-(减)运算的优先级只比*(乘),/(除)低，也比栈内优先级低
            // rule 2: E -> E + E

            case 0: // rule 3: E -> E - E

            case 2:
              has = true;

              if (tr != null) {
                switch (tr.index()) {
                  // rule 2: E -> E + E
                  // rule 3: E -> E - E
                  // rule 4: E -> E * E
                  // rule 5: E -> E / E
                  case 2:
                  case 3:
                  case 4:
                  case 5:
                    has = false;
                    break;
                }
              }

              break;
            // *(乘),/(除)运算具有最高的优先级，但比栈内优先级低
            // rule 4: E -> E * E

            case 3: // rule 5: E -> E / E

            case 4:
              has = true;

              if (tr != null) {
                switch (tr.index()) {
                  // rule 4: E -> E * E
                  // rule 5: E -> E / E
                  case 4:
                  case 5:
                    has = false;
                }
              }

              break;
            // rule 6: E -> E & E

            case 1:
              has = true;
              break;
            // ||(逻辑或)的优先级低于所有算术运算、关系运算和逻辑与，逻辑非
            // rule 9: E -> E || E

            case 5:
              has = true;

              if (tr != null) {
                switch (tr.index()) {
                  // rule 2: E -> E + E
                  // rule 3: E -> E - E
                  // rule 4: E -> E * E
                  // rule 5: E -> E / E
                  // rule 6: E -> E & E
                  // rule 8: E -> ! E
                  // rule 10: E -> E && E
                  // rule 11: E -> E == E
                  // rule 12: E -> E <> E
                  // rule 13: E -> E > E
                  // rule 14: E -> E >= E
                  // rule 15: E -> E < E
                  // rule 16: E -> E <= E
                  case 2:
                  case 3:
                  case 4:
                  case 5:
                  case 6:
                  case 8:
                  case 10:
                  case 11:
                  case 12:
                  case 13:
                  case 14:
                  case 15:
                  case 16:
                    has = false;
                    break;
                }
              }

              break;
            // &&(逻辑与)的优先级低于所有算术运算和关系运算
            // rule 10: E -> E && E

            case 6:
              has = true;

              if (tr != null) {
                switch (tr.index()) {
                  // rule 2: E -> E + E
                  // rule 3: E -> E - E
                  // rule 4: E -> E * E
                  // rule 5: E -> E / E
                  // rule 6: E -> E & E
                  // rule 8: E -> ! E
                  // rule 11: E -> E == E
                  // rule 12: E -> E <> E
                  // rule 13: E -> E > E
                  // rule 14: E -> E >= E
                  // rule 15: E -> E < E
                  // rule 16: E -> E <= E
                  case 2:
                  case 3:
                  case 4:
                  case 5:
                  case 6:
                  case 8:
                  case 11:
                  case 12:
                  case 13:
                  case 14:
                  case 15:
                  case 16:
                    has = false;
                    break;
                }
              }

              break;
            // 关系运算的优先级低于所有运算运算
            // rule 11: E -> E == E

            case 7: // rule 12: E -> E <> E

            case 9: // rule 13: E -> E > E

            case 10: // rule 14: E -> E >= E

            case 11: // rule 15: E -> E < E

            case 12: // rule 16: E -> E <= E

            case 13:
              has = true;

              if (tr != null) {
                switch (tr.index()) {
                  // rule 2: E -> E + E
                  // rule 3: E -> E - E
                  // rule 4: E -> E * E
                  // rule 5: E -> E / E
                  // rule 6: E -> E & E
                  case 2:
                  case 3:
                  case 4:
                  case 5:
                  case 6:
                    has = false;
                    break;
                }
              }

              break;
            // rule 24: PL -> E , PL

            case 19:
              has = true; // ,的优先级低于所有运算符，除了;

              if (tr != null) {
                switch (tr.index()) {
                  // rule 2: E -> E + E
                  // rule 3: E -> E - E
                  // rule 4: E -> E * E
                  // rule 5: E -> E / E
                  // rule 6: E -> E & E
                  // rule 8: E -> ! E
                  // rule 9: E -> E || E
                  // rule 10: E -> E && E
                  // rule 11: E -> E == E
                  // rule 12: E -> E <> E
                  // rule 13: E -> E > E
                  // rule 14: E -> E >= E
                  // rule 15: E -> E < E
                  // rule 16: E -> E <= E
                  // rule 31: E -> E = E
                  case 2:
                  case 3:
                  case 4:
                  case 5:
                  case 6:
                  case 8:
                  case 9:
                  case 10:
                  case 11:
                  case 12:
                  case 13:
                  case 14:
                  case 15:
                  case 16:
                  case 31:
                    has = false;
                    break;
                }
              }

              break;
            // rule 31: E -> E = E

            case 8:
              has = true;
              break;
          }

          break;
      }

      return has;
    },
    consum: function consum(tree, srt, ft) {
      srt.consum(ft.left());
      var pos = srt.pos();
      var ff = tree.pop();
      var sf = tree.peek();
      var cf = sf.item(pos);
      cf.merge(ff);
    }
  });
})();

_YIUIBase.YIUI.ExprUtil = function () {
  var Return = {
    //仅拿来处理Dict值
    getImplValue: function getImplValue(form, key, cxt, obj) {
      var value = null;

      if (!obj) {
        var clt = form.getCellLocation(key);

        if (clt) {
          var comp = form.getComponent(clt.key);

          if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.GRID) {
            comp.setFocusRowIndex(cxt.rowIndex);
          }
        }

        value = form.getValue(key);
      } else {
        var grid = form.getGridInfoByTableKey(obj);
        var lv = form.getListView(obj);
        var rowIndex = cxt.rowIndex;
        var doc = form.getDocument();
        var table = doc.getByKey(obj);

        if (lv) {
          table.setPos(rowIndex);
          value = table.getByKey(key);
        } else if (grid) {
          grid = form.getComponent(grid.key);
          var gridRow = grid.getRowDataAt(rowIndex);
          var bkmk = gridRow.bookmark;
          if (bkmk < 0) return null;
          table.setByBkmk(bkmk);
          value = table.getByKey(key);
        } else {
          if (table) {
            value = table.getByKey(key);
          }
        }
      }

      return value; // var comp = form.getComponent(key), clt = form.getCellLocation(key), value = form.getValue(key);
      // value = this.convertValue(value);
      //            if (clt) {
      //                comp = form.getComponent(clt.key);
      //                if (comp.type == YIUI.CONTROLTYPE.GRID) {
      //                    var ri = cxt.rowIndex == undefined ? comp.getFocusRowIndex() : cxt.rowIndex;
      //                    if (comp.dataModel.data[ri] !== undefined) {
      //                        var cellKey = comp.dataModel.data[ri].cellKeys[clt.column],
      //                            editOpt = comp.dataModel.colModel.cells[cellKey];
      //                        if (editOpt && editOpt.edittype == "dict" && value !== null && value !== undefined) {
      //                            value = (typeof  value == "object") ? value.oid : JSON.parse(value).oid;
      //                        }
      //                    }
      //                }
      //            }
      // return value;
    },
    setImplValue: function setImplValue(form, key, value, cxt) {
      var cmp = form.getComponent(key);
      var cell = form.getCellLocation(key);

      if (cmp) {
        form.setComponentValue(key, value, true);
      } else if (cell) {
        var rowIndex = cxt.rowIndex;
        form.setCellValByKey(cell.key, rowIndex, key, value);
      }
    },
    convertValue: function convertValue(value) {
      if (value && value instanceof _YIUIBase.YIUI.ItemData) {
        value = value.getOID();
      } else if (_yesCommon.lodash.isObject(value) && value.oid) {
        value = value.oid;
      }

      return value;
    }
  };
  return Return;
}();

var View = View || {};
exports.View = View;

(function () {
  var TblFuns = {
    insert: function insert(name, cxt, args) {
      var table = cxt.obj;
      var row = table.addRow();
      return row;
    },
    append: function append(name, cxt, args) {
      var table = cxt.obj;
      var row = table.addRow();
      return row;
    },
    beforefirst: function beforefirst(name, cxt, args) {
      var table = cxt.obj;
      table.beforeFirst();
      return true;
    },
    next: function next(name, cxt, args) {
      var table = cxt.obj;
      var result = table.next();
      return result;
    },
    size: function size(name, cxt, args) {
      var table = cxt.obj;
      return table.size();
    }
  };
  View.TblFunImpl = new _YIUICommon.HashMap();
  Expr.regCluster(View.TblFunImpl, TblFuns);
  var DocFuns = {
    get: function get(name, cxt, args) {
      var doc = cxt.obj;

      var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      var table = doc.getByKey(key);
      return table;
    },
    set: function set(name, cxt, args) {
      var doc = cxt.obj;

      var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      var table = args[1];
      doc.setByKey(key, table);
      return true;
    }
  };
  View.DocFunImpl = new _YIUICommon.HashMap();
  Expr.regCluster(View.DocFunImpl, TblFuns);
  var JSONObjFuns = {
    length: function length(name, cxt, args) {
      var obj = cxt.obj;
      return Object.getOwnPropertyNames(obj).length;
    },
    get: function get(name, cxt, args) {
      var obj = cxt.obj;

      var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      return obj[key];
    },
    has: function has(name, cxt, args) {
      var obj = cxt.obj;

      var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      return obj.hasOwnProperty(key);
    },
    put: function put(name, cxt, args) {
      var obj = cxt.obj;

      var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      var value = args[1];
      obj[key] = value;
      return obj;
    },
    remove: function remove(name, cxt, args) {
      var obj = cxt.obj;

      var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      delete obj[key];
      return obj;
    }
  };
  View.JSONObjFunImpl = new _YIUICommon.HashMap();
  Expr.regCluster(View.JSONObjFunImpl, JSONObjFuns);
  var JSONArrFuns = {
    length: function length(name, cxt, args) {
      var obj = cxt.obj;
      return obj.length;
    },
    get: function get(name, cxt, args) {
      var obj = cxt.obj;

      var index = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

      return obj[index];
    },
    put: function put(name, cxt, args) {
      var obj = cxt.obj;

      var index = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

      var value = args[1];
      obj[index] = value;
      return obj;
    },
    remove: function remove(name, cxt, args) {
      var obj = cxt.obj;

      var index = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

      obj.splice(index, 1);
      return obj;
    }
  };
  View.JSONArrFunImpl = new _YIUICommon.HashMap();
  Expr.regCluster(View.JSONArrFunImpl, JSONArrFuns);
})();

(function () {
  _YIUIBase.YIUI.TblValImpl = function (options) {
    var Return = {
      getValue: function getValue(tbl, id) {
        return tbl.getByKey(id);
      },
      setValue: function setValue(tbl, id, value) {
        tbl.setByKey(id, value);
      }
    };
    return Return;
  };

  _YIUIBase.YIUI.DocValImpl = function (options) {
    var Return = {
      getValue: function getValue(doc, id) {
        return doc.getByKey(id);
      },
      setValue: function setValue(doc, id, value) {
        doc.setByKey(id, value);
      }
    };
    return Return;
  };

  _YIUIBase.YIUI.JSONObjValImpl = function (options) {
    var Return = {
      getValue: function getValue(json, id) {
        return json[id];
      },
      setValue: function setValue(json, id, value) {
        json[id] = value;
      }
    };
    return Return;
  };

  _YIUIBase.YIUI.JSONArrValImpl = function (options) {
    var Return = {
      getValue: function getValue(arr, id) {
        return arr.get(id);
      },
      setValue: function setValue(arr, id, value) {
        arr.put(id, value);
      }
    };
    return Return;
  };
})();

_YIUIBase.UI.BaseFuns = function () {
  var funs = {};

  var splitPara = function splitPara(para) {
    if (!para) {
      return null;
    }

    var mapCallback = {},
        len = para.length,
        key = "",
        deep = 0,
        start = 0;

    for (var i = 0; i < len; i++) {
      var c = para.charAt(i);

      if (c == ':' && deep === 0) {
        key = para.substring(start, i).trim();
      } else if (c == ',' && deep === 0) {
        start = ++i;
      } else if (c == '{') {
        if (deep === 0) {
          start = ++i;
        }

        deep++;
      } else if (c == '}') {
        deep--;

        if (deep == 0) {
          mapCallback[key] = para.substring(start, i);
        }
      }
    }

    return mapCallback;
  };

  var processPara = function processPara(cxt) {
    var form = cxt.form;

    if (form != null) {
      var paraCollection = form.getParaCollection();

      if (paraCollection != null) {
        for (var i = 0, len = paraCollection.length; i < len; i++) {
          var para = paraCollection[i],
              value;

          switch (para.type) {
            case _YIUIBase.YIUI.ParameterSourceType.CONST:
              value = para.value;
              break;

            case _YIUIBase.YIUI.ParameterSourceType.FORMULA:
              value = form.eval(para.formula, cxt);
              break;
          }

          form.setPara(para.key, value);
        }
      }
    }
  };

  funs.Confirm =
  /*#__PURE__*/
  function () {
    var _ref = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee47(name, cxt, args) {
      var type, msg, result, mapCallback, form;
      return regeneratorRuntime.wrap(function _callee47$(_context47) {
        while (1) {
          switch (_context47.prev = _context47.next) {
            case 0:
              type = "OK", msg = args[0];

              if (args.length > 1) {
                type = args[1];
              }

              _context47.next = 4;
              return _confirm["default"].current('确认', msg, type);

            case 4:
              result = _context47.sent;
              // //提示框显示样式
              // var type = YIUI.Dialog_MsgType.DEFAULT;
              // if (args.length > 1) {
              //     if (args[1] == "YES_NO") {
              //         type = YIUI.Dialog_MsgType.YES_NO;
              //     } else if (args[1] == "YES_NO_CANCEL") {
              //         type = YIUI.Dialog_MsgType.YES_NO_CANCEL;
              //     }
              // }
              // var options = {
              //     msg: YIUI.TypeConvertor.toString(args[0]),
              //     msgType: type
              // };
              // var dialog = new YIUI.Control.Dialog(options);
              // dialog.render();
              // var optKey = cxt.optKey;
              // if(optKey){
              // 	var option = cxt.form.getOptMap()[optKey];
              // 	if(option) {
              // 		var excpAction = option.opt.excpAction;
              // 		dialog.regExcp(excpAction);
              // 	}
              // }
              mapCallback = {};

              if (args.length > 2) {
                if (args[2]) {
                  mapCallback = splitPara(args[2]);
                }
              }

              if (!mapCallback[result]) {
                _context47.next = 11;
                break;
              }

              form = cxt.form;
              _context47.next = 11;
              return form.eval(mapCallback[result].trim(), cxt, null);

            case 11:
            case "end":
              return _context47.stop();
          }
        }
      }, _callee47);
    }));

    return function (_x100, _x101, _x102) {
      return _ref.apply(this, arguments);
    };
  }();

  funs.HasParent =
  /*#__PURE__*/
  function () {
    var _ref2 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee48(name, cxt, args) {
      return regeneratorRuntime.wrap(function _callee48$(_context48) {
        while (1) {
          switch (_context48.prev = _context48.next) {
            case 0:
              return _context48.abrupt("return", false);

            case 1:
            case "end":
              return _context48.stop();
          }
        }
      }, _callee48);
    }));

    return function (_x103, _x104, _x105) {
      return _ref2.apply(this, arguments);
    };
  }();

  funs.GetFormByType =
  /*#__PURE__*/
  function () {
    var _ref3 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee49(name, cxt, args) {
      var paras, list;
      return regeneratorRuntime.wrap(function _callee49$(_context49) {
        while (1) {
          switch (_context49.prev = _context49.next) {
            case 0:
              paras = {
                cmd: "GetFormByType",
                service: "PureMeta",
                filter: args[0]
              };
              _context49.next = 3;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

            case 3:
              list = _context49.sent;
              return _context49.abrupt("return", list.items);

            case 5:
            case "end":
              return _context49.stop();
          }
        }
      }, _callee49);
    }));

    return function (_x106, _x107, _x108) {
      return _ref3.apply(this, arguments);
    };
  }();

  funs.GetDictValue =
  /*#__PURE__*/
  function () {
    var _ref4 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee50(name, cxt, args) {
      var itemKey, oid, fieldKey;
      return regeneratorRuntime.wrap(function _callee50$(_context50) {
        while (1) {
          switch (_context50.prev = _context50.next) {
            case 0:
              itemKey = args[0];
              oid = args[1];
              fieldKey = args[2];
              _context50.next = 5;
              return _YIUIBase.YIUI.DictService.getDictValue(itemKey, oid, fieldKey);

            case 5:
              return _context50.abrupt("return", _context50.sent);

            case 6:
            case "end":
              return _context50.stop();
          }
        }
      }, _callee50);
    }));

    return function (_x109, _x110, _x111) {
      return _ref4.apply(this, arguments);
    };
  }();

  funs.GetDictOID =
  /*#__PURE__*/
  function () {
    var _ref5 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee51(name, cxt, args) {
      var itemKey, fieldKey, value, oid, item;
      return regeneratorRuntime.wrap(function _callee51$(_context51) {
        while (1) {
          switch (_context51.prev = _context51.next) {
            case 0:
              itemKey = args[0];
              fieldKey = args[1];
              value = args[2];
              oid = 0;
              _context51.next = 6;
              return _YIUIBase.YIUI.DictService.locate(itemKey, fieldKey, value);

            case 6:
              item = _context51.sent;

              if (item) {
                oid = item.oid;
              }

              return _context51.abrupt("return", oid);

            case 9:
            case "end":
              return _context51.stop();
          }
        }
      }, _callee51);
    }));

    return function (_x112, _x113, _x114) {
      return _ref5.apply(this, arguments);
    };
  }();

  funs.ClearSelection = function (name, cxt, args) {
    var controlKey = args[0];
    var form = cxt.form;
    var cmp = form.getComponent(controlKey);

    if (cmp && cmp.type == _YIUIBase.YIUI.CONTROLTYPE.DICTVIEW) {
      cmp.clearSelection();
    }
  };

  funs.CopyNew = function (name, cxt, args) {
    var form = cxt.form;
    var opt = new _YIUIBase.YIUI.CopyNewOpt(form);
    opt.doOpt();
    return true;
  };

  funs.ClearAllRows =
  /*#__PURE__*/
  function () {
    var _ref6 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee52(name, cxt, args) {
      var form, key, comp, grid, row, len, i;
      return regeneratorRuntime.wrap(function _callee52$(_context52) {
        while (1) {
          switch (_context52.prev = _context52.next) {
            case 0:
              form = cxt.form, key = args[0], comp = form.getComponent(key);

              if (!comp) {
                _context52.next = 20;
                break;
              }

              _context52.t0 = comp.type;
              _context52.next = _context52.t0 === _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW ? 5 : _context52.t0 === _YIUIBase.YIUI.CONTROLTYPE.GRID ? 7 : 19;
              break;

            case 5:
              comp.clearAllRows(); // comp.repaint();

              return _context52.abrupt("break", 20);

            case 7:
              grid = comp;
              len = grid.getRowCount();
              i = len - 1;

            case 10:
              if (!(i >= 0)) {
                _context52.next = 18;
                break;
              }

              row = grid.getRowDataAt(i);

              if (!(row.isDetail && row.bookmark !== undefined)) {
                _context52.next = 15;
                break;
              }

              _context52.next = 15;
              return grid.deleteGridRow(i);

            case 15:
              i--;
              _context52.next = 10;
              break;

            case 18:
              return _context52.abrupt("break", 20);

            case 19:
              return _context52.abrupt("break", 20);

            case 20:
            case "end":
              return _context52.stop();
          }
        }
      }, _callee52);
    }));

    return function (_x115, _x116, _x117) {
      return _ref6.apply(this, arguments);
    };
  }();

  funs.ToLongArray = function (name, cxt, args) {
    //参数
    var arrs = [];
    var obj = args[0];

    if (_yesCommon.lodash.isString(obj)) {
      var str = obj.split(",");

      for (var i = 0, len = str.length; i < len; i++) {
        var arr = _YIUIBase.YIUI.TypeConvertor.toLong(str[i]);

        arrs.push(arr);
      }
    } else if (_yesCommon.lodash.isArray(obj)) {
      for (var i = 0, len = obj.length; i < len; i++) {
        var arr;
        var o = obj[i];

        if (o instanceof _YIUIBase.YIUI.ItemData) {
          arr = o.getOID();
        }

        arrs.push(arr);
      }
    }

    return arrs;
  };

  funs.ToJSONArray = function (name, cxt, args) {
    var arrs = [];
    var obj = args[0];

    if (_yesCommon.lodash.isString(obj)) {
      var str = obj.split(",");

      for (var i = 0, len = str.length; i < len; i++) {
        var arr = _yesCommon.lodash.toJSON(str[i]);

        arrs.push(arr);
      }
    } else if (_yesCommon.lodash.isArray(obj)) {
      for (var i = 0, len = obj.length; i < len; i++) {
        var arr;
        var o = obj[i];

        if (o instanceof _YIUIBase.YIUI.ItemData) {
          arr = o.toJSON();
        }

        arrs.push(arr);
      }
    }

    return arrs;
  };

  funs.UICheck =
  /*#__PURE__*/
  function () {
    var _ref7 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee53(name, cxt, args) {
      var form, uiCheckOpt;
      return regeneratorRuntime.wrap(function _callee53$(_context53) {
        while (1) {
          switch (_context53.prev = _context53.next) {
            case 0:
              form = cxt.form, uiCheckOpt = new _YIUIBase.YIUI.UICheckOpt(form);
              _context53.next = 3;
              return uiCheckOpt.doOpt();

            case 3:
            case "end":
              return _context53.stop();
          }
        }
      }, _callee53);
    }));

    return function (_x118, _x119, _x120) {
      return _ref7.apply(this, arguments);
    };
  }();

  funs.ChangePWD = function (name, cxt, args) {
    var operatorID = args[0];
    var password = args[1];
    var newPassword = args[2];
    var rsa = new RSAKey();

    var publicKey = _YIUISvr["default"].SvrMgr.getPublicKey({
      async: false
    });

    rsa.setPublic(publicKey.modulus, publicKey.exponent);
    password = rsa.encrypt(password);
    password = BASE64.encoder(password);
    newPassword = rsa.encrypt(newPassword);
    newPassword = BASE64.encoder(newPassword);
    var paras = {
      service: "SessionRights",
      cmd: "ChangePWD",
      operatorID: operatorID.toString(),
      password: password,
      newPassword: newPassword
    };

    _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);
  };

  funs.New = function (name, cxt, args) {
    var formKey = args[0];
    var target = _YIUIBase.YIUI.FormTarget.NEWTAB;

    if (args.length > 1) {
      target = _YIUIBase.YIUI.FormTarget.parse(args[1]);
    }

    var tsParas;

    if (args.length > 2) {
      tsParas = args[2];
    }

    if (target != _YIUIBase.YIUI.FormTarget.SELF) {
      var params = {
        formKey: formKey,
        cmd: "PureNewForm"
      };

      if (tsParas) {
        tsParas = splitPara(tsParas);

        for (var key in tsParas) {
          var value = cxt.form.eval(tsParas[key], cxt);
          cxt.form.setCallPara(key, value);
        }

        var callParas = cxt.form.getCallParas();
        params.callParas = JSON.stringify(callParas.getMap());
      }

      var success = function success(jsonObj) {
        var form = _YIUIBase.YIUI.FormBuilder.build(jsonObj, target, cxt.form.formID);

        var container = cxt.form.getContainer();

        if (target != _YIUIBase.YIUI.FormTarget.MODAL) {
          if (target == _YIUIBase.YIUI.FormTarget.STACK) {
            form.isStack = true;
          }

          container.build(form);
        }

        form.setOperationState(_YIUIBase.YIUI.Form_OperationState.New);
        form.resetUIStatus(_YIUIBase.YIUI.FormUIStatusMask.ENABLE | _YIUIBase.YIUI.FormUIStatusMask.VISIBLE | _YIUIBase.YIUI.FormUIStatusMask.OPERATION);
        form.getUIProcess().checkAll();
        form.initFirstFocus();
      };

      _YIUISvr["default"].SvrMgr.dealWithPureForm(params, success);
    } else {
      var opt = new _YIUIBase.YIUI.NewOpt(cxt.form, true);
      opt.doOpt();
    }
  };

  funs.Show = function (name, cxt, args) {
    var form = cxt.form;
    var formKey = args[0];
    var container = form.getContainer();
    var target = _YIUIBase.YIUI.FormTarget.NEWTAB;

    if (args.length > 1) {
      target = _YIUIBase.YIUI.FormTarget.parse(args[1]);
    }

    var params = {
      formKey: formKey,
      cmd: "PureShowForm"
    };

    var success = function success(jsonObj) {
      var newForm = _YIUIBase.YIUI.FormBuilder.build(jsonObj, target, form.formID);

      var container = form.getContainer();

      if (target != _YIUIBase.YIUI.FormTarget.MODAL) {
        if (target == _YIUIBase.YIUI.FormTarget.STACK) {
          newForm.isStack = true;
        }

        container.build(newForm);
      }

      newForm.resetUIStatus(_YIUIBase.YIUI.FormUIStatusMask.ENABLE | _YIUIBase.YIUI.FormUIStatusMask.VISIBLE | _YIUIBase.YIUI.FormUIStatusMask.OPERATION);
      newForm.getUIProcess().checkAll();
      newForm.initFirstFocus();
    };

    _YIUISvr["default"].SvrMgr.dealWithPureForm(params, success);

    return true;
  };

  funs.Load =
  /*#__PURE__*/
  function () {
    var _ref8 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee54(name, cxt, args) {
      var form, loadOpt;
      return regeneratorRuntime.wrap(function _callee54$(_context54) {
        while (1) {
          switch (_context54.prev = _context54.next) {
            case 0:
              form = cxt.form;
              loadOpt = new _YIUIBase.YIUI.LoadOpt(form);
              _context54.next = 4;
              return loadOpt.doOpt();

            case 4:
            case "end":
              return _context54.stop();
          }
        }
      }, _callee54);
    }));

    return function (_x121, _x122, _x123) {
      return _ref8.apply(this, arguments);
    };
  }();

  funs.Edit =
  /*#__PURE__*/
  function () {
    var _ref9 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee55(name, cxt, args) {
      var form, editOpt;
      return regeneratorRuntime.wrap(function _callee55$(_context55) {
        while (1) {
          switch (_context55.prev = _context55.next) {
            case 0:
              form = cxt.form;
              editOpt = new _YIUIBase.YIUI.EditOpt(form);
              _context55.next = 4;
              return editOpt.doOpt();

            case 4:
            case "end":
              return _context55.stop();
          }
        }
      }, _callee55);
    }));

    return function (_x124, _x125, _x126) {
      return _ref9.apply(this, arguments);
    };
  }();

  funs.Open = function (name, cxt, args) {
    var form = cxt.form;
    var formKey = args[0],
        OID = args[1];
    var container = form.getContainer();
    var target = _YIUIBase.YIUI.FormTarget.NEWTAB;

    if (args.length > 2) {
      target = _YIUIBase.YIUI.FormTarget.parse(args[2]);
    }

    var tsParas;

    if (args.length > 3) {
      tsParas = args[3];
    }

    var params = {
      formKey: formKey,
      oid: OID.toString(),
      cmd: "PureOpenForm"
    };

    if (tsParas) {
      tsParas = splitPara(tsParas);

      for (var key in tsParas) {
        var value = form.eval(tsParas[key], cxt);
        cxt.form.setCallPara(key, value);
      }

      var callParas = form.getCallParas();
      params.callParas = JSON.stringify(callParas.getMap());
    }

    var success = function success(jsonObj) {
      if (target == _YIUIBase.YIUI.FormTarget.SELF) {
        _YIUIBase.YIUI.UIUtil.replaceForm(form, jsonObj, cxt);

        return;
      }

      var newForm = _YIUIBase.YIUI.FormBuilder.build(jsonObj, target, form.formID);

      if (target != _YIUIBase.YIUI.FormTarget.MODAL) {
        if (target == _YIUIBase.YIUI.FormTarget.STACK) {
          newForm.isStack = true;
        }

        container.build(newForm);
      }

      newForm.setOperationState(_YIUIBase.YIUI.Form_OperationState.Default);
      newForm.pFormID = form.formID;
    };

    _YIUISvr["default"].SvrMgr.dealWithPureForm(params, success);

    return true;
  };

  funs.Save =
  /*#__PURE__*/
  function () {
    var _ref10 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee56(name, cxt, args) {
      var saveOpt;
      return regeneratorRuntime.wrap(function _callee56$(_context56) {
        while (1) {
          switch (_context56.prev = _context56.next) {
            case 0:
              saveOpt = _YIUIBase.YIUI.SaveOpt(cxt.form);
              _context56.next = 3;
              return saveOpt.doOpt();

            case 3:
            case "end":
              return _context56.stop();
          }
        }
      }, _callee56);
    }));

    return function (_x127, _x128, _x129) {
      return _ref10.apply(this, arguments);
    };
  }();

  funs.CalcCondition =
  /*#__PURE__*/
  function () {
    var _ref11 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee57(name, cxt, args) {
      var form, items, component, exps, i, size;
      return regeneratorRuntime.wrap(function _callee57$(_context57) {
        while (1) {
          switch (_context57.prev = _context57.next) {
            case 0:
              form = cxt.form;

              if (form.getUIProcess()) {
                _context57.next = 3;
                break;
              }

              return _context57.abrupt("return");

            case 3:
              items = [];
              exps = form.dependency.calcTree.items;
              i = 0, size = exps.length;

            case 6:
              if (!(i < size)) {
                _context57.next = 14;
                break;
              }

              if (!(exps[i].objectType !== _YIUIBase.YIUI.ExprItem_Type.Item)) {
                _context57.next = 9;
                break;
              }

              return _context57.abrupt("continue", 11);

            case 9:
              component = form.getComponent(exps[i].source);

              if (component && component.condition) {
                items.push(exps[i]);
              }

            case 11:
              i++;
              _context57.next = 6;
              break;

            case 14:
              _context57.next = 16;
              return form.getUIProcess().calcItems(items);

            case 16:
            case "end":
              return _context57.stop();
          }
        }
      }, _callee57);
    }));

    return function (_x130, _x131, _x132) {
      return _ref11.apply(this, arguments);
    };
  }();

  funs.SaveData =
  /*#__PURE__*/
  function () {
    var _ref12 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee58(name, cxt, args) {
      var form, checkUI, opt, paras, formDoc, params, data;
      return regeneratorRuntime.wrap(function _callee58$(_context58) {
        while (1) {
          switch (_context58.prev = _context58.next) {
            case 0:
              form = cxt.form;
              checkUI = true;

              if (args.length > 0) {
                checkUI = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);
              }

              if (!checkUI) {
                _context58.next = 7;
                break;
              }

              opt = new _YIUIBase.YIUI.UICheckOpt(form);
              _context58.next = 7;
              return opt.doOpt();

            case 7:
              paras = form != null ? form.getParas() : null;
              formDoc = form.getDocument();
              formDoc = _YIUIBase.YIUI.DataUtil.toJSONDoc(formDoc, true);
              params = {
                service: "SaveFormData",
                parameters: paras.toJSON(),
                document: _yesCommon.lodash.toJSON(formDoc),
                formKey: form.getFormKey(),
                async: false
              }; // var success = function(doc) {
              //     var dataObj = form.getDataObject();
              //     if(dataObj && dataObj.secondaryType != YIUI.SECONDARYTYPE.DATAOBJECTLIST){
              //         form.setDocument(doc); 
              //         var newOID = doc.oid;
              //         form.getFilterMap().setOID(newOID);
              //     }
              //     form.setOperationState(YIUI.Form_OperationState.Default);
              //     form.showDocument();
              // };

              _context58.next = 13;
              return _YIUISvr["default"].SvrMgr.saveFormData(params, form);

            case 13:
              data = _context58.sent;
              form.setOperationState(_YIUIBase.YIUI.Form_OperationState.Default);
              _context58.next = 17;
              return viewReload(form, data, cxt);

            case 17:
            case "end":
              return _context58.stop();
          }
        }
      }, _callee58);
    }));

    return function (_x133, _x134, _x135) {
      return _ref12.apply(this, arguments);
    };
  }();

  funs.LoadData =
  /*#__PURE__*/
  function () {
    var _ref13 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee59(name, cxt, args) {
      var form, doc, key, dd;
      return regeneratorRuntime.wrap(function _callee59$(_context59) {
        while (1) {
          switch (_context59.prev = _context59.next) {
            case 0:
              form = cxt.form; // let doc = form.getDocument();

              form.setWillShow(true);
              form.resetFilterMap();
              _context59.next = 5;
              return _YIUIBase.YIUI.DocService.loadFormData(form, form.getFilterMap().OID, form.getFilterMap(), form.getCondParas());

            case 5:
              doc = _context59.sent;
              key = "".concat(form.formKey, ".").concat(doc.oid);
              _context59.next = 9;
              return _cache["default"].current.FormDataCache.put(key, {
                key: key,
                data: doc,
                time: Date.now()
              });

            case 9:
              dd = _YIUIBase.YIUI.DataUtil.fromJSONDoc(doc);
              (0, _dispatchers["default"])().dispatch({
                type: 'STOPEVENT'
              });
              _context59.prev = 11;
              form.setDocument(dd);
              _context59.next = 15;
              return form.showDocument();

            case 15:
              _context59.prev = 15;
              (0, _dispatchers["default"])().dispatch({
                type: 'ENABLEEVENT'
              });
              return _context59.finish(15);

            case 18:
            case "end":
              return _context59.stop();
          }
        }
      }, _callee59, null, [[11,, 15, 18]]);
    }));

    return function (_x136, _x137, _x138) {
      return _ref13.apply(this, arguments);
    };
  }();

  funs.Logout =
  /*#__PURE__*/
  function () {
    var _ref14 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee60(name, ctx, args) {
      var result;
      return regeneratorRuntime.wrap(function _callee60$(_context60) {
        while (1) {
          switch (_context60.prev = _context60.next) {
            case 0:
              _context60.next = 2;
              return _YIUISvr["default"].SvrMgr.doLogout();

            case 2:
              result = _context60.sent;

              if (result) {
                (0, _session.clear)();
                (0, _dispatchers["default"])().dispatch((0, _actions.logouted)());
              }

            case 4:
            case "end":
              return _context60.stop();
          }
        }
      }, _callee60);
    }));

    return function (_x139, _x140, _x141) {
      return _ref14.apply(this, arguments);
    };
  }();

  funs.ShowData = function (name, cxt, args) {
    var form = cxt.form; //		form.showDocument();
  };

  funs.DealCondition =
  /*#__PURE__*/
  function () {
    var _ref15 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee61(name, cxt, args) {
      var isParent, form, target, condFomKey, paras, comp, condition, condItem, value, compList, i, cmp, items, filter, filterMap, len;
      return regeneratorRuntime.wrap(function _callee61$(_context61) {
        while (1) {
          switch (_context61.prev = _context61.next) {
            case 0:
              isParent = false;

              if (args.length > 0) {
                isParent = args[0];
              }

              form = cxt.form;
              target = isParent && form.getParentForm() != null ? form.getParentForm() : form;
              condFomKey = null;
              condFomKey = form.getFormKey();
              paras = target.getCondParas();

              if (_yesCommon.lodash.isEmptyObject(paras)) {
                paras = new _conditionparas.ConditionParas();
                target.setCondParas(paras);
              } else {
                paras.clear();
              }

              paras.setCondFormKey(condFomKey);
              comp = null;
              condition = null;
              condItem = null;
              value = null;
              compList = form.getComponentList();
              _context61.t0 = regeneratorRuntime.keys(compList);

            case 15:
              if ((_context61.t1 = _context61.t0()).done) {
                _context61.next = 61;
                break;
              }

              i = _context61.t1.value;
              cmp = compList[i];

              if (!(!cmp.isNull() && cmp.condition)) {
                _context61.next = 48;
                break;
              }

              condition = cmp.condition;
              value = cmp.getValue();

              if (!(cmp.type == _YIUIBase.YIUI.CONTROLTYPE.DATEPICKER)) {
                _context61.next = 25;
                break;
              }

              value = value.getTime();
              _context61.next = 44;
              break;

            case 25:
              if (!(cmp.type == _YIUIBase.YIUI.CONTROLTYPE.NUMBEREDITOR)) {
                _context61.next = 30;
                break;
              }

              if (!(value == 0)) {
                _context61.next = 28;
                break;
              }

              return _context61.abrupt("continue", 15);

            case 28:
              _context61.next = 44;
              break;

            case 30:
              if (!(cmp.type == _YIUIBase.YIUI.CONTROLTYPE.COMPDICT || cmp.type == _YIUIBase.YIUI.CONTROLTYPE.DICT || cmp.type == _YIUIBase.YIUI.CONTROLTYPE.DYNAMICDICT)) {
                _context61.next = 44;
                break;
              }

              if (!cmp.multiSelect) {
                _context61.next = 41;
                break;
              }

              items = value.split(',');

              if (!(value.length == 1 && value[0] == 0)) {
                _context61.next = 35;
                break;
              }

              return _context61.abrupt("continue", 15);

            case 35:
              //多选的情况 添加界面过滤条件。 汇总节点选中， 并非所有子节点 都显示。
              cmp.checkDict();
              filter = cmp.getDictTree().dictFilter;

              if (filter != null) {
                condition.filter = filter;
              }

              value = items.map(function (item) {
                return new _YIUIBase.YIUI.ItemData({
                  oid: item,
                  itemKey: cmp.itemKey
                });
              });
              _context61.next = 42;
              break;

            case 41:
              value = new _YIUIBase.YIUI.ItemData({
                oid: value,
                itemKey: cmp.itemKey
              });

            case 42:
              condition.itemKey = cmp.itemKey;
              condition.stateMask = cmp.stateMask;

            case 44:
              condition.value = value;
              paras.add(condition);
              _context61.next = 59;
              break;

            case 48:
              if (!(cmp.condition && cmp.condition.limitToSource)) {
                _context61.next = 59;
                break;
              }

              condition = cmp.condition;

              if (!(cmp.type == _YIUIBase.YIUI.CONTROLTYPE.COMPDICT || cmp.type == _YIUIBase.YIUI.CONTROLTYPE.DICT || cmp.type == _YIUIBase.YIUI.CONTROLTYPE.DYNAMICDICT)) {
                _context61.next = 59;
                break;
              }

              //多选的情况 添加界面过滤条件。 汇总节点选中， 并非所有子节点 都显示。
              cmp.checkDict();
              filter = cmp.getDictTree().dictFilter;

              if (!(filter == null)) {
                _context61.next = 55;
                break;
              }

              return _context61.abrupt("continue", 15);

            case 55:
              condition.filter = filter;
              condition.itemKey = cmp.itemKey;
              condition.stateMask = cmp.stateMask;
              paras.add(condition);

            case 59:
              _context61.next = 15;
              break;

            case 61:
              filterMap = form.getFilterMap().filterMap;

              for (i = 0, len = filterMap.length; i < len; i++) {
                filterMap[i].startRow = 0;
              }

            case 63:
            case "end":
              return _context61.stop();
          }
        }
      }, _callee61);
    }));

    return function (_x142, _x143, _x144) {
      return _ref15.apply(this, arguments);
    };
  }();

  funs.ReadOnly = function (name, cxt, args) {
    var form = cxt.form;
    var operationState = -1;

    if (form != null) {
      operationState = form.getOperationState();
    }

    return operationState == _YIUIBase.YIUI.Form_OperationState.Default;
  };

  funs.Cancel =
  /*#__PURE__*/
  function () {
    var _ref16 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee62(name, cxt, args) {
      var form;
      return regeneratorRuntime.wrap(function _callee62$(_context62) {
        while (1) {
          switch (_context62.prev = _context62.next) {
            case 0:
              form = cxt.form;

              if (!(form.getOperationState() === _YIUIBase.YIUI.Form_OperationState.New)) {
                _context62.next = 6;
                break;
              }

              _context62.next = 4;
              return funs.Close(name, cxt, []);

            case 4:
              _context62.next = 10;
              break;

            case 6:
              if (!(form.getOperationState() == _YIUIBase.YIUI.Form_OperationState.Edit)) {
                _context62.next = 10;
                break;
              }

              // const doc = await YIUI.DocService.loadFormData(form, form.getFilterMap().OID, form.getFilterMap(), form.getCondParas());
              form.setOperationState(_YIUIBase.YIUI.Form_OperationState.Default);
              _context62.next = 10;
              return funs.LoadData(name, cxt, []);

            case 10:
            case "end":
              return _context62.stop();
          }
        }
      }, _callee62);
    }));

    return function (_x145, _x146, _x147) {
      return _ref16.apply(this, arguments);
    };
  }();

  funs.SetEnable = function (name, cxt, args) {
    var form = cxt.form,
        controlKey = args[0],
        enable = args[1],
        cmp = form.getComponent(controlKey);

    if (cmp != null) {
      cmp.setEnable(enable);
    }
  };

  funs.GetClusterID = function (name, cxt, args) {
    var id = _yesCommon.lodash.cookie("clusterid") || -1;
    return parseInt(id);
  };

  funs.OpenDict =
  /*#__PURE__*/
  function () {
    var _ref17 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee63(name, cxt, args) {
      var pForm, formKey, OID, target, tsParas, key, value, stackContainer;
      return regeneratorRuntime.wrap(function _callee63$(_context63) {
        while (1) {
          switch (_context63.prev = _context63.next) {
            case 0:
              pForm = cxt.form;
              formKey = args[0], OID = _YIUIBase.YIUI.TypeConvertor.toString(args[1]), target = args[2];

              if (args.length > 3) {
                tsParas = args[3];
              }

              if (!tsParas) {
                _context63.next = 14;
                break;
              }

              tsParas = splitPara(tsParas);
              _context63.t0 = regeneratorRuntime.keys(tsParas);

            case 6:
              if ((_context63.t1 = _context63.t0()).done) {
                _context63.next = 14;
                break;
              }

              key = _context63.t1.value;
              _context63.next = 10;
              return pForm.eval(tsParas[key], cxt);

            case 10:
              value = _context63.sent;
              pForm.setCallPara(key, value);
              _context63.next = 6;
              break;

            case 14:
              stackContainer = pForm.getComponent(target);

              if (stackContainer) {
                stackContainer.open(formKey, OID);
              } // var container = pForm.getDefContainer();
              // if (container == null) {
              //     container = pForm.getContainer();
              // }
              // var builder = new YIUI.YIUIBuilder(formKey);
              // builder.setContainer(container);
              // builder.setParentForm(pForm);
              // builder.setOperationState(YIUI.Form_OperationState.Default);
              // builder.newEmpty().then(function(emptyForm){
              //     if(OID > 0){
              //         var filterMap = emptyForm.getFilterMap();
              //         filterMap.setOID(OID);
              //         emptyForm.setOptQueue(new YIUI.OptQueue(new YIUI.LoadOpt(emptyForm)));
              //     }
              //     YIUI.FormParasUtil.processCallParas(pForm, emptyForm);
              //     emptyForm.regEvent(YIUI.FormEvent.ShowDocument, function(){
              //         pForm.resetUIStatus(YIUI.FormUIStatusMask.OPERATION);
              //     });
              //     return builder.builder(emptyForm);
              // });
              // var pForm = cxt.form;
              // var formKey = args[0],
              //     OID = YIUI.TypeConvertor.toString(args[1]);
              // //参数3保留
              // var tsParas;
              // if (args.length > 3) {
              //     tsParas = args[3];
              // }
              // var params = {
              //     async: false,
              //     formKey: formKey,
              //     oid: OID,
              //     cmd: "PureOpenForm"
              // };
              // if (tsParas) {
              //     tsParas = splitPara(tsParas);
              //     for (var key in tsParas) {
              //         var value = pForm.eval(tsParas[key], cxt);
              //         pForm.setCallPara(key, value);
              //     }
              //     var callParas = pForm.getCallParas();
              //     params.callParas = JSON.stringify(callParas.getMap());
              // }
              // var success = function (jsonObj) {
              //     var form = YIUI.FormBuilder.build(jsonObj);
              //     var container = pForm.getDefContainer();
              //     if (!container) {
              //         container = pForm.getContainer();
              //     }
              //     container.build(form, null, pForm.formID);
              //     if (pForm.mergeOpt) {
              //         var expOpts = {
              //             formID: form.formID,
              //             items: form.options
              //         };
              //         pForm.expOpts = expOpts;
              //     }
              //     pForm.getUIProcess().resetUIStatus(YIUI.FormUIStatusMask.OPERATION);
              // };
              // Svr.SvrMgr.dealWithPureForm(params, success);


            case 16:
            case "end":
              return _context63.stop();
          }
        }
      }, _callee63);
    }));

    return function (_x148, _x149, _x150) {
      return _ref17.apply(this, arguments);
    };
  }();

  funs.NewDict = function (name, cxt, args) {
    var formKey = args[0];
    var pForm = cxt.form; //参数2保留

    var tsParas;

    if (args.length > 2) {
      tsParas = args[2];
    }

    var params = {
      async: false,
      formKey: formKey,
      cmd: "PureNewForm"
    };

    if (tsParas) {
      tsParas = splitPara(tsParas);

      for (var key in tsParas) {
        var value = pForm.eval(tsParas[key], cxt);
        pForm.setCallPara(key, value);
      }

      var callParas = pForm.getCallParas();
      params.callParas = JSON.stringify(callParas.getMap());
    }

    var success = function success(jsonObj) {
      var form = _YIUIBase.YIUI.FormBuilder.build(jsonObj, null, pForm.formID);

      var container = pForm.getDefContainer();

      if (!container) {
        container = pForm.getContainer();
      }

      container.build(form);

      if (pForm.mergeOpt) {
        var expOpts = {
          formID: form.formID,
          items: form.options
        };
        pForm.expOpts = expOpts;
      }

      pForm.getUIProcess().resetUIStatus(_YIUIBase.YIUI.FormUIStatusMask.OPERATION);
    };

    var resultJson = _YIUISvr["default"].SvrMgr.dealWithPureForm(params, success);
  }; // var doMap = function (name, cxt, args) {
  //     if (args.length < 2)
  //         YIUI.ViewException.throwException(YIUI.ViewException.NO_KEY_TARGET_BILL);
  //     var form = cxt.form,
  //         mapKey = args[0],
  //         toNewForm = true,
  //         tgFormKey = args[1],
  //         formDoc = form.getDocument(),
  //         copyDoc = formDoc.clone();
  //     var gridMap = form.getGridInfoMap(), tableKey, shadowTableKey, shadowTable;
  //     for (var i = 0, len = gridMap.length; i < len; i++) {
  //         tableKey = gridMap[i].tableKey;
  //         shadowTableKey = YIUI.DataUtil.getShadowTableKey(tableKey);
  //         shadowTable = copyDoc.getByKey(shadowTableKey);
  //         if (shadowTable != undefined || shadowTable != null) {
  //             copyDoc.remove(tableKey);
  //             shadowTable.key = tableKey;
  //         }
  //     }
  //     var doc = YIUI.DataUtil.toJSONDoc(copyDoc);
  //     var params = {
  //         tgFormKey: tgFormKey,
  //         srcFormKey: form.formKey,
  //         srcDoc: $.toJSON(doc),
  //         cmd: "PureMap",
  //         toNewForm: toNewForm,
  //         mapKey: mapKey
  //     };
  //     var resultJson = Svr.SvrMgr.doMapEvent(params);
  //     var newForm = YIUI.FormBuilder.build(resultJson);
  //     var container = form.getContainer();
  //     container.build(newForm);
  //     newForm.pFormID = form.formID;
  //     var mapWorkitemInfo = false;
  //     if (args.length > 2)
  //         mapWorkitemInfo = YIUI.TypeConvertor.toBoolean(args[2]);
  //     if (mapWorkitemInfo) {
  //         var info = form.getSysExpVals(YIUI.BPMConstants.WORKITEM_INFO);
  //         var newDoc = newForm.getDocument();
  //         newDoc.putExpandData(YIUI.BPMKeys.SaveBPMMap_KEY, info.WorkitemID);
  //         newDoc.expDataType[YIUI.BPMKeys.SaveBPMMap_KEY] = YIUI.ExpandDataType.LONG;
  //     }
  //     if (args.length > 3) {
  //         var postFormula = YIUI.TypeConvertor.toString(args[3]);
  //         newForm.eval(postFormula, { form: newForm }, null);
  //     }
  // };
  // var doMap = async function (name, cxt, args) {
  //     var form = cxt.form,
  //         mapKey = YIUI.TypeConvertor.toString(args[0]),
  //         tgtFormKey = YIUI.TypeConvertor.toString(args[1]);
  //     var mapWorkitemInfo = false;
  //     if (args.length > 2) {
  //         mapWorkitemInfo = YIUI.TypeConvertor.toBoolean(args[2]);
  //     }
  //     var postFormula = '';
  //     if (args.length > 3) {
  //         postFormula = YIUI.TypeConvertor.toString(args[3]);
  //     }
  //     await mapData(form, null, tgtFormKey, mapKey, mapWorkitemInfo, postFormula);
  // };
  // var mapData = async function (srcForm, tgtForm, tgtFormKey, mapKey, mapWorkitemInfo, postFormula) {
  //     var srcFormKey = srcForm.formKey,
  //         formDoc = srcForm.getDocument();
  //     var srcDoc = YIUI.DataUtil.toJSONDoc(formDoc, true);
  //     var tgtDoc = tgtForm ? tgtForm.getDocument() : null;
  //     if( tgtDoc ) {
  //         tgtDoc = YIUI.DataUtil.toJSONDoc(tgtDoc);
  //     }
  //     const json = await new YIUI.DataMapService(srcForm).mapData(mapKey, srcFormKey, tgtFormKey, srcDoc, tgtDoc);
  //     await afterDoMap(srcForm, tgtForm, tgtFormKey, json, mapWorkitemInfo, postFormula);
  // };
  // var afterDoMap =async  function(srcForm, tgtForm, tgtFormKey, json, mapWorkitemInfo, postFormula) {
  //     var show = async function (tgtForm) {
  //         var document = YIUI.DataUtil.fromJSONDoc(json.document),
  //             ignoreKeys = json.ignoreKeys,
  //             gridKeys = json.gridKeys;
  //         if( ignoreKeys ) {
  //             tgtForm.setSysExpVals("IgnoreKeys", ignoreKeys);
  //         }
  //         // tgtForm.regEvent(YIUI.FormEvent.ShowDocument, function(){
  //         //     if( gridKeys ) {
  //         //       gridKeys.forEach(function (key) {
  //         //         var grid = tgtForm.getComponent(key);
  //         //         grid && grid.getHandler().dealWithSequence(tgtForm,grid,0);
  //         //       });
  //         //     }
  //         // });
  //         tgtForm.setDocument(document);
  //         // tgtForm.showDocument();
  //         if (mapWorkitemInfo) {
  //             if(srcForm){
  //                 var info = srcForm.getSysExpVals(YIUI.BPMConstants.WORKITEM_INFO);
  //                 if (info != null){
  //                     document.putExpData(YIUI.BPMKeys.SaveBPMMap_KEY, info.WorkitemID);
  //                     document.expDataType[YIUI.BPMKeys.SaveBPMMap_KEY] = YIUI.ExpandDataType.LONG;
  //                 }
  //             }
  //         }
  //         if (postFormula) {
  //             var cxt = new View.Context(tgtForm);
  //             await tgtForm.eval(postFormula, cxt, null);
  //         }
  //     }
  //     const billForm = await BillformStore.createDummyForm(tgtFormKey + json.document.oid, true);
  //     billForm.form.pFormID = srcForm.formID;
  //     const ff = billForm.form;
  //     await show(ff);
  //     await YIUI.FormParasUtil.processCallParas(pForm, ff);
  //     getDispatcher().dispatch(openClientForm(tgtFormKey, json.document.oid, 'NEW'));
  // };
  // funs.Map = async function (name, cxt, args) {
  //     await doMap(name, cxt, args);
  // };
  // funs.MidMap = async function (name, cxt, args) {
  //     await doMap(name, cxt, args);
  // };


  funs.MapEx = function (name, cxt, args) {
    var form = cxt.form;

    var mapKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    if (args.length < 2) {
      _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.MAP_MISS_FORMKEY);
    }

    var formKey = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

    var srcOID = _YIUIBase.YIUI.TypeConvertor.toLong(args[2]);

    var params = {
      formKey: formKey,
      srcOID: srcOID,
      cmd: "PureMapEx",
      mapKey: mapKey
    };

    var resultJson = _YIUISvr["default"].SvrMgr.doMapEvent(params);

    var newForm = _YIUIBase.YIUI.FormBuilder.build(resultJson);

    var container = form.getContainer();
    container.build(newForm);
    newForm.pFormID = form.formID;
    return true;
  };

  funs.AutoMap = function (name, cxt, args) {
    var mapKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var form = cxt.form,
        formKey = form.formKey,
        formDoc = form.getDocument(),
        copyDoc = formDoc.clone();
    var gridMap = form.getGridInfoMap(),
        tableKey,
        shadowTableKey,
        shadowTable;

    for (var i = 0, len = gridMap.length; i < len; i++) {
      tableKey = gridMap[i].tableKey;
      shadowTableKey = _YIUIBase.YIUI.DataUtil.getShadowTableKey(tableKey);
      shadowTable = copyDoc.getByKey(shadowTableKey);

      if (shadowTable != undefined || shadowTable != null) {
        copyDoc.remove(tableKey);
        shadowTable.key = tableKey;
      }
    }

    var doc = _YIUIBase.YIUI.DataUtil.toJSONDoc(copyDoc); // 执行数据映射


    var params = {
      formKey: formKey,
      cmd: "PureAutoMap",
      srcDoc: _yesCommon.lodash.toJSON(doc),
      mapKey: mapKey
    };

    var resultJson = _YIUISvr["default"].SvrMgr.doMapEvent(params);

    return true;
  };

  var mapInForm = function mapInForm(form, mapKey, formKey, toNewForm) {
    var tgFormKey = null,
        toNewForm = toNewForm;
    var formDoc = null;
    var pForm = null;

    if (!toNewForm) {
      pForm = _YIUIBase.YIUI.FormStack.getForm(form.pFormID);
      tgFormKey = pForm.formKey;
    } else {
      tgFormKey = formKey;
    }

    formDoc = form.getDocument();
    var copyDoc = formDoc.clone();
    var gridMap = form.getGridInfoMap(),
        tableKey,
        shadowTableKey,
        shadowTable;

    for (var i = 0, len = gridMap.length; i < len; i++) {
      tableKey = gridMap[i].tableKey;
      shadowTableKey = _YIUIBase.YIUI.DataUtil.getShadowTableKey(tableKey);
      shadowTable = copyDoc.getByKey(shadowTableKey);

      if (shadowTable != undefined || shadowTable != null) {
        copyDoc.remove(tableKey);
        shadowTable.key = tableKey;
      }
    }

    var doc = _YIUIBase.YIUI.DataUtil.toJSONDoc(copyDoc);

    var params = {
      tgFormKey: tgFormKey,
      srcFormKey: form.formKey,
      srcDoc: _yesCommon.lodash.toJSON(doc),
      cmd: "PureMap",
      toNewForm: toNewForm,
      mapKey: mapKey
    };

    if (toNewForm) {
      var resultJson = _YIUISvr["default"].SvrMgr.doMapEvent(params);

      var newForm = _YIUIBase.YIUI.FormBuilder.build(resultJson);

      var container = form.getContainer();
      container.build(newForm);
    } else {
      var result = _YIUISvr["default"].SvrMgr.doMapEvent(params);

      var gridKey = result.key;
      var pGrid = pForm.getComponent(gridKey),
          dataTable;

      if (result.dataTable) {
        dataTable = _YIUIBase.YIUI.DataUtil.fromJSONDataTable(result.dataTable); //            pForm.getDocument().setByKey(pGrid.tableKey, dataTable);
      }

      pGrid.addGridRows(result.data, dataTable, false);
      var calcProcess = new _YIUIBase.YIUI.UICalcProcess(pForm);
      var affectItems = pForm.dependency.calcTree.affectItems,
          item,
          expItems,
          expItem,
          value,
          comp;

      for (var i = 0, len = affectItems.length; i < len; i++) {
        item = affectItems[i];
        var col = pGrid.dataModel.colModel.cells[item.key];
        if (!col) continue;
        expItems = item.expItems;

        for (var j = 0, length = expItems.length; j < length; j++) {
          expItem = expItems[j];
          comp = pForm.getComponent(expItem.source);

          if (comp && comp.type != _YIUIBase.YIUI.CONTROLTYPE.GRID) {
            value = calcProcess.getCalcValue(expItem);
            comp.setValue(value, true, false);
          }
        }
      }
    }
  };

  funs.ViewMap = function (name, cxt, args) {
    if (args.length > 1) {
      var toNewForm = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);

      if (toNewForm) {
        _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.DATA_BINDING_ERROR);
      }
    }

    return _YIUIBase.UI.BaseFuns.MapToForm(name, cxt, args);
  };

  funs.MapToForm = function (name, cxt, args) {
    var form = cxt.form,
        mapKey = args[0];
    mapInForm(form, mapKey, null, false);
    return true;
  };

  funs.HasDataMaped = function (name, cxt, args) {
    var form = cxt.form;
    var doc = form.getDocument();
    if (doc.isNew()) return false;
    var tbls = doc.tbls;

    for (var i = 0, len = tbls.length; i < len; i++) {
      var tbl = tbls[i];
      if (!tbl) continue;
      tbl.beforeFirst();

      while (tbl.next()) {
        var count = tbl.getByKey(_YIUIBase.YIUI.SystemField.MAPCOUNT_SYS_KEY);
        if (count && count > 0) return true;
      }
    }

    return false;
  };

  {
    //ViewBatchProcessFunction
    funs.BatchDeleteData = function (name, cxt, args) {
      var form = cxt.form;

      var objectKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      var tableKey = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

      var OIDFieldKey = _YIUIBase.YIUI.SystemField.OID_SYS_KEY;
      if (args.length > 2) OIDFieldKey = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);

      var OIDList = _YIUIBase.YIUI.BatchUtil.getSelectOIDs(form, tableKey, OIDFieldKey, false);

      var params = {
        cmd: "BatchDeleteData",
        ObjectKey: objectKey,
        OIDListStr: _yesCommon.lodash.toJSON(OIDList)
      };

      _YIUISvr["default"].SvrMgr.delPureData(params);

      return true;
    };

    funs.BatchMap = function (name, cxt, args) {
      var mapKey = args[0];
      var tblKey = args[1];
      var fieldKey = args[2];
      var form = cxt.form;
      var comp;
      var lv = form.getListView(tblKey);
      var grid = form.getGridInfoByTableKey(tblKey);
      var oids = [];

      if (lv) {
        oids = lv.getFieldArray(form, fieldKey);
      } else if (grid) {
        oids = grid.getFieldArray(form, fieldKey);
      }

      var paras = {
        mapKey: mapKey,
        cmd: "PureBatchMap",
        oidList: _yesCommon.lodash.toJSON(oids)
      };

      var resultJson = _YIUISvr["default"].SvrMgr.doMapEvent(paras);
    };
  }

  funs.ShowModal = function (name, cxt, args) {
    var pForm = cxt.form,
        formKey = args[0],
        tsParas = args[1],
        callbackList = args[2];
    var paras = {
      formKey: formKey
    },
        callBack = {};

    if (callbackList) {
      paras.callbackList = callbackList;
      callBack = splitPara(callbackList);
    }

    if (tsParas) {
      tsParas = splitPara(tsParas);

      for (var key in tsParas) {
        var value = pForm.eval(tsParas[key], cxt);
        pForm.setCallPara(key, value);
      }
    } // var container = pForm.getContainer();
    // var builder = new YIUI.YIUIBuilder(formKey);
    // builder.setContainer(container);
    // builder.setParentForm(pForm);
    // builder.setTarget(YIUI.FormTarget.MODAL);


    builder.newEmpty().then(function (emptyForm) {
      _YIUIBase.YIUI.FormParasUtil.processCallParas(pForm, emptyForm);

      for (var o in callBack) {
        emptyForm.regEvent(o, function (opt) {
          pForm.eval(callBack[opt].trim(), cxt, null);
        });
      }

      builder.builder(emptyForm);
    });
    return true;
  };

  funs.doEventCallback = function (name, cxt, args) {
    var form = cxt.form;
    var event = form.getEventCallback(args[0]);
    event.doTask();
  };

  funs.CloseTo = function (name, cxt, args) {
    var form = cxt.form;
    var targetKey = args[0];
    var container = form.getContainer();
    container.closeTo(targetKey);
  };

  funs.Close =
  /*#__PURE__*/
  function () {
    var _ref18 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee64(name, cxt, args) {
      var form;
      return regeneratorRuntime.wrap(function _callee64$(_context64) {
        while (1) {
          switch (_context64.prev = _context64.next) {
            case 0:
              form = cxt.form; // await form.fireClose();

              (0, _dispatchers["default"])().dispatch((0, _actions.navigateBack)());

            case 2:
            case "end":
              return _context64.stop();
          }
        }
      }, _callee64);
    }));

    return function (_x151, _x152, _x153) {
      return _ref18.apply(this, arguments);
    };
  }();

  funs.GetDataObjectKey = function (name, cxt, args) {
    var form = cxt.form;
    var formKey = form.formKey;

    if (args.length > 0) {} else {
      return form.getDataObjectKey();
    }

    if (args[0]) {
      formKey = args[0];
    }

    var params = {
      formKey: formKey,
      cmd: "GetDBKey"
    };

    var dbKey = _YIUISvr["default"].SvrMgr.doDictViewEvent(params);

    return dbKey;
  };

  funs.GetOID = function (name, cxt, args) {
    var form = cxt.form;
    var oid = -1;

    if (form) {
      var doc = form.getDocument();

      if (doc) {
        oid = doc.oid;
      }
    }

    return oid;
  };

  funs.GetInitOperationState = function (name, cxt, args) {
    var form = cxt.form;
    var state = -1;

    if (form) {
      state = form.getInitOperationState();
    }

    return state;
  };

  funs.DeleteData =
  /*#__PURE__*/
  function () {
    var _ref19 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee65(name, cxt, args) {
      var form, formDoc, doc, oid, paras, params;
      return regeneratorRuntime.wrap(function _callee65$(_context65) {
        while (1) {
          switch (_context65.prev = _context65.next) {
            case 0:
              form = cxt.form;
              formDoc = form.getDocument();
              doc = _YIUIBase.YIUI.DataUtil.toJSONDoc(formDoc);
              oid = doc.oid;
              processPara(cxt);
              paras = form != null ? form.getParas() : null;
              params = {
                oid: oid,
                cmd: "DeleteFormData",
                parameters: paras.toJSON(),
                formKey: form.getFormKey()
              };
              _context65.next = 9;
              return _YIUISvr["default"].SvrMgr.delPureData(params);

            case 9:
              form.setInitOperationState(_YIUIBase.YIUI.Form_OperationState.Delete);
              form.setOperationState(_YIUIBase.YIUI.Form_OperationState.Delete);
              (0, _dispatchers["default"])().dispatch((0, _actions.workflowchange)());
              return _context65.abrupt("return", true);

            case 13:
            case "end":
              return _context65.stop();
          }
        }
      }, _callee65);
    }));

    return function (_x154, _x155, _x156) {
      return _ref19.apply(this, arguments);
    };
  }();

  funs.ShowDictView = function (name, cxt, args) {
    var form = cxt.form;
  };

  funs.EnabledDict = function (name, cxt, args) {
    var form = cxt.form;
    var itemKey = args[0];
    var oid = args[1];
    var enable = 1;
    var allChildren = true;

    if (args.length > 2) {
      enable = args[2];
    }

    if (args.length > 3) {
      allChildren = args[3];
    }

    _YIUIBase.YIUI.DictService.enableDict(itemKey, oid, enable, allChildren);

    form.setInitOperationState(_YIUIBase.YIUI.Form_OperationState.Edit);
    return true;
  };

  funs.IsNew = function (name, cxt, args) {
    var form = cxt.form;
    var state = -1;

    if (form) {
      state = form.getOperationState();
    }

    return state == _YIUIBase.YIUI.Form_OperationState.New;
  };

  funs.IsNewOrEdit = function (name, cxt, args) {
    var form = cxt.form;
    var state = -1;

    if (form) {
      state = form.getOperationState();
    }

    return state == _YIUIBase.YIUI.Form_OperationState.Edit || state == _YIUIBase.YIUI.Form_OperationState.New;
  };

  funs.SetValue =
  /*#__PURE__*/
  function () {
    var _ref20 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee66(name, cxt, args) {
      var form, controlKey, value, fireEvent;
      return regeneratorRuntime.wrap(function _callee66$(_context66) {
        while (1) {
          switch (_context66.prev = _context66.next) {
            case 0:
              form = cxt.form;
              controlKey = args[0], value = args[1];
              fireEvent = false;

              if (args.length > 2) {
                if (args[2] === true || args[2] === "true") fireEvent = true;
              }

              _context66.next = 6;
              return form.setComponentValue(controlKey, value, fireEvent);

            case 6:
            case "end":
              return _context66.stop();
          }
        }
      }, _callee66);
    }));

    return function (_x157, _x158, _x159) {
      return _ref20.apply(this, arguments);
    };
  }();

  funs.GetValue =
  /*#__PURE__*/
  function () {
    var _ref21 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee67(name, cxt, args) {
      var form, controlKey;
      return regeneratorRuntime.wrap(function _callee67$(_context67) {
        while (1) {
          switch (_context67.prev = _context67.next) {
            case 0:
              form = cxt.form;
              controlKey = args[0];
              return _context67.abrupt("return", _YIUIBase.YIUI.ExprUtil.getImplValue(form, controlKey, cxt));

            case 3:
            case "end":
              return _context67.stop();
          }
        }
      }, _callee67);
    }));

    return function (_x160, _x161, _x162) {
      return _ref21.apply(this, arguments);
    };
  }();

  funs.CommitValue = function (name, cxt, args) {
    var form = cxt.form;

    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var comp = form.getComponent(key);
    comp.commitValue();
    return true;
  };

  funs.RollbackValue = function (name, cxt, args) {
    var form = cxt.form;

    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var comp = form.getComponent(key);
    comp.rollbackValue();
    return true;
  };

  funs.SumExpand = function (name, cxt, args) {
    var count = new _decimal["default"](0),
        form = cxt.form,
        cellKey = args[0],
        ri = cxt.rowIndex,
        cLoc = form.getCellLocation(cellKey),
        grid = form.getComponent(cLoc.key);
    if (grid == undefined) return count;
    var gr = grid.getRowDataAt(ri),
        isMatch = false,
        cellV;

    if (gr) {
      for (var i = 0, len = gr.cellKeys.length; i < len; i++) {
        if (gr.cellKeys[i] == cellKey) {
          cellV = grid.getValueAt(ri, i);

          if (cellV != null) {
            count = count.plus(cellV);
          }

          isMatch = true;
        } else {
          if (isMatch) break;
        }
      }
    }

    return count;
  };

  funs.Sum = function (name, cxt, args) {
    var form = cxt.form,
        cellKey = args[0].toString(),
        cellLoc = form.getCellLocation(cellKey),
        grid = form.getComponent(cellLoc.key);
    if (grid == undefined) return 0;

    var sumInGrid = function sumInGrid(grid, rowIndex, colIndex) {
      var count = new _decimal["default"](0),
          len = grid.getRowCount(),
          rowData = grid.getRowDataAt(rowIndex),
          value;

      switch (rowData.rowType) {
        case "Fix":
        case "Total":
          var colInfoes = grid.getColInfoByKey(cellKey),
              isMatch = false;

          if (colInfoes != null) {
            for (var j = 0, jlen = colInfoes.length; j < jlen; j++) {
              if (colIndex == colInfoes[j].colIndex) {
                isMatch = true;
                break;
              }
            }
          }

          if (isMatch) {
            for (var i = 0, rlen = grid.getRowCount(); i < rlen; i++) {
              rowData = grid.getRowDataAt(i);

              if (rowData.isDetail && rowData.bookmark != null) {
                value = rowData.data[colIndex][0];

                if (value !== null) {
                  count = count.plus(value);
                }
              }
            }
          } else {
            sumOutGrid(grid, cellKey);
          }

          break;

        case "Group":
          if (rowData.isGroupHead) {
            for (var nextRi = rowIndex + 1; nextRi < len; nextRi++) {
              var nextRD = grid.getRowDataAt(nextRi);

              if (nextRD.rowGroupLevel == rowData.rowGroupLevel) {
                break;
              }

              if (nextRD.isDetail && nextRD.bookmark != null) {
                value = nextRD.data[colIndex][0];

                if (value !== null) {
                  count = count.plus(value);
                }
              }
            }
          } else if (rowData.isGroupTail) {
            for (var preRi = rowIndex - 1; preRi >= 0; preRi--) {
              var preRD = grid.getRowDataAt(preRi);

              if (preRD.rowGroupLevel == rowData.rowGroupLevel) {
                break;
              }

              if (preRD.isDetail && preRD.bookmark != null) {
                value = preRD.data[colIndex][0];

                if (value !== null) {
                  count = count.plus(value);
                }
              }
            }
          }

          break;
      }

      return count;
    };

    var sumOutGrid = function sumOutGrid(grid, cellKey) {
      var rowData,
          colInfoes,
          colIndex,
          count = new _decimal["default"](0),
          value;

      for (var i = 0, len = grid.getRowCount(); i < len; i++) {
        // rowData = grid.getRowDataAt(i);
        if (grid.isRowDetail(i) && grid.getRowBookmark(i) != null) {
          colInfoes = grid.getColInfoByKey(cellKey);
          if (colInfoes == null) continue;

          for (var j = 0, jlen = colInfoes.length; j < jlen; j++) {
            colIndex = colInfoes[j].colIndex; // value = rowData.data[colIndex][0];

            value = grid.getValueAt(i, colIndex);

            if (value !== null) {
              count = count.plus(value);
            }
          }
        }
      }

      return count;
    };

    var targetCellLocation = form.getCellLocation(cxt.target);

    if (targetCellLocation == null || targetCellLocation.key != grid.key) {
      return sumOutGrid(grid, cellKey);
    } else if (cxt.rowIndex !== undefined && cxt.colIndex != undefined && cellLoc !== undefined) {
      return sumInGrid(grid, cxt.rowIndex, cxt.colIndex);
    } else {
      return sumOutGrid(grid, cellKey);
    }
  };

  funs.UpdateView =
  /*#__PURE__*/
  function () {
    var _ref22 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee68(name, cxt, args) {
      return regeneratorRuntime.wrap(function _callee68$(_context68) {
        while (1) {
          switch (_context68.prev = _context68.next) {
            case 0:
              return _context68.abrupt("return");

            case 1:
            case "end":
              return _context68.stop();
          }
        }
      }, _callee68);
    }));

    return function (_x163, _x164, _x165) {
      return _ref22.apply(this, arguments);
    };
  }();

  var updateView = function updateView(cxt) {
    var form = cxt.form;
    var tag = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_VIEW);

    if (tag == _YIUIBase.YIUI.BPMConstants.WORKITEM_VIEW) {
      return;
    }

    var pFormID = form.pFormID;

    if (!pFormID) {
      return true;
    }

    var viewForm = _YIUIBase.YIUI.FormStack.getForm(pFormID);

    if (!viewForm || viewForm.type != _YIUIBase.YIUI.Form_Type.View) {
      return;
    }

    var doc = form.getDocument();
    var viewDoc = viewForm.getDocument();
    var OID = doc.oid;
    var viewFound = false;
    var mTblKey = viewForm.mainTableKey;
    var listView = null;

    if (mTblKey) {
      listView = viewForm.getListView(mTblKey);
    } else {
      listView = viewForm.getListView(0);
    }

    if (!listView) return;
    var rowCount = listView.getRowCount();
    var row = -1,
        colKey;

    for (var cInfo, ci = 0, len = listView.columnInfo.length; ci < len; ci++) {
      cInfo = listView.columnInfo[ci];

      if (cInfo.columnKey == _YIUIBase.YIUI.SystemField.OID_SYS_KEY) {
        colKey = cInfo.key;
        break;
      }
    }

    if (!colKey) return;

    for (var i = 0; i < rowCount; i++) {
      if (listView.getValByKey(i, colKey) == OID) {
        row = i;
        break;
      }
    }

    if (form.getOperationState() == _YIUIBase.YIUI.Form_OperationState.Delete) {
      listView.deleteRow(row);
    } else {
      var data = {};
      var compList = form.getComponentList();
      var columnInfo = listView.columnInfo;

      _yesCommon.lodash.each(columnInfo, function (i, column) {
        var tbl, val; //下拉框 多选下拉框 字典 数值框 日期控件， 因为值和显示值不一致， 先从控件取  ， 之后再填到界面上

        if (column.columnType == _YIUIBase.YIUI.CONTROLTYPE.COMBOBOX || column.columnType == _YIUIBase.YIUI.CONTROLTYPE.CHECKLISTBOX || column.columnType == _YIUIBase.YIUI.CONTROLTYPE.DICT || column.columnType == _YIUIBase.YIUI.CONTROLTYPE.NUMBEREDITOR || column.columnType == _YIUIBase.YIUI.CONTROLTYPE.DATEPICKER || column.columnType == _YIUIBase.YIUI.CONTROLTYPE.TEXTEDITOR) {
          var exists = false;

          _yesCommon.lodash.each(compList, function (i, comp) {
            if (column.columnKey && comp.getMetaObj().columnKey == column.columnKey) {
              var text = comp.getText();
              var value = comp.getValue();

              if (column.columnType == _YIUIBase.YIUI.CONTROLTYPE.NUMBEREDITOR) {
                var settings = {
                  //组大小
                  dGroup: column.groupingSize,
                  //小数位数
                  mDec: column.decScale,
                  //四舍五入方式
                  mRound: column.roundingMode
                };
                text = _YIUIBase.YIUI.DecimalFormat.format(value, settings);
              } else if (column.columnType == _YIUIBase.YIUI.CONTROLTYPE.TEXTEDITOR) {
                text = _YIUIBase.YIUI.TextFormat.format(value, column);
              } else if (column.columnType == _YIUIBase.YIUI.CONTROLTYPE.DATEPICKER) {
                text = _YIUIBase.YIUI.DateFormat.format(value, column);
              }

              data[column.key] = {
                'value': value,
                'caption': text
              };
              exists = true;
              return;
            }
          });

          if (!exists) {
            tbl = doc.getByKey(column.tableKey);

            if (tbl && tbl.first() && column.columnKey) {
              val = tbl.getByKey(column.columnKey);
              data[column.key] = {
                'value': val,
                'caption': val
              };
            }
          }
        } else {
          data[column.key] = {
            'value': column.value || "",
            'caption': column.value || column.caption || ""
          };

          if (column.defaultValue) {
            data[column.key] = {
              'value': column.defaultValue,
              'caption': column.defaultValue
            };
          }

          if (column.defaultFormula) {
            var value = viewForm.eval(column.defaultFormula, {
              form: viewForm,
              rowIndex: row
            }, null);
            data[column.key] = {
              'value': _YIUIBase.YIUI.TypeConvertor.toString(value),
              'caption': _YIUIBase.YIUI.TypeConvertor.toString(value)
            };
          }

          if (column.tableKey) {
            tbl = doc.getByKey(column.tableKey);

            if (tbl && tbl.first() && column.columnKey) {
              val = tbl.getByKey(column.columnKey);

              if (val) {
                data[column.key] = {
                  'value': val,
                  'caption': val
                };
              }
            }
          }
        }
      }); //未找到行


      if (row == -1) {
        listView.addNewRow(data);
      } else {
        _yesCommon.lodash.each(columnInfo, function (i, column) {
          var cv = data[column.key];

          if (cv) {
            var caption = cv.caption;
            listView.setValByKey(row, column.key, caption, true, true);
          }
        });
      }
    }
  };

  funs.GetSelectedValue = function (name, cxt, args) {
    var form = cxt.form;
    var controlKey = args[0];
    var colKey = null;

    if (args.length > 1) {
      colKey = args[1];
    }

    var comp = form.getComponent(controlKey);

    if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.DICTVIEW) {
      var value = comp.getSelectedValue(colKey);
      return value;
    }

    return null;
  };

  funs.RefreshDictView = function (name, cxt, args) {
    var form = cxt.form;
    var controlKey = args[0];
    var itemKey = args[1];
    var oid = args[2];
    var optState = args[3] || _YIUIBase.YIUI.Form_OperationState.Default;
    var dictView = form.getComponent(controlKey);
    var itemData = {};
    itemData.itemKey = itemKey;
    itemData.oid = oid.toString();
    var curID = itemData.itemKey + '_' + itemData.oid;

    if (dictView.type == _YIUIBase.YIUI.CONTROLTYPE.DICTVIEW) {
      switch (optState) {
        case _YIUIBase.YIUI.Form_OperationState.New:
          //目前 不同字典 是通过重新加载 父节点来实现 刷新的。
          if (dictView.isChainDict()) {
            var item = _YIUIBase.YIUI.DictService.getItem(itemData.itemKey, itemData.oid);

            dictView.addNodeByItem(null, item);
          } else {
            //获取当前节点所有父节点
            var parents = _YIUIBase.YIUI.DictService.getParentPath(dictView.itemKey, itemData);

            parents = parents[0];
            var last = parents[parents.length - 1];
            var id = last.itemKey + '_' + last.oid; //父节点

            if (dictView.find(id)) {
              dictView.expandNode(id, true);
            } else {
              for (var i = 0; i < parents.length; i++) {
                id = parents[i].itemKey + '_' + parents[i].oid;
                dictView.expandNode(id);
              }
            }
          } //选中当前节点


          dictView.focusNode(curID);
          break;

        case _YIUIBase.YIUI.Form_OperationState.Delete:
          // 修改previd属性
          dictView.removeNode(itemData.itemKey + '_' + itemData.oid);
          break;

        default:
          if (dictView.isChainDict()) {
            var item = _YIUIBase.YIUI.DictService.getItem(itemData.itemKey, itemData.oid); //dictView.removeNode( itemData.itemKey+'_'+itemData.oid);
            //dictView.addNodeByItem(null,item);


            dictView.refreshNode(itemData.itemKey + '_' + itemData.oid);
          } else {
            //删除原来的节点
            dictView.removeNode(itemData.itemKey + '_' + itemData.oid); //获取当前节点所有父节点

            var parents = _YIUIBase.YIUI.DictService.getParentPath(dictView.itemKey, itemData);

            parents = parents[0];
            var last = parents[parents.length - 1];
            var id = last.itemKey + '_' + last.oid; //父节点

            if (dictView.find(id)) {
              dictView.expandNode(id, true);
            } else {
              for (var i = 0; i < parents.length; i++) {
                id = parents[i].itemKey + '_' + parents[i].oid;
                dictView.expandNode(id);
              }
            }

            var node = dictView.getNode(curID);

            if (node.attr('enable') == 1) {
              for (var i = 0; i < parents.length; i++) {
                id = parents[i].itemKey + '_' + parents[i].oid;
                var parent = dictView.getNode(id);
                parent.attr('enable', 1);
                parent.removeClass('invalid').removeClass('disabled');
              }
            }
          }

          dictView.focusNode(curID);
      }
    }
  };

  funs.IsEnable = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var comp = form.getComponent(key);
    var enable = false;

    if (comp != null) {
      enable = comp.isEnable();
    }

    return enable;
  };

  funs.GetFocusRow = function (name, cxt, args) {
    var form = cxt.form;

    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var com = form.getComponent(key);

    if (com == null) {
      _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPONENT_NOT_EXISTS, key);
    }

    return com.getFocusRowIndex();
  };

  funs.SetFocusRow = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        rowIndex = parseInt(args[1].toString() || 0),
        com = form.getComponent(key);

    if (com && com.type == _YIUIBase.YIUI.CONTROLTYPE.GRID) {
      com.setFocusRowIndex(rowIndex, true);
    }
  };

  funs.GetRowIndex = function (name, cxt, args) {
    if (!cxt) return -1;
    return cxt.rowIndex;
  };

  funs.SetRowIndex = function (name, ctx, args) {
    ctx.rowIndex = args[1];
  };

  funs.GetRowCount = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var comp = form.getComponent(key);
    var includeEmpty = false;

    if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW) {
      return comp.totalRowCount;
    } else if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.GRID) {
      var rowCount = comp.getRowCount();

      if (args.length > 1) {
        includeEmpty = args[1];
      }

      if (!includeEmpty) {
        var data,
            count = 0;

        for (var i = 0, len = rowCount; i < len; i++) {
          data = comp.getRowDataAt(i);

          if (data.isDetail && data.bookmark !== undefined) {
            count++;
          }
        }

        return count;
      }

      return rowCount;
    }

    return -1;
  };

  funs.SetVisible = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];

    var visible = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);

    var comp = form.getComponent(key);

    if (comp) {
      comp.setVisible(visible);
      var ownerCt = comp.ownerCt;

      if (ownerCt.type == _YIUIBase.YIUI.CONTROLTYPE.FLUIDTABLELAYOUTPANEL) {
        ownerCt.reLayout();
      }
    }

    return true;
  };

  funs.IsVisible = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var comp = form.getComponent(key);
    var visible = false;

    if (comp != null) {
      visible = comp.visible;
    }

    return visible;
  };

  funs.SetFocus =
  /*#__PURE__*/
  function () {
    var _ref23 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee69(name, cxt, args) {
      return regeneratorRuntime.wrap(function _callee69$(_context69) {
        while (1) {
          switch (_context69.prev = _context69.next) {
            case 0:
              return _context69.abrupt("return");

            case 1:
            case "end":
              return _context69.stop();
          }
        }
      }, _callee69);
    }));

    return function (_x166, _x167, _x168) {
      return _ref23.apply(this, arguments);
    };
  }();

  funs.IsControlNull = function (name, cxt, args) {
    var key = args[0];
    var form = cxt.form;
    var comp = form.getComponent(key);

    if (comp != null) {
      return comp.isNull();
    }

    var location = form.getCellLocation(key);

    if (location != null) {
      var gridKey = location.key;
      var grid = form.getComponent(gridKey);
      var focusRowIndex = grid.getFocusRowIndex();

      if (focusRowIndex != -1) {
        return grid.isCellNull(focusRowIndex, key);
      }
    }

    return false;
  };

  funs.SetCellValue = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var rowIndex = parseInt(args[1].toString());
    var comp = form.getComponent(key);

    if (rowIndex == undefined || rowIndex == null || rowIndex == -1) {
      rowIndex = cxt.rowIndex;
    }

    if ((rowIndex == undefined || rowIndex == null || rowIndex == -1) && comp) {
      rowIndex = comp.getFocusRowIndex();
    }

    var fireEvent = true;
    var col = args[2].toString();

    if (_yesCommon.lodash.isNumeric(col)) {
      var value = args[3];

      if (args.length > 4) {
        fireEvent = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[4]);
      }

      form.setCellValByIndex(key, rowIndex, parseInt(col), value, fireEvent);
    } else {
      var value = args[3];

      if (args.length > 4) {
        fireEvent = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[4]);
      }

      form.setCellValByKey(key, rowIndex, col, value, fireEvent);
    }

    return true;
  };

  funs.GetCellValue = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var rowIndex = cxt.rowIndex != null ? cxt.rowIndex : args[1].toString();
    var comp = form.getComponent(key);

    if (rowIndex == -1 && comp) {
      rowIndex = comp.getFocusRowIndex();
    }

    var result = null;
    var col = args[2].toString();

    if (_yesCommon.lodash.isNumeric(col)) {
      result = form.getCellValByIndex(key, rowIndex, col);
    } else {
      result = form.getCellValByKey(key, rowIndex, col);
    }

    result = _YIUIBase.YIUI.ExprUtil.convertValue(result);
    return result;
  };

  funs.setCellBackColor = function (key, rowIndex, colIndex, color) {
    return; // var comp = this.getComponent(key);
    // var type = comp == undefined ? -1 : comp.type;
    // switch (type) {
    //     case YIUI.CONTROLTYPE.GRID:
    //         comp.setCellBackColor(rowIndex, colIndex, color);
    //         break;
    //     default:
    //         break;
    // }
  };

  funs.setCellForeColor = function (key, rowIndex, colIndex, color) {
    return; // var comp = this.getComponent(key);
    // var type = comp == undefined ? -1 : comp.type;
    // switch (type) {
    //     case YIUI.CONTROLTYPE.GRID:
    //         comp.setCellForeColor(rowIndex, colIndex, color);
    //         break;
    //     default:
    //         break;
    // }
  };

  funs.GetOperationState = function (name, cxt, args) {
    var form = cxt.form;
    var state = -1;

    if (form) {
      state = form.getOperationState();
    }

    return state;
  };

  funs.ApplyNewOID =
  /*#__PURE__*/
  function () {
    var _ref24 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee70(name, cxt, args) {
      var form, paras, result, oid;
      return regeneratorRuntime.wrap(function _callee70$(_context70) {
        while (1) {
          switch (_context70.prev = _context70.next) {
            case 0:
              form = cxt.form;
              paras = {};
              paras.service = "ApplyNewOID";
              _context70.next = 5;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

            case 5:
              result = _context70.sent;
              oid = result.OID;
              return _context70.abrupt("return", oid);

            case 8:
            case "end":
              return _context70.stop();
          }
        }
      }, _callee70);
    }));

    return function (_x169, _x170, _x171) {
      return _ref24.apply(this, arguments);
    };
  }();

  funs.RunValueChanged = function (name, cxt, args) {
    var form = cxt.form;

    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var cmp = form.getComponent(key);
    var cell = form.getCellLocation(key);
    var valueChange = "";

    if (cmp) {
      valueChange = cmp.valueChanged;
      valueChange && form.eval(valueChange, cxt, null);
    } else if (cell) {
      var compKey = cell.key;
      var grid = form.getComponent(compKey);

      if (grid.type == _YIUIBase.YIUI.CONTROLTYPE.GRID) {
        var rowIndex = grid.getFocusRowIndex();
        if (rowIndex == -1) return;
        var row = grid.dataModel.data[rowIndex],
            cellKey = key,
            editOpt = grid.dataModel.colModel.cells[cellKey],
            meatRow = grid.metaRowInfo.rows[row.metaRowIndex];
        var colIndex = grid.getColInfoByKey(cellKey)[0].colIndex; //			    form.uiProcess.doCellValueChanged(grid, rowIndex, colIndex, cellKey);

        _YIUIBase.YIUI.GridSumUtil.evalAffectSum(form, grid, rowIndex, colIndex);

        var cellCEvent = meatRow.cells[colIndex].valueChanged;

        if (cellCEvent !== undefined && cellCEvent.length > 0) {
          form.eval(_yesCommon.lodash.trim(cellCEvent), {
            form: form,
            rowIndex: rowIndex
          }, null);
        }
      }
    }

    return true;
  };

  funs.RunClick = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var comp = form.getComponent(key);

    switch (comp.type) {
      case _YIUIBase.YIUI.CONTROLTYPE.BUTTON:
      case _YIUIBase.YIUI.CONTROLTYPE.HYPERLINK:
        var clickContent = comp.clickContent;
        clickContent && form.eval(clickContent, cxt, null);
        break;
    }

    return true;
  };

  funs.RunOpt = function (name, cxt, args) {
    var form = cxt.form;
    var optKey = args[0]; //toolbar

    var optInfo = form.getOptMap()[optKey],
        tbr;

    if (optInfo) {
      var action = optInfo.opt.action;
      action && form.eval(action, cxt, null);
    }

    return true;
  };

  funs.SetOID = function (name, cxt, args) {
    var form = cxt.form;
    var filterMap = form.getFilterMap();

    var OID = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

    filterMap.setOID(OID);
    return true;
  };

  funs.ResetCondition = function (name, cxt, args) {
    var form = cxt.form;
    var compList = form.getComponentList();
    var cmp = null;

    for (var i in compList) {
      cmp = compList[i];

      if (cmp.condition) {
        cmp.setValue("", true, false);
      }
    }

    return true;
  };

  funs.GetStatusItems = function (name, cxt, args) {
    return cxt.form.statusItems;
  };

  funs.GetGroupValue = function (name, cxt, args) {
    var form = cxt.form;

    var groupKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var paraGroups = form.paraGroups;
    var items = null;

    if (paraGroups) {
      items = paraGroups[groupKey];
    }

    return items;
  };

  funs.SetClose = function (name, cxt, args) {
    var form = cxt.form;
    form.showFlag = _YIUIBase.YIUI.FormShowFlag.Close;
    return true;
  };

  funs.GetFormKey =
  /*#__PURE__*/
  function () {
    var _ref25 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee71(name, cxt, args) {
      var form;
      return regeneratorRuntime.wrap(function _callee71$(_context71) {
        while (1) {
          switch (_context71.prev = _context71.next) {
            case 0:
              form = cxt.form;
              return _context71.abrupt("return", form.getFormKey());

            case 2:
            case "end":
              return _context71.stop();
          }
        }
      }, _callee71);
    }));

    return function (_x172, _x173, _x174) {
      return _ref25.apply(this, arguments);
    };
  }();

  funs.GetRelationFormKey = function (name, cxt, args) {
    var formKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var dataObjKey = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

    var paras = {
      cmd: "GetRelationFormKey",
      service: "PureMeta",
      dataObjKey: dataObjKey,
      formKey: formKey
    };

    var key = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

    return key;
  };

  funs.GetFormRelation = function (name, cxt, args) {
    var formKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var paras = {
      cmd: "GetFormRelation",
      service: "PureMeta",
      formKey: formKey
    };

    var sRet = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

    return sRet;
  };

  funs.GetTag = function (name, cxt, args) {
    return cxt.tag;
  };

  funs.GetCellCaption = function (name, cxt, args) {
    var form = cxt.form;

    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var rowIndex = _YIUIBase.YIUI.TypeConvertor.toInt(args[1]);

    var grid = form.getComponent(key); // 从环境中取出表格的当前行

    if (rowIndex == -1) {
      rowIndex = grid.getFocusRowIndex();
    }

    if (rowIndex == -1) {
      return "";
    }

    var cell = null;
    var caption = "";
    var column = args[2];

    if (_yesCommon.lodash.isNumeric(column)) {
      var colIndex = _YIUIBase.YIUI.TypeConvertor.toInt(column);

      cell = grid.getCellDataAt(rowIndex, colIndex);
      caption = cell[1];
    } else if (column instanceof String) {
      var cellKey = column.toString();
      cell = grid.getCellDataByKey(rowIndex, cellKey);
      caption = cell[1];
    }

    return caption;
  };

  funs.GetFocusRow = function (name, cxt, args) {
    var form = cxt.form;

    var gridKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var grid = form.getComponent(gridKey);

    if (grid == null) {
      _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPONENT_NOT_EXISTS);
    }

    return grid.getFocusRowIndex();
  };

  funs.GetFocusColumn = function (name, cxt, args) {
    var form = cxt.form;

    var gridKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var grid = form.getComponent(gridKey);

    if (grid == null) {
      _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPONENT_NOT_EXISTS);
    }

    return grid.getFocusColIndex();
  }; //    funs.StartTimerTask = function (name, cxt, args) {
  //    	
  //    };
  //    
  //    funs.StopTimerTask = function (name, cxt, args) {
  //    	
  //    };
  //    
  //    funs.IsTimerTaskStarted = function (name, cxt, args) {
  //    	
  //    };
  // funs.Format = function (name, cxt, args) {
  //     var value = args[0], formatStr = args[1], result = value;
  //     if (value != null && value instanceof Date) {
  //         result = YIUI.DateFormat.format(value, {isOnlyDate: false});
  //     }
  //     return result;
  // };


  funs.ServerDate = function (name, cxt, args) {
    return (0, _session.getServerDate)();
  };

  funs.ServerDBDate = function (name, cxt, args) {
    var form = cxt.form;
    var paras = {};
    paras.service = "DateService";
    paras.cmd = "ServerDBDate";

    var result = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

    return new Date(result);
  };

  funs.LocalDate = function (name, cxt, args) {
    return new Date();
  };

  funs.GetLocale = function (name, ctx, args) {
    return (0, _session.getLocale)();
  };

  funs.GetStatus = function (name, cxt, args) {
    var form = cxt.form,
        doc = form.getDocument(),
        mtKey = form.mainTableKey,
        mtTable = doc.getByKey(mtKey);
    var status = null;

    if (mtTable && mtTable.getRowCount() > 0) {
      mtTable.first();
      status = mtTable.getByKey("Status");
    }

    return status;
  };

  funs.SetPara = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        value = args[1];
    form.setPara(key, value);
  };

  funs.PushPara = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        value = args[1];
    form.setCallPara(key, value);
  };

  funs.GetPara = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0];
    return form && form.getPara(key);
  };

  funs.Para =
  /*#__PURE__*/
  function () {
    var _ref26 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee72(name, cxt, args) {
      var form, key;
      return regeneratorRuntime.wrap(function _callee72$(_context72) {
        while (1) {
          switch (_context72.prev = _context72.next) {
            case 0:
              form = cxt.form, key = args[0];
              return _context72.abrupt("return", form && form.getPara(key));

            case 2:
            case "end":
              return _context72.stop();
          }
        }
      }, _callee72);
    }));

    return function (_x175, _x176, _x177) {
      return _ref26.apply(this, arguments);
    };
  }();

  funs.GetParentOID = function (name, cxt, args) {
    var form = cxt.form,
        ptForm = _YIUIBase.YIUI.FormStack.getForm(form.pFormID);

    if (ptForm) {
      return ptForm.OID;
    }

    return -1;
  };

  funs.RollData = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        endPeriod = "";

    if (args.length > 1) {
      endPeriod = args[1];
    }

    var paras = {};
    paras.service = "Migration";
    paras.cmd = "RollData";
    paras.dataObjectKey = key;
    paras.endPeriod = endPeriod;

    _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);
  };

  funs.SetRowIndex = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        rowIndex = parseInt(args[1].toString() || 0),
        com = form.getComponent(key);

    if (com && com.type == _YIUIBase.YIUI.CONTROLTYPE.GRID) {
      com.setFocusRowIndex(rowIndex);
    }
  };

  funs.IsEmptyRow = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        rowIndex = parseInt(args[1].toString() || 0),
        com = form.getComponent(key);

    if (com && com.type == _YIUIBase.YIUI.CONTROLTYPE.GRID) {
      if (rowIndex == -1) {
        rowIndex = com.getFocusRowIndex();
      }

      var gr = com.getRowDataAt(rowIndex);
      return _YIUIBase.YIUI.GridUtil.isEmptyRow(gr);
    }

    return false;
  };

  funs.CheckDuplicate = function (name, cxt, args) {
    var form = cxt.form;

    if (args.length > 0) {
      var cellKey = args[0],
          cl = form.getCellLocation(cellKey),
          grid = form.getComponent(cl.key);

      if (grid) {
        var datas = [],
            length = grid.getRowCount(),
            row,
            cv,
            values;

        for (var i = 0; i < length; i++) {
          row = grid.getRowDataAt(i);
          values = [];

          if (row.isDetail && row.bookmark !== undefined) {
            for (var j = 0; j < args.length; j++) {
              cellKey = args[j];
              cl = form.getCellLocation(cellKey);
              cv = grid.getValueAt(i, cl.column);
              values.push(cv);
            }

            datas.push(values);
          }
        }

        var data_A, data_B, data1, data2;

        for (var m = 0; m < datas.length; m++) {
          data_A = datas[m];

          for (var n = m + 1; n < datas.length; n++) {
            data_B = datas[n];

            for (var k = 0; k < data_A.length; k++) {
              data1 = data_A[k];
              data2 = data_B[k];

              var cpType = _YIUIBase.YIUI.UIUtil.getCompareType(data1, data2),
                  isBreak = false;

              switch (cpType) {
                case _YIUICommon.DataType.STRING:
                  isBreak = _YIUIBase.YIUI.TypeConvertor.toString(data1) != _YIUIBase.YIUI.TypeConvertor.toString(data2);
                  break;

                case _YIUICommon.DataType.NUMERIC:
                  isBreak = !_YIUIBase.YIUI.TypeConvertor.toDecimal(data1).equals(_YIUIBase.YIUI.TypeConvertor.toDecimal(data2));
                  break;

                case _YIUICommon.DataType.BOOLEAN:
                  isBreak = _YIUIBase.YIUI.TypeConvertor.toBoolean(data1) != _YIUIBase.YIUI.TypeConvertor.toBoolean(data2);
                  break;

                default:
                  if (data1 == null && data2 != null || data1 != null && data2 == null) {
                    isBreak = true;
                  } else {
                    if (_typeof(data1) == "object" && _typeof(data2) == "object") {
                      isBreak = data1.oid != data2.oid;
                    }
                  }

                  break;
              }

              if (isBreak) break;

              if (k == data_A.length - 1) {
                return true;
              }
            }
          }
        }
      }
    }

    return false;
  };

  funs.InsertRow =
  /*#__PURE__*/
  function () {
    var _ref27 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee73(name, cxt, args) {
      var form, key, rowIndex, comp, newRi, afterCur;
      return regeneratorRuntime.wrap(function _callee73$(_context73) {
        while (1) {
          switch (_context73.prev = _context73.next) {
            case 0:
              form = cxt.form, key = args[0], rowIndex = -1;
              comp = form.getComponent(key);

              if (args.length > 1) {
                rowIndex = parseInt(args[1].toString() || -1, 10);
              }

              newRi = -1, afterCur = true;

              if (!(comp == undefined)) {
                _context73.next = 6;
                break;
              }

              return _context73.abrupt("return", newRi);

            case 6:
              _context73.t0 = comp.type;
              _context73.next = _context73.t0 === _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW ? 9 : _context73.t0 === _YIUIBase.YIUI.CONTROLTYPE.GRID ? 10 : 17;
              break;

            case 9:
              return _context73.abrupt("break", 17);

            case 10:
              if (rowIndex == -1 && afterCur && args.length == 1) {
                rowIndex = comp.getFocusRowIndex();
              }

              _context73.next = 13;
              return comp.addGridRow(null, rowIndex);

            case 13:
              newRi = _context73.sent;
              newRi = comp.getRowIndexByID(newRi.rowID);
              comp.setFocusRowIndex(newRi);
              return _context73.abrupt("break", 17);

            case 17:
              return _context73.abrupt("return", newRi);

            case 18:
            case "end":
              return _context73.stop();
          }
        }
      }, _callee73);
    }));

    return function (_x178, _x179, _x180) {
      return _ref27.apply(this, arguments);
    };
  }();

  funs.DeleteRow =
  /*#__PURE__*/
  function () {
    var _ref28 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee74(name, cxt, args) {
      var form, key, rowIndex, com;
      return regeneratorRuntime.wrap(function _callee74$(_context74) {
        while (1) {
          switch (_context74.prev = _context74.next) {
            case 0:
              form = cxt.form, key = args[0], rowIndex = -1;

              if (args.length > 1) {
                rowIndex = parseInt(args[1].toString() || 0, 10);
              }

              com = form.getComponent(key);

              if (!(com == undefined)) {
                _context74.next = 5;
                break;
              }

              return _context74.abrupt("return");

            case 5:
              _context74.t0 = com.type;
              _context74.next = _context74.t0 === _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW ? 8 : _context74.t0 === _YIUIBase.YIUI.CONTROLTYPE.GRID ? 9 : 13;
              break;

            case 8:
              return _context74.abrupt("break", 13);

            case 9:
              if (rowIndex == -1) {
                rowIndex = com.getFocusRowIndex();
              }

              _context74.next = 12;
              return com.deleteGridRow(rowIndex);

            case 12:
              return _context74.abrupt("break", 13);

            case 13:
              return _context74.abrupt("return", true);

            case 14:
            case "end":
              return _context74.stop();
          }
        }
      }, _callee74);
    }));

    return function (_x181, _x182, _x183) {
      return _ref28.apply(this, arguments);
    };
  }();

  funs.CopyGridRow = function (name, cxt, args) {
    var form = cxt.form,
        gridKey = args[0],
        rowIndex = parseInt(args[1].toString() || 0, 10),
        splitKeys = [],
        splitValues;
    var grid = form.getComponent(gridKey);

    if (grid) {
      if (grid.hasColExpand) return -1;

      if (rowIndex == -1) {
        rowIndex = cxt.rowIndex == undefined ? -1 : cxt.rowIndex;
      }

      if (rowIndex == -1) {
        rowIndex = grid.getFocusRowIndex();
      }

      if (rowIndex == -1) return -1;

      if (args.length > 2) {
        var splitKeyString = args[2],
            size = splitKeyString.split(",").length,
            count = 0;

        for (var j = 0, len = grid.getColumnCount(); j < len; j++) {
          var metaCell = grid.getMetaCellInDetail(j);

          if (splitKeyString.indexOf(metaCell.key) >= 0) {
            splitKeys.push(metaCell.columnKey);
            count++;
          }

          if (count == size) break;
        }
      }

      if (args.length > 3) {
        splitValues = [];

        for (var i = 3; i < args.length; i++) {
          splitValues.push(args[i]);
        }
      }

      var layer = -1,
          dtlRi = grid.metaRowInfo.dtlRowIndex,
          metaDtlRow = grid.metaRowInfo.rows[dtlRi];

      if (metaDtlRow.defaultLayer !== -1) {
        var bookmark = grid.getRowDataAt(rowIndex).bookmark;
        var table = form.getDocument().getByKey(grid.tableKey);
        table.setByBkmk(bookmark);
        layer = parseInt(table.get(table.indexByKey("Layer")), 10);
      }

      return grid.gridHandler.copyRow(form, grid, rowIndex, splitKeys, splitValues, layer);
    }

    return -1;
  };

  funs.SetNewEmptyRow = function (name, cxt, args) {
    var form = cxt.form;

    var gridKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var flag = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);

    var grid = form.getComponent(gridKey);

    if (grid == null) {
      _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPONENT_NOT_EXISTS);
    }

    grid.newEmptyRow = flag;
    grid.removeAutoRowAndGroup();
    grid.setGridEnable(grid.isEnable());
    return true;
  };

  funs.IsInBounds = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var point = args[1];
    var map = form.getComponent(key);
    map.IsInBounds(point);
  };

  funs.ClearMap = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var map = form.getComponent(key);
    map.clear();
  };

  funs.SetDriveRoute = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var map = form.getComponent(key);
    var start = args[1];
    var end = args[2];
    var waypoints = args[3];
    map.setDriveRoute(start, end, waypoints);
  };

  funs.DrawMarker = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var map = form.getComponent(key);

    var marker = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

    map.drawMarker(marker);
  };

  funs.DrawPolyline = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var map = form.getComponent(key);

    var polyline = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

    map.drawPolyline(polyline);
  };

  funs.DrawPolygon = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var map = form.getComponent(key);

    var polygon = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

    map.drawPolygon(polygon);
  };

  funs.GetMapInfo = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var map = form.getComponent(key);
    var mapInfo = map.getMapInfo();
    return mapInfo;
  };

  funs.ShowMapInfo = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var map = form.getComponent(key);

    var mapInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

    map.showMapInfo(mapInfo);
  };

  funs.BPMMap = function (name, cxt, args) {
    if (args.length < 2) _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.NO_KEY_TARGET_BILL);
    var form = cxt.form,
        mapKey = args[0],
        toNewForm = true,
        tgFormKey = args[1];
    var formDoc = null;
    var pForm = null;
    formDoc = form.getDocument();

    var doc = _YIUIBase.YIUI.DataUtil.toJSONDoc(formDoc);

    var tableKey = form.mapGrids[mapKey];
    var gridMap = form.getGridInfoMap(),
        grid,
        gMap,
        oids = [];

    for (var i = 0, len = gridMap.length; i < len; i++) {
      gMap = gridMap[i];
      grid = form.getComponent(gMap.key);

      if (gMap.tableKey == tableKey) {
        oids = grid.getFieldArray(form, _YIUIBase.YIUI.SystemField.OID_SYS_KEY);
        break;
      }
    }

    var params = {
      tgFormKey: tgFormKey,
      srcFormKey: form.formKey,
      srcDoc: _yesCommon.lodash.toJSON(doc),
      oidList: _yesCommon.lodash.toJSON(oids),
      cmd: "PureMap",
      toNewForm: toNewForm,
      mapKey: mapKey
    };

    var resultJson = _YIUISvr["default"].SvrMgr.doMapEvent(params);

    var newForm = _YIUIBase.YIUI.FormBuilder.build(resultJson);

    var info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
    var WID = info.WorkitemID;
    var newDoc = newForm.getDocument();
    var expKey = "SaveBPMMap";
    newDoc.expData[expKey] = WID;
    newDoc.expDataType[expKey] = _YIUIBase.YIUI.ExpandDataType.LONG;
    var container = form.getContainer();
    container.build(newForm);
    return true;
  };

  funs.GetFormCaption = function (name, cxt, args) {
    var form = cxt.form;
    return form.getFormCaption();
  };

  funs.GetFormAbbrCaption = function (name, cxt, args) {
    var form = cxt.form;
    return form.getAbbrCaption();
  };

  funs.StatusValue = function (name, cxt, args) {
    var form = cxt.form;
    var statusKey = args[0];
    var status = form.statusItems;

    for (var i = 0, len = status.length; i < len; i++) {
      if (status[i].key == statusKey) {
        return status[i].value;
      }
    }

    return null;
  };

  funs.SetResult = function (name, cxt, args) {
    var form = cxt.form;
    var result = args[0];
    form.setResult(result);
    return true;
  };

  funs.GetResult = function (name, cxt, args) {
    var form = cxt.form;
    var result = form.getResult();
    return result;
  };

  funs.GetCaption = function (name, cxt, args) {
    var caption = "";
    var key = args[0];
    var form = cxt.form;
    var comp = form.getComponent(key);

    if (comp != null) {
      caption = comp.caption;
    }

    return caption;
  };

  funs.ReplaceTable = function (name, cxt, args) {
    var form = cxt.form;
    var document = form.getDocument();
    var key = args[0];
    var table = args[1];

    if (!(table instanceof _YIUICommon.DataDef.DataTable)) {
      table = _YIUIBase.YIUI.DataUtil.fromJSONDataTable(table);
    }

    document.remove(key);
    document.remove(_YIUIBase.YIUI.DataUtil.getShadowTableKey(key));
    table.key = key;
    document.add(key, table);
    return true;
  };

  funs.ReloadTable = function (name, cxt, args) {
    var tableKey = args[0];
    var form = cxt.form;
    form.reloadTable(tableKey);
  };

  funs.RefreshControl = function (name, cxt, args) {
    var form = cxt.form;
    var key = args[0];
    var comp = form.getComponent(key);
    var type = comp.type;

    switch (type) {
      case _YIUIBase.YIUI.CONTROLTYPE.GRID:
        comp.rootGroupBkmk = [];
        comp.reload();
        break;

      case _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW:
        comp.repaint();
        break;

      default:
        var tableKey = comp.tableKey;
        var table = form.getDocument().getByKey(tableKey);

        if (table != null) {
          var value = _YIUIBase.YIUI.UIUtil.getCompValue(comp, table);

          comp.setValue(value, false, false);
        }

        break;
    }

    return true;
  };

  funs.GetOperator =
  /*#__PURE__*/
  function () {
    var _ref29 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee75(name, cxt, args) {
      var userinfo;
      return regeneratorRuntime.wrap(function _callee75$(_context75) {
        while (1) {
          switch (_context75.prev = _context75.next) {
            case 0:
              _context75.next = 2;
              return (0, _session.getUserInfo)();

            case 2:
              userinfo = _context75.sent;
              return _context75.abrupt("return", userinfo.id);

            case 4:
            case "end":
              return _context75.stop();
          }
        }
      }, _callee75);
    }));

    return function (_x184, _x185, _x186) {
      return _ref29.apply(this, arguments);
    };
  }();

  funs.DBUpdate =
  /*#__PURE__*/
  function () {
    var _ref30 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee76(name, cxt, args) {
      var form, SQL, values, i, len, paras, params, result;
      return regeneratorRuntime.wrap(function _callee76$(_context76) {
        while (1) {
          switch (_context76.prev = _context76.next) {
            case 0:
              form = cxt.form;
              SQL = args[0];
              values = [];

              for (i = 1, len = args.length; i < len; i++) {
                values.push(args[i]);
              }

              paras = _YIUIBase.YIUI.YesJSONUtil.toJSONArray(values);
              params = {
                service: "DBUpdate",
                cmd: "DBUpdate",
                sql: SQL,
                paras: _yesCommon.lodash.toJSON(paras)
              };
              _context76.next = 8;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, params);

            case 8:
              result = _context76.sent;
              return _context76.abrupt("return", result);

            case 10:
            case "end":
              return _context76.stop();
          }
        }
      }, _callee76);
    }));

    return function (_x187, _x188, _x189) {
      return _ref30.apply(this, arguments);
    };
  }();

  funs.DBNamedUpdate =
  /*#__PURE__*/
  function () {
    var _ref31 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee77(name, cxt, args) {
      var form, SQLName, values, i, len, paras, params, result;
      return regeneratorRuntime.wrap(function _callee77$(_context77) {
        while (1) {
          switch (_context77.prev = _context77.next) {
            case 0:
              form = cxt.form;
              SQLName = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);
              values = [];

              for (i = 1, len = args.length; i < len; i++) {
                values.push(args[i]);
              }

              paras = _YIUIBase.YIUI.YesJSONUtil.toJSONArray(values);
              params = {
                service: "DBUpdate",
                cmd: "DBNamedUpdate",
                formKey: form.getFormKey(),
                sql: SQLName,
                paras: _yesCommon.lodash.toJSON(paras) // values: $.toJSON(values)

              };
              _context77.next = 8;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, params, true, false);

            case 8:
              result = _context77.sent;
              return _context77.abrupt("return", result);

            case 10:
            case "end":
              return _context77.stop();
          }
        }
      }, _callee77);
    }));

    return function (_x190, _x191, _x192) {
      return _ref31.apply(this, arguments);
    };
  }();

  funs.DBQuery =
  /*#__PURE__*/
  function () {
    var _ref32 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee78(name, cxt, args) {
      var form, SQL, values, i, sqlParas, paras, table;
      return regeneratorRuntime.wrap(function _callee78$(_context78) {
        while (1) {
          switch (_context78.prev = _context78.next) {
            case 0:
              form = cxt.form;
              SQL = args[0];
              values = [];

              for (i = 1; i < args.length; i++) {
                values.push(args[i]);
              }

              sqlParas = _YIUIBase.YIUI.YesJSONUtil.toJSONArray(values);
              paras = {
                service: "DBQuery",
                cmd: "DBQuery",
                sql: SQL,
                paras: _yesCommon.lodash.toJSON(sqlParas)
              };
              _context78.next = 8;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras, true, false);

            case 8:
              table = _context78.sent;

              if (table) {
                _context78.next = 11;
                break;
              }

              return _context78.abrupt("return", null);

            case 11:
              // var table = YIUI.DataUtil.fromJSONDataTable(tableJson);
              if (table) {
                table.first();
              }

              return _context78.abrupt("return", table);

            case 13:
            case "end":
              return _context78.stop();
          }
        }
      }, _callee78);
    }));

    return function (_x193, _x194, _x195) {
      return _ref32.apply(this, arguments);
    };
  }();

  funs.DBNamedQuery =
  /*#__PURE__*/
  function () {
    var _ref33 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee79(name, cxt, args) {
      var form, sqlName, values, i, len, paras, params, table;
      return regeneratorRuntime.wrap(function _callee79$(_context79) {
        while (1) {
          switch (_context79.prev = _context79.next) {
            case 0:
              form = cxt.form;
              sqlName = args[0];
              values = [];

              for (i = 1, len = args.length; i < len; i++) {
                values.push(args[i]);
              }

              paras = _YIUIBase.YIUI.YesJSONUtil.toJSONArray(values);
              params = {
                service: "DBQuery",
                cmd: "DBNamedQuery",
                sql: sqlName,
                formKey: form.getFormKey(),
                paras: _yesCommon.lodash.toJSON(paras)
              };
              _context79.next = 8;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, params, true, false);

            case 8:
              table = _context79.sent;

              if (table) {
                _context79.next = 11;
                break;
              }

              return _context79.abrupt("return", null);

            case 11:
              // var table = YIUI.DataUtil.fromJSONDataTable(tableJson);
              if (table) {
                table.first();
              }

              return _context79.abrupt("return", table);

            case 13:
            case "end":
              return _context79.stop();
          }
        }
      }, _callee79);
    }));

    return function (_x196, _x197, _x198) {
      return _ref33.apply(this, arguments);
    };
  }();

  funs.DBQueryValue =
  /*#__PURE__*/
  function () {
    var _ref34 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee80(name, cxt, args) {
      var table, obj;
      return regeneratorRuntime.wrap(function _callee80$(_context80) {
        while (1) {
          switch (_context80.prev = _context80.next) {
            case 0:
              _context80.next = 2;
              return funs.DBQuery(name, cxt, args);

            case 2:
              table = _context80.sent;
              obj = null;

              if (table.first()) {
                obj = table.get(0);
              }

              return _context80.abrupt("return", obj);

            case 6:
            case "end":
              return _context80.stop();
          }
        }
      }, _callee80);
    }));

    return function (_x199, _x200, _x201) {
      return _ref34.apply(this, arguments);
    };
  }();

  funs.DBNamedQueryValue =
  /*#__PURE__*/
  function () {
    var _ref35 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee81(name, cxt, args) {
      var table, obj;
      return regeneratorRuntime.wrap(function _callee81$(_context81) {
        while (1) {
          switch (_context81.prev = _context81.next) {
            case 0:
              _context81.next = 2;
              return funs.DBNamedQuery(name, cxt, args);

            case 2:
              table = _context81.sent;
              obj = null;

              if (table.first()) {
                obj = table.get(0);
              }

              return _context81.abrupt("return", obj);

            case 6:
            case "end":
              return _context81.stop();
          }
        }
      }, _callee81);
    }));

    return function (_x202, _x203, _x204) {
      return _ref35.apply(this, arguments);
    };
  }();

  funs.SetColumnCaption = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        columnKey = args[1],
        caption = args[2],
        comp = form.getComponent(key);
    if (comp == undefined) return;

    if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.GRID || comp.type == _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW) {
      comp.setColumnCaption(columnKey, caption);
    }
  };

  funs.SetColumnVisible = function (name, cxt, args) {
    var form = cxt.form,
        key = args[0],
        columnKey = args[1],
        visible = args[2],
        comp = form.getComponent(key);
    if (comp == undefined) return;

    if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.GRID || comp.type == _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW) {
      comp.setColumnVisible(columnKey, visible);
    }
  };

  funs.RepaintGrid = function (name, cxt, args) {
    var form = cxt.form,
        gridKey = args[0],
        grid = form.getComponent(gridKey);
    grid && grid.repaint();
  };

  funs.ReloadGrid =
  /*#__PURE__*/
  function () {
    var _ref36 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee82(name, cxt, args) {
      var form, doc, gridKey, sourceKey, state, grid, tableKey, filterMap, type, filterDetail, dt, table, totalRowCount, gridLoader;
      return regeneratorRuntime.wrap(function _callee82$(_context82) {
        while (1) {
          switch (_context82.prev = _context82.next) {
            case 0:
              form = cxt.form, doc = form.getDocument(), gridKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]), sourceKey = "", state = 0;

              if (!(doc == null)) {
                _context82.next = 3;
                break;
              }

              return _context82.abrupt("return");

            case 3:
              if (args.length > 3) {
                if ("new" == _YIUIBase.YIUI.TypeConvertor.toString(args[3]).toLowerCase()) {
                  state = 1;
                }
              }

              grid = form.getComponent(gridKey);

              if (grid == null) {
                _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPONENT_NOT_EXISTS);
              }

              tableKey = grid.getMetaObj().tableKey;

              if (grid.tableKey) {
                _context82.next = 9;
                break;
              }

              return _context82.abrupt("return");

            case 9:
              // 表的查询条件
              filterMap = form.getFilterMap();
              type = filterMap.getType();
              filterMap.setType(_YIUIBase.YIUI.DocumentType.DETAIL);
              filterDetail = filterMap.getTblFilter(tableKey);

              if (args.length > 2) {
                sourceKey = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);
                filterDetail.setSourceKey(sourceKey);
              }

              _context82.next = 16;
              return _YIUIBase.YIUI.DocService.loadFormData(form, form.getFilterMap().OID, form.getFilterMap(), form.getCondParas());

            case 16:
              dt = _context82.sent;
              doc = _YIUIBase.YIUI.DataUtil.fromJSONDoc(dt);
              table = doc.getByKey(tableKey);
              form.getDocument().remove(tableKey);
              form.getDocument().add(tableKey, table);
              totalRowCount = _YIUIBase.YIUI.TotalRowCountUtil.getRowCount(doc, tableKey);

              _YIUIBase.YIUI.TotalRowCountUtil.setRowCount(form.getDocument(), tableKey, totalRowCount);

              filterMap.setType(type);
              gridLoader = new _YIUIBase.YIUI.ShowGridData(form, grid);
              gridLoader.load(); // grid.load(true);
              // var form = cxt.form, doc = form.getDocument(), gridKey = YIUI.TypeConvertor.toString(args[0]), sourceKey = "", state = 0;
              // if (doc == null) return;
              // if (args.length > 2) {
              //     sourceKey = YIUI.TypeConvertor.toString(args[2]);
              // }
              // if (args.length > 3) {
              //     if ("new" == YIUI.TypeConvertor.toString(args[3]).toLowerCase()) {
              //         state = 1;
              //     }
              // }
              // var grid = form.getComponent(gridKey);
              // if (grid == null) {
              //     YIUI.ViewException.throwException(YIUI.ViewException.COMPONENT_NOT_EXISTS);
              // }
              // var tableKey = grid.tableKey, filterMap = form.filterMap;
              // filterMap.setType(YIUI.DocumentType.DETAIL);
              // var dataTable = doc.getByKey(tableKey);
              // if (dataTable == null) return;
              // var tableFilter = filterMap.getTblFilter(tableKey);
              // tableFilter.SourceKey = sourceKey;
              // var params = {
              //     cmd: "PureLoadData",
              //     oid: form.getDocument().oid,
              //     form: form.toJSON(),
              //     formKey: form.getFormKey(),
              //     parameters: form.getParas().toJSON(),
              //     filterMap: $.toJSON(filterMap),
              //     condition: $.toJSON(form.getCondParas())
              // };
              // var result = Svr.SvrMgr.loadFormData(params);
              // if ($.isEmptyObject(result.form) || $.isEmptyObject(result.document)) return;
              // var newDoc = YIUI.DataUtil.fromJSONDoc(result.document),
              //     newTable = newDoc.getByKey(tableKey);
              // if (state == 1) {
              //     newTable.setNew();
              // }
              // doc.remove(tableKey);
              // doc.add(tableKey, newTable);
              // grid.getHandler().diffFromFormJson(grid, result.form);

            case 26:
            case "end":
              return _context82.stop();
          }
        }
      }, _callee82);
    }));

    return function (_x205, _x206, _x207) {
      return _ref36.apply(this, arguments);
    };
  }();

  funs.SetForeColor = function (name, cxt, args) {
    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var color = "";

    if (args.length > 1) {
      color = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);
    }

    var form = cxt.form;
    var comp = form.getComponent(key);

    if (comp != null) {
      comp.setForeColor(color);
    }
  };

  funs.SetBackColor = function (name, cxt, args) {
    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var color = "";

    if (args.length > 1) {
      color = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);
    }

    var form = cxt.form;
    var comp = form.getComponent(key);

    if (comp != null) {
      comp.setBackColor(color);
    }
  };

  funs.SetCellBackColor = function (name, cxt, args) {
    return true;
  };

  funs.SetCellForeColor = function (name, cxt, args) {
    return true;
  };

  funs.RefreshUIStatus = function (name, cxt, args) {
    cxt.form.resetUIStatus(_YIUIBase.YIUI.FormUIStatusMask.ENABLE | _YIUIBase.YIUI.FormUIStatusMask.VISIBLE | _YIUIBase.YIUI.FormUIStatusMask.OPERATION);
    return true;
  };

  funs.RefreshOperation = function (name, cxt, args) {
    cxt.form.resetUIStatus(_YIUIBase.YIUI.FormUIStatusMask.OPERATION);
    return true;
  };

  funs.GetDocument = function (name, cxt, args) {
    var form = cxt.form;
    return form.getDocument();
  };

  funs.SetDocument = function (name, cxt, args) {
    var form = cxt.form;

    if (args[0] instanceof _YIUICommon.DataDef.Document) {
      var doc = args[0];
      form.setDocument(doc);
      return true;
    } else {
      return false;
    }
  };

  funs.PushPara = function (name, cxt, args) {
    var form = cxt.form;

    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var value = args[1];
    form.setCallPara(key, value);
    return true;
  };

  funs.EvalMidExp =
  /*#__PURE__*/
  function () {
    var _ref37 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee83(name, cxt, args) {
      var form, withDoc, exp, doc, count, newArgs, i, size, formParas, paras;
      return regeneratorRuntime.wrap(function _callee83$(_context83) {
        while (1) {
          switch (_context83.prev = _context83.next) {
            case 0:
              form = cxt.form;
              withDoc = false;
              withDoc = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);
              exp = "";
              exp = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);
              doc = null;

              if (withDoc) {
                doc = form.getDocument();
              }

              count = args.length - 2;

              if (count < 0) {
                count = 0;
              }

              newArgs = [];

              for (i = 2, size = args.length; i < size; ++i) {
                newArgs.push(args[i]);
              }

              doc = _YIUIBase.YIUI.DataUtil.toJSONDoc(doc);
              formParas = form != null ? form.getParas() : null;
              paras = {
                service: "EvalMidExp",
                cmd: "EvalMidExp",
                exp: exp,
                document: _yesCommon.lodash.toJSON(doc),
                paras: _yesCommon.lodash.toJSON(newArgs),
                parameters: formParas.toJSON()
              };
              _context83.next = 16;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

            case 16:
              return _context83.abrupt("return", _context83.sent);

            case 17:
            case "end":
              return _context83.stop();
          }
        }
      }, _callee83);
    }));

    return function (_x208, _x209, _x210) {
      return _ref37.apply(this, arguments);
    };
  }();

  funs.InvokeService =
  /*#__PURE__*/
  function () {
    var _ref38 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee84(name, cxt, args) {
      var form, serviceName, withDoc, refresh, paras, mapParas, map, data, i, para, key, value, doc, result;
      return regeneratorRuntime.wrap(function _callee84$(_context84) {
        while (1) {
          switch (_context84.prev = _context84.next) {
            case 0:
              form = cxt.form;
              serviceName = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);
              withDoc = false;

              if (args.length > 1) {
                withDoc = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);
              }

              refresh = false;

              if (args.length > 2) {
                refresh = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[2]);
              }

              paras = null;
              mapParas = null;

              if (args.length == 4) {
                if (args[3]) {
                  map = {};
                  mapParas = splitPara(args[3]);
                }
              } // console.log(`invokeService arguments:${args}`);


              data = {
                service: "InvokeService",
                extSvrName: serviceName,
                paras: _yesCommon.lodash.toJSON(paras)
              }; // console.log(`invokeService arguments:paras = ${data}`);

              if (!_yesCommon.lodash.isEmptyObject(mapParas)) {
                _context84.next = 15;
                break;
              }

              if (args.length > 3) {
                paras = [];

                for (i = 3; i < args.length; i++) {
                  para = args[i];
                  paras.push(para);
                }

                paras = _YIUIBase.YIUI.YesJSONUtil.toJSONArray(paras);
              }

              data.cmd = "InvokeExtService";
              _context84.next = 26;
              break;

            case 15:
              _context84.t0 = regeneratorRuntime.keys(mapParas);

            case 16:
              if ((_context84.t1 = _context84.t0()).done) {
                _context84.next = 24;
                break;
              }

              key = _context84.t1.value;
              _context84.next = 20;
              return form.eval(mapParas[key], cxt);

            case 20:
              value = _context84.sent;
              map[key] = value;
              _context84.next = 16;
              break;

            case 24:
              paras = _YIUIBase.YIUI.YesJSONUtil.toJSONObject(map);
              data.cmd = "InvokeExtService2";

            case 26:
              data.paras = _yesCommon.lodash.toJSON(paras);
              doc = null;

              if (withDoc) {
                doc = form.getDocument();
                doc = _YIUIBase.YIUI.DataUtil.toJSONDoc(doc);
                data.document = _yesCommon.lodash.toJSON(doc);
              } //返回值为document


              _context84.next = 31;
              return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, data);

            case 31:
              result = _context84.sent;

              if (!(refresh && result instanceof _YIUICommon.DataDef.Document)) {
                _context84.next = 41;
                break;
              }

              // var doc = YIUI.DataUtil.fromJSONDoc(result);
              (0, _dispatchers["default"])().dispatch({
                type: 'STOPEVENT'
              });
              _context84.prev = 34;
              form.setDocument(result);
              _context84.next = 38;
              return form.showDocument();

            case 38:
              _context84.prev = 38;
              (0, _dispatchers["default"])().dispatch({
                type: 'ENABLEEVENT'
              });
              return _context84.finish(38);

            case 41:
              return _context84.abrupt("return", result);

            case 42:
            case "end":
              return _context84.stop();
          }
        }
      }, _callee84, null, [[34,, 38, 41]]);
    }));

    return function (_x211, _x212, _x213) {
      return _ref38.apply(this, arguments);
    };
  }();

  funs.ContainsKey = function (name, cxt, args) {
    var form = cxt.form;

    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var comp = form.getComponent(key);
    if (comp) return true;
    var location = form.getCellLocation(key);
    if (location) return true;
    return false;
  };

  funs.NewJSONObject = function (name, cxt, args) {
    var content = "";

    if (args.length > 0) {
      content = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);
    }

    var obj = null;

    if (content) {
      obj = _yesCommon.lodash.parseJSON(content);
    } else {
      obj = {};
    }

    return obj;
  };

  funs.NewJSONArray = function (name, cxt, args) {
    var content = "";

    if (args.length > 0) {
      content = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);
    }

    var obj = null;

    if (content) {
      obj = new _YIUIBase.YIUI.JSONArray(content);
    } else {
      obj = new _YIUIBase.YIUI.JSONArray();
    }

    return obj;
  };

  funs.SetSessionPara = function (name, cxt, args) {
    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

    var value = args[1]; // var paras = $.parseJSON($.cookie("sessionParas"));

    var paras = (0, _session.getParas)();
    paras[key] = value; // $.cookie("sessionParas", $.toJSON(paras));

    return true;
  };

  funs.GetSessionPara = function (name, cxt, args) {
    var key = _YIUIBase.YIUI.TypeConvertor.toString(args[0]); // var paras = $.parseJSON($.cookie("sessionParas"));


    var paras = (0, _session.getParas)();
    return paras[key];
  };

  funs.ImportExcel = function (name, cxt, args) {
    var needResult = false;

    if (args.length > 0) {
      needResult = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);
    }

    var callback = null;

    if (needResult) {
      callback = function callback() {
        var options = {
          msg: "导入成功！",
          msgType: _YIUIBase.YIUI.Dialog_MsgType.DEFAULT
        };
        var dialog = new _YIUIBase.YIUI.Control.Dialog(options);
        dialog.render();
      };
    }

    var clearOriginalData = true;

    if (args.length > 1) {
      clearOriginalData = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);
    }

    var fileChooser = new _YIUIBase.YIUI.FileChooser();
    var options = {
      clearOriginalData: clearOriginalData
    };
    fileChooser.upload(options, callback);
  };

  funs.Print = function (name, cxt, args) {
    var form = cxt.form;
    var paras = {};
    paras.service = "PrintService";
    paras.cmd = "Print";
    paras.OID = form.OID;
    paras.formKey = form.formKey;
    var reportKey = "";

    if (args.length > 0) {
      reportKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);
    }

    paras.reportKey = reportKey;
    var fillEmptyPrint = false;

    if (args.length > 1) {
      fillEmptyPrint = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);
    }

    paras.fillEmptyPrint = fillEmptyPrint;

    var url = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

    _YIUIBase.YIUI.Print.print(url, form.formKey);
  };

  funs.PrintPreview = function (name, cxt, args) {
    var form = cxt.form;
    var paras = {};
    paras.service = "PrintService";
    paras.cmd = "Print";
    paras.OID = form.OID;
    paras.formKey = form.formKey;
    var reportKey = "";

    if (args.length > 0) {
      reportKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);
    }

    paras.reportKey = reportKey;
    var fillEmptyPrint = false;

    if (args.length > 1) {
      fillEmptyPrint = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);
    }

    paras.fillEmptyPrint = fillEmptyPrint;

    var url = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

    url = url.replace(/\\/g, "/");
    var opts = {
      formKey: form.formKey,
      url: url
    };
    var pv = new _YIUIBase.YIUI.PrintPreview(opts);
  };

  {
    //BPM函数
    funs.GetProcessKey = function (name, cxt, args) {
      var form = cxt.form,
          processKey = "",
          doc = form.getDocument(),
          expData = doc.getExpDataInfo("BPM").data,
          table = _YIUIBase.YIUI.DataUtil.fromJSONDataTable(expData);

      if (table && table.getRowCount() > 0) {
        table.first();
        processKey = table.getByKey("ProcessKey");
      }

      return processKey;
    };

    funs.GetProcessVer = function (name, cxt, args) {
      var form = cxt.form,
          processVer = -1,
          tempVer = -1,
          doc = form.getDocument(),
          expData = doc.getExpDataInfo("BPM").data,
          table = _YIUIBase.YIUI.DataUtil.fromJSONDataTable(expData);

      if (table && table.getRowCount() > 0) {
        table.first();
        tempVer = table.getByKey("Version");

        if (tempVer) {
          processVer = parseInt(tempVer);
        }
      }

      return processVer;
    };

    funs.AddDelegateData = function (name, cxt, args) {
      var index = 0;
      var delegateType = args[index++].toString();
      var srcOperatorID = args[index++].toString();
      var tgtOperatorID = args[index++].toString();
      var objectType = args[index++].toString();
      var objectKey = args[index++];
      var nodeKey = args[index++];
      var StartTime = args[index++];
      var alwaysValid = args[index++];
      var EndTime = alwaysValid ? new Date(0) : args[index++];
      var paras = {
        delegateType: delegateType,
        srcOperatorID: srcOperatorID,
        tgtOperatorID: tgtOperatorID,
        objectType: objectType,
        objectKey: objectKey,
        nodeKey: nodeKey,
        StartTime: StartTime ? StartTime.getTime() : null,
        alwaysValid: alwaysValid,
        EndTime: EndTime ? EndTime.getTime() : null,
        cmd: "PureAddDelegateData",
        service: "PureWebBPM"
      };

      _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);
    };

    funs.DeleteDelegateData = function (name, cxt, args) {
      var paras = {
        delegateID: args[0].toString(),
        cmd: "PureDeleteDelegateData",
        service: "PureWebBPM"
      };

      _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);
    };

    funs.GetActiveWorkitemID =
    /*#__PURE__*/
    function () {
      var _ref39 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee85(name, cxt, args) {
        var info, WID;
        return regeneratorRuntime.wrap(function _callee85$(_context85) {
          while (1) {
            switch (_context85.prev = _context85.next) {
              case 0:
                _context85.next = 2;
                return getActiveWorkitem(cxt, args);

              case 2:
                info = _context85.sent;

                if (!info) {
                  _context85.next = 6;
                  break;
                }

                WID = info.WorkitemID;
                return _context85.abrupt("return", WID);

              case 6:
                return _context85.abrupt("return", null);

              case 7:
              case "end":
                return _context85.stop();
            }
          }
        }, _callee85);
      }));

      return function (_x214, _x215, _x216) {
        return _ref39.apply(this, arguments);
      };
    }();

    funs.GetActiveInstanceID = function (name, cxt, args) {
      var info = getActiveWorkitem(cxt, args);
      var instanceID = info.InstanceID;
      return instanceID;
    };

    funs.GetActiveNodeID = function (name, cxt, args) {
      var nodeID = -1;
      var info = getActiveWorkitem(cxt, args);

      if (info) {
        nodeID = info.NodeID;
      }

      return nodeID;
    };

    var getActiveWorkitem = function getActiveWorkitem(cxt, args) {
      var fromParent = false;

      if (args.length > 0) {
        fromParent = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);
      }

      var form = cxt.form;

      if (fromParent) {
        var pFormID = form.pFormID;
        form = _YIUIBase.YIUI.FormStack.getForm(pFormID);
      }

      var info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO); // if (!info) {
      //     YIUI.BPMException.throwException(YIUI.BPMException.NO_ACTIVE_WORKITEM);
      // }

      return info;
    };

    funs.GetActiveWorkitemFormKey = function (name, cxt, args) {
      var info = getActiveWorkitem(cxt, args);
      var formKey = info.FormKey;
      return formKey;
    };

    funs.RestartInstanceByOID = function (name, cxt, args) {
      var instanceOID = null;

      if (args.length > 0) {
        instanceOID = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);
      } else {
        var form = cxt.form;

        if (form) {
          var doc = form.getDocument();

          if (doc) {
            instanceOID = doc.oid;
          }
        }
      }

      var paras = {};
      paras.service = "PureWebBPM";
      paras.cmd = "PureRestartInstanceByOID";
      paras.formKey = form.formKey;
      paras.oid = form.OID;
      paras.instanceOID = instanceOID;

      var result = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

      _YIUIBase.YIUI.UIUtil.replaceForm(form, result, cxt);
    };

    funs.IsInstanceStarted = function (name, cxt, args) {
      var OID = null;

      if (args.length > 0) {
        OID = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);
      } else {
        var form = cxt.form;

        if (form) {
          var doc = form.getDocument();

          if (doc) {
            OID = doc.oid;
          }
        }
      }

      var paras = {};
      paras.service = "BPM";
      paras.cmd = "IsInstanceStarted";
      paras.OID = OID;

      var json = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

      var started = _YIUIBase.YIUI.TypeConvertor.toBoolean(json.result);

      return started;
    };

    funs.DisableDelegateData = function (name, cxt, args) {
      var form = cxt.form;

      var delegateID = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

      var paras = {};
      paras.service = "BPM";
      paras.cmd = "UpdateDelegateDataState";
      paras.delegateID = delegateID;
      paras.onUse = false;

      _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

      return true;
    };

    funs.SetDelegateDataInUse = function (name, cxt, args) {
      var form = cxt.form;

      var delegateID = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

      var paras = {};
      paras.service = "BPM";
      paras.cmd = "UpdateDelegateDataState";
      paras.delegateID = delegateID;
      paras.onUse = true;

      _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

      return true;
    };

    funs.OpenWorkitem =
    /*#__PURE__*/
    function () {
      var _ref40 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee86(name, cxt, args) {
        var form, onlyOpen, loadInfo;
        return regeneratorRuntime.wrap(function _callee86$(_context86) {
          while (1) {
            switch (_context86.prev = _context86.next) {
              case 0:
                form = cxt.form;
                onlyOpen = false;
                loadInfo = true;

                if (args.length > 1) {
                  onlyOpen = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);
                }

                if (args.length > 2) {
                  loadInfo = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[2]);
                } // const workitemInfo = await YIUI.BPMService.loadWorkitemInfo(args[0].toString());
                // var paras = {
                //     WID: args[0].toString(),
                //     onlyOpen: onlyOpen,
                //     cmd: "PureOpenWorkitem",
                //     service: "PureWebBPM"
                // };
                // var result = await Svr.Request.getSyncData(Svr.SvrMgr.ServletURL, paras);
                // Cache.current.FormCache.put(result.form.key,result.form);
                // result.document.key = `${result.form.key}.${result.document.oid}`;
                // Cache.current.FormDataCache.put(result.document.key,result.document);
                // let formKey = workitemInfo.FormKey;
                // if(workitemInfo.TemplateKey){
                //     formKey = `${formKey}|${workitemInfo.TemplateKey}`;
                // }
                // getDispatcher().dispatch(openform(formKey,workitemInfo.OID));


                (0, _dispatchers["default"])().dispatch((0, _actions.openWorkitem)(args[0], onlyOpen, loadInfo)); // var newForm = YIUI.FormBuilder.build(result);
                // var container = form.getDefContainer();
                // if (container == null) {
                //     container = form.getContainer();
                // }
                // newForm.setContainer(container);
                // newForm.setSysExpVals(YIUI.BPMConstants.WORKITEM_VIEW, YIUI.BPMConstants.WORKITEM_VIEW);
                // newForm.pFormID = form.formID;
                // container.build(newForm);

              case 6:
              case "end":
                return _context86.stop();
            }
          }
        }, _callee86);
      }));

      return function (_x217, _x218, _x219) {
        return _ref40.apply(this, arguments);
      };
    }();

    funs.IsTransit =
    /*#__PURE__*/
    function () {
      var _ref41 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee87(name, cxt, args) {
        var sql, WorkitemID, fromParent, form, pFormID, args1, table, result;
        return regeneratorRuntime.wrap(function _callee87$(_context87) {
          while (1) {
            switch (_context87.prev = _context87.next) {
              case 0:
                sql = "select COUNT(*) as count from BPM_TransitTo where WorkitemID=?";
                WorkitemID = _YIUIBase.YIUI.TypeConvertor.toLong(args[0]);

                if (!(WorkitemID == -1)) {
                  _context87.next = 11;
                  break;
                }

                fromParent = false;

                if (args.length > 1) {
                  fromParent = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);
                }

                form = cxt.form;

                if (fromParent) {
                  pFormID = form.pFormID;
                  form = _YIUIBase.YIUI.FormStack.getForm(pFormID);
                }

                info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);

                if (!(info == null)) {
                  _context87.next = 10;
                  break;
                }

                return _context87.abrupt("return", false);

              case 10:
                WorkitemID = info.WorkitemID;

              case 11:
                args1 = [];
                args1[0] = sql;
                args1[1] = WorkitemID;
                _context87.next = 16;
                return funs.DBQuery(name, cxt, args1);

              case 16:
                table = _context87.sent;
                result = false;

                if (table.first()) {
                  result = _YIUIBase.YIUI.TypeConvertor.toInt(table.get(0)) > 0;
                }

                return _context87.abrupt("return", result);

              case 20:
              case "end":
                return _context87.stop();
            }
          }
        }, _callee87);
      }));

      return function (_x220, _x221, _x222) {
        return _ref41.apply(this, arguments);
      };
    }();

    var CommitWorkitem =
    /*#__PURE__*/
    function () {
      var _ref42 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee88(cxt, args) {
        var form, doc, WID, info, change, options, tsParas, pattern, backSite, backSiteOpt, saveDoc, keepParts, status, allowTransit, srcOperator, tmpBackSite, Opt, flag, newArgs, data, b;
        return regeneratorRuntime.wrap(function _callee88$(_context88) {
          while (1) {
            switch (_context88.prev = _context88.next) {
              case 0:
                form = cxt.form;
                doc = form.getDocument();
                WID = args[0].toString();
                info = null;
                change = {};

                if (WID == -1) {
                  info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
                  WID = info.WorkitemID;
                  change.AuditResult = info.AuditResult;
                  change.UserInfo = info.UserInfo;
                  info.AuditResult = args[1];
                  info.UserInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);
                } else {
                  options = {
                    WorkitemID: args[0],
                    AuditResult: args[1],
                    UserInfo: _YIUIBase.YIUI.TypeConvertor.toString(args[2])
                  };
                  info = new _YIUIBase.YIUI.WorkitemInfo(options);
                } //解析动态驳回和单据保存相关参数


                if (args.length > 3) {
                  tsParas = args[3];
                }

                if (tsParas) {
                  tsParas = splitPara(tsParas);
                }

                backSite = -1, backSiteOpt = -1, saveDoc = false, keepParts = false, status = -1, allowTransit = true, srcOperator = -1;

                if (!tsParas) {
                  _context88.next = 32;
                  break;
                }

                if (tsParas["pattern"]) {
                  pattern = tsParas["pattern"];
                }

                if (!tsParas["backSite"]) {
                  _context88.next = 16;
                  break;
                }

                _context88.next = 14;
                return form.eval(tsParas["backSite"], cxt);

              case 14:
                tmpBackSite = _context88.sent;
                backSite = _YIUIBase.YIUI.TypeConvertor.toInt(tmpBackSite);

              case 16:
                if (!tsParas["backSiteOpt"]) {
                  _context88.next = 22;
                  break;
                }

                _context88.t0 = _YIUIBase.YIUI.TypeConvertor;
                _context88.next = 20;
                return form.eval(tsParas["backSiteOpt"], cxt);

              case 20:
                _context88.t1 = _context88.sent;
                backSiteOpt = _context88.t0.toInt.call(_context88.t0, _context88.t1);

              case 22:
                if (tsParas["saveDoc"]) {
                  saveDoc = _YIUIBase.YIUI.TypeConvertor.toBoolean(tsParas["saveDoc"]);
                }

                if (tsParas["keepParts"]) {
                  keepParts = _YIUIBase.YIUI.TypeConvertor.toBoolean(tsParas["keepParts"]);
                }

                if (tsParas["status"]) {
                  status = _YIUIBase.YIUI.TypeConvertor.toInt(tsParas["status"]);
                }

                if (!tsParas["srcOpt"]) {
                  _context88.next = 31;
                  break;
                }

                _context88.t2 = _YIUIBase.YIUI.TypeConvertor;
                _context88.next = 29;
                return form.eval(tsParas["srcOpt"], cxt);

              case 29:
                _context88.t3 = _context88.sent;
                srcOperator = _context88.t2.toLong.call(_context88.t2, _context88.t3);

              case 31:
                if (tsParas["allowTransit"]) {
                  allowTransit = _YIUIBase.YIUI.TypeConvertor.toBoolean(tsParas["allowTransit"]);
                }

              case 32:
                if (!saveDoc) {
                  _context88.next = 36;
                  break;
                }

                Opt = new _YIUIBase.YIUI.UICheckOpt(form);
                _context88.next = 36;
                return Opt.doOpt();

              case 36:
                if (srcOperator > 0) {
                  change.SrcOperator = info.SrcOperator;
                  info.SrcOperator = srcOperator;
                }

                form.setSysExpVals(_YIUIBase.YIUI.BPMKeys.LOAD_WORKITEM_INFO, true);
                flag = true;
                newArgs = [];
                newArgs[0] = WID; // if (pattern && pattern == "Transit" && !funs.IsTransit(null, cxt, newArgs)) {
                //     flag = false;
                // }
                // if (pattern && flag) {
                //     if (pattern == "Transit") {
                //         change.KeepParts = info.KeepParts;
                //         info.KeepParts = keepParts;
                //         try {
                //             let dt = await YIUI.BPMService.transferToNode(info, saveDoc ? doc : null);
                //             form.setSysExpVals(YIUI.BPMConstants.WORKITEM_INFO, null);
                //             form.hostUIStatusProxy = null;
                //             let a = form.getDocument().getExpData(YIUI.BPMKeys.WORKITEM_INFO);
                //             if (a != null) {
                //                 form.getDocument().rmExpData(YIUI.BPMKeys.WORKITEM_INFO);
                //             }
                //             viewReload(form, dt);
                //             return;
                //         } finally {
                //             Object.assign(info, change);
                //         }
                //     } else if (pattern == "Return") {
                //         if (backSite != -1) {
                //             change.BackSite = info.BackSite;
                //             change.BackSiteOpt = info.BackSiteOpt;
                //             change.Status = info.Status;
                //             info.BackSite = backSite;
                //             info.BackSiteOpt = backSiteOpt;
                //             info.Status = status;
                //             // if(saveDoc) {
                //             // 	YIUI.BPMService.commitWorkitem(info, doc).then(function(data){
                //             // 		form.setSysExpVals(YIUI.BPMKeys.WORKITEM_INFO, null);
                //             // 		var b = form.getDocument().getExpData(YIUI.BPMKeys.WORKITEM_INFO);
                //             // 		if (b != null) {
                //             // 			form.getDocument().rmExpData(YIUI.BPMKeys.WORKITEM_INFO);
                //             // 		}
                //             // 		viewReload(form);
                //             //   });
                //             // } else {
                //             // 	YIUI.BPMService.commitWorkitem(info).then(function(data){
                //             // 		form.setSysExpVals(YIUI.BPMKeys.WORKITEM_INFO, null);
                //             // 		var b = form.getDocument().getExpData(YIUI.BPMKeys.WORKITEM_INFO);
                //             // 		if (b != null) {
                //             // 			form.getDocument().rmExpData(YIUI.BPMKeys.WORKITEM_INFO);
                //             // 		}
                //             // 		viewReload(form);
                //             //   });
                //             // }
                //         } else {
                //             // $.error("驳回位置不可用!");
                //             throw new Error("驳回位置不可用!");
                //         }
                //     }
                // }
                // } else {

                if (pattern) {
                  if (pattern == "Transit") {
                    change.KeepParts = info.KeepParts;
                    change.OperationType = info.OperationType;
                    info.KeepParts = keepParts;
                    info.OperationType = _YIUIBase.YIUI.BPMOperationType.OPT_TRANSIT;
                  } else if (pattern == "Return") {
                    if (backSite != -1) {
                      change.BackSite = info.BackSite;
                      change.BackSiteOpt = info.BackSiteOpt;
                      change.Status = info.Status;
                      change.AllowTransit = info.AllowTransit;
                      change.OperationType = info.OperationType;
                      info.BackSite = backSite;
                      info.BackSiteOpt = backSiteOpt;
                      info.Status = status;
                      info.AllowTransit = allowTransit;
                      info.OperationType = _YIUIBase.YIUI.BPMOperationType.OPT_RETURN;
                    } else {
                      _yesCommon.lodash.error("驳回位置不可用!");
                    }
                  }
                } else {
                  change.OperationType = info.OperationType;
                  info.OperationType = _YIUIBase.YIUI.BPMOperationType.OPT_COMMIT;
                } // }
                // else {
                // 	if(saveDoc) {
                // 		YIUI.BPMService.commitWorkitem(info, doc).then(function(data){
                // 			form.setSysExpVals(YIUI.BPMKeys.WORKITEM_INFO, null);
                // 			var b = form.getDocument().getExpData(YIUI.BPMKeys.WORKITEM_INFO);
                // 			if (b != null) {
                // 				form.getDocument().rmExpData(YIUI.BPMKeys.WORKITEM_INFO);
                // 			}
                // 			viewReload(form);
                //       });
                // 	} else {
                // 		YIUI.BPMService.commitWorkitem(info).then(function(data){
                // 			form.setSysExpVals(YIUI.BPMKeys.WORKITEM_INFO, null);
                // 			var b = form.getDocument().getExpData(YIUI.BPMKeys.WORKITEM_INFO);
                // 			if (b != null) {
                // 				form.getDocument().rmExpData(YIUI.BPMKeys.WORKITEM_INFO);
                // 			}
                // 			viewReload(form);
                //       });
                // 	}
                // }
                // var form = cxt.form;
                // var WID = args[0].toString();
                // var info = null;
                // if (WID == -1) {
                //     info = form.getSysExpVals(YIUI.BPMConstants.WORKITEM_INFO);
                //     WID = info.WorkitemID;
                //     info.AuditResult = args[1];
                //     info.UserInfo = args[2] || '';
                // } else {
                //     info = new YIUI.WorkitemInfo({
                //         AuditResult: args[1],
                //         UserInfo: args[2] || ''
                //     });
                // }


                data = null;
                _context88.prev = 43;
                _context88.next = 46;
                return _YIUIBase.YIUI.BPMService.commitWorkitem(info, saveDoc ? doc : null);

              case 46:
                data = _context88.sent;

              case 47:
                _context88.prev = 47;
                Object.assign(info, change);
                return _context88.finish(47);

              case 50:
                // if (__DEV__) {
                console.log('commitworkitem'); // console.log(data);
                // }

                form.setSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO, null);
                form.hostUIStatusProxy = null;
                b = form.getDocument().getExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);

                if (b != null) {
                  form.getDocument().rmExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO); // form.getDocument().putExpData(YIUI.BPMKeys.WORKITEM_INFO, null);
                }

                _context88.next = 57;
                return viewReload(form);

              case 57:
              case "end":
                return _context88.stop();
            }
          }
        }, _callee88, null, [[43,, 47, 50]]);
      }));

      return function CommitWorkitem(_x223, _x224) {
        return _ref42.apply(this, arguments);
      };
    }();

    funs.AuditWorkitem =
    /*#__PURE__*/
    function () {
      var _ref43 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee89(name, cxt, args) {
        return regeneratorRuntime.wrap(function _callee89$(_context89) {
          while (1) {
            switch (_context89.prev = _context89.next) {
              case 0:
                _context89.next = 2;
                return CommitWorkitem(cxt, args);

              case 2:
              case "end":
                return _context89.stop();
            }
          }
        }, _callee89);
      }));

      return function (_x225, _x226, _x227) {
        return _ref43.apply(this, arguments);
      };
    }();

    funs.CommitWorkitem =
    /*#__PURE__*/
    function () {
      var _ref44 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee90(name, cxt, args) {
        return regeneratorRuntime.wrap(function _callee90$(_context90) {
          while (1) {
            switch (_context90.prev = _context90.next) {
              case 0:
                _context90.next = 2;
                return CommitWorkitem(cxt, args);

              case 2:
              case "end":
                return _context90.stop();
            }
          }
        }, _callee90);
      }));

      return function (_x228, _x229, _x230) {
        return _ref44.apply(this, arguments);
      };
    }();

    funs.BatchCommitWorkitem = function (name, cxt, args) {
      var form = cxt.form;
      var OIDList = null;
      var result = 1;
      var userInfo = "";

      if (args.length == 3) {
        OIDList = [];
        var list = args[0];

        for (var i = 0, len = list.length; i < len; i++) {
          OIDList.push(_YIUIBase.YIUI.TypeConvertor.toLong(list[i]));
        }

        result = _YIUIBase.YIUI.TypeConvertor.toInt(args[1]);
        userInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);
      } else {
        var tableKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

        var OIDFieldKey = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

        OIDList = _YIUIBase.YIUI.BatchUtil.getSelectOIDs(form, tableKey, OIDFieldKey, true);
        result = _YIUIBase.YIUI.TypeConvertor.toInt(args[2]);
        userInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[3]);
      }

      var paras = {
        userInfo: userInfo,
        result: result,
        WIDList: _yesCommon.lodash.toJSON(OIDList),
        cmd: "PureBatchCommitWorkitem",
        service: "PureWebBPM"
      };

      var result = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

      return true;
    };

    funs.BatchCommitWorkitemByWID =
    /*#__PURE__*/
    function () {
      var _ref45 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee91(name, cxt, args) {
        var form, WIDList, result, userInfo, list, i, len, tableKey, WIDFieldKey;
        return regeneratorRuntime.wrap(function _callee91$(_context91) {
          while (1) {
            switch (_context91.prev = _context91.next) {
              case 0:
                // if(args.length < 3) {
                //     throw new YIUI.ViewException(YIUI.ViewException.UNEQUAL_PARAM_NUM,
                //         YIUI.ViewException.formatMessage(YIUI.ViewException.UNEQUAL_PARAM_NUM, name));
                // }
                form = cxt.form;
                WIDList = null;
                result = 1;
                userInfo = "";

                if (args.length == 3) {
                  WIDList = [];
                  list = args[0];

                  if (typeof list == "string") {
                    WIDList = list.split(",");
                  } else {
                    for (i = 0, len = list.length; i < len; i++) {
                      WIDList.push(_YIUIBase.YIUI.TypeConvertor.toLong(list[i]));
                    }
                  }

                  result = _YIUIBase.YIUI.TypeConvertor.toInt(args[1]);
                  userInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);
                } else {
                  tableKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);
                  WIDFieldKey = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);
                  WIDList = _YIUIBase.YIUI.BatchUtil.getSelectOIDs(form, tableKey, WIDFieldKey, true);
                  result = _YIUIBase.YIUI.TypeConvertor.toInt(args[2]);
                  userInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[3]);
                }

                _context91.next = 7;
                return _YIUIBase.YIUI.BPMService.batchCommitWorkitemByWID(WIDList, result, userInfo);

              case 7:
                return _context91.abrupt("return", true);

              case 8:
              case "end":
                return _context91.stop();
            }
          }
        }, _callee91);
      }));

      return function (_x231, _x232, _x233) {
        return _ref45.apply(this, arguments);
      };
    }(); // funs.StartInstance = function (name, cxt, args) {
    //     var form = cxt.form;
    //     var paras = {};
    //     paras.service = "PureWebBPM";
    //     paras.cmd = "PureStartInstance";
    //     paras.formKey = form.formKey;
    //     paras.processKey = args[0];
    //     paras.oid = form.OID;
    //     var result = Svr.Request.getSyncData(Svr.SvrMgr.ServletURL, paras);
    //     viewReload(form, result, cxt);
    // };


    funs.StartInstance =
    /*#__PURE__*/
    function () {
      var _ref46 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee92(name, cxt, args) {
        var form, doc, table, processKey, tsParas, pattern, saveDoc, auditResult, instanceID, data, b, _data, _data2, _data3;

        return regeneratorRuntime.wrap(function _callee92$(_context92) {
          while (1) {
            switch (_context92.prev = _context92.next) {
              case 0:
                form = cxt.form;
                doc = form.getDocument();
                table = doc.getExpData(_YIUIBase.YIUI.BPMKeys.LoadBPM_KEY);

                if (table != null) {
                  table = _YIUIBase.YIUI.DataUtil.fromJSONDataTable(table);
                }

                processKey = args[0];

                if (args.length > 1) {
                  tsParas = args[1];
                }

                if (tsParas) {
                  tsParas = splitPara(tsParas);
                }

                saveDoc = false;
                auditResult = 1;

                if (tsParas) {
                  if (tsParas["pattern"]) {
                    pattern = tsParas["pattern"];
                  }

                  if (tsParas["saveDoc"]) {
                    saveDoc = _YIUIBase.YIUI.TypeConvertor.toBoolean(tsParas["saveDoc"]);
                  }

                  if (tsParas["auditResult"]) {
                    auditResult = _YIUIBase.YIUI.TypeConvertor.toInt(tsParas["auditResult"]);
                  }
                }

                if (!(pattern == "Transit")) {
                  _context92.next = 33;
                  break;
                }

                instanceID = table.getByKey(_YIUIBase.YIUI.BPMConstants.BPM_INSTANCE_ID);

                if (!saveDoc) {
                  _context92.next = 23;
                  break;
                }

                _context92.next = 15;
                return _YIUIBase.YIUI.BPMService.dirStartInstance(instanceID, auditResult, doc);

              case 15:
                data = _context92.sent;
                form.setSysExpVals(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO, null);
                b = form.getDocument().getExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);

                if (b != null) {
                  form.getDocument().rmExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);
                }

                _context92.next = 21;
                return viewReload(form);

              case 21:
                _context92.next = 31;
                break;

              case 23:
                _context92.next = 25;
                return _YIUIBase.YIUI.BPMService.dirStratInstance(instanceID, auditResult);

              case 25:
                _data = _context92.sent;
                form.setSysExpVals(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO, null);
                b = form.getDocument().getExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);

                if (b != null) {
                  form.getDocument().rmExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);
                }

                _context92.next = 31;
                return viewReload(form);

              case 31:
                _context92.next = 52;
                break;

              case 33:
                if (!saveDoc) {
                  _context92.next = 44;
                  break;
                }

                _context92.next = 36;
                return _YIUIBase.YIUI.BPMService.startInstance(form.formKey, form.getOID(), args[0], doc);

              case 36:
                _data2 = _context92.sent;
                form.setSysExpVals(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO, null);
                b = form.getDocument().getExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);

                if (b != null) {
                  form.getDocument().rmExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);
                }

                _context92.next = 42;
                return viewReload(form);

              case 42:
                _context92.next = 52;
                break;

              case 44:
                _context92.next = 46;
                return _YIUIBase.YIUI.BPMService.startInstance(form.formKey, form.getOID(), args[0]);

              case 46:
                _data3 = _context92.sent;
                form.setSysExpVals(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO, null);
                b = form.getDocument().getExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);

                if (b != null) {
                  form.getDocument().rmExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);
                }

                _context92.next = 52;
                return viewReload(form);

              case 52:
              case "end":
                return _context92.stop();
            }
          }
        }, _callee92);
      }));

      return function (_x234, _x235, _x236) {
        return _ref46.apply(this, arguments);
      };
    }();

    funs.RestartInstance =
    /*#__PURE__*/
    function () {
      var _ref47 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee93(name, cxt, args) {
        var form, instanceID, info, paras, result;
        return regeneratorRuntime.wrap(function _callee93$(_context93) {
          while (1) {
            switch (_context93.prev = _context93.next) {
              case 0:
                form = cxt.form;
                instanceID = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

                if (instanceID == -1) {
                  info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
                  instanceID = info.InstanceID;
                }

                paras = {};
                paras.service = "PureWebBPM";
                paras.cmd = "PureRestartInstance";
                paras.formKey = form.formKey;
                paras.oid = form.OID;
                paras.instanceID = instanceID;
                _context93.next = 11;
                return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

              case 11:
                result = _context93.sent;
                _context93.next = 14;
                return viewReload(form, result, cxt);

              case 14:
                return _context93.abrupt("return", true);

              case 15:
              case "end":
                return _context93.stop();
            }
          }
        }, _callee93);
      }));

      return function (_x237, _x238, _x239) {
        return _ref47.apply(this, arguments);
      };
    }();

    funs.EndInstance =
    /*#__PURE__*/
    function () {
      var _ref48 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee94(name, cxt, args) {
        var instanceID, userinfo, form, info, data, doc, b;
        return regeneratorRuntime.wrap(function _callee94$(_context94) {
          while (1) {
            switch (_context94.prev = _context94.next) {
              case 0:
                instanceID = _YIUIBase.YIUI.TypeConvertor.toLong(args[0]);
                userinfo = "";
                if (args.length > 2) userinfo = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);
                form = cxt.form;

                if (instanceID == -1) {
                  info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
                  instanceID = info.InstanceID;
                }

                _context94.next = 7;
                return _YIUIBase.YIUI.BPMService.endInstance(form, instanceID, userinfo);

              case 7:
                data = _context94.sent;
                form.setSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO, null);
                doc = form.getDocument();
                b = doc.getExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);

                if (b != null) {
                  form.getDocument().rmExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);
                }

                _context94.next = 14;
                return viewReload(form);

              case 14:
                return _context94.abrupt("return", true);

              case 15:
              case "end":
                return _context94.stop();
            }
          }
        }, _callee94);
      }));

      return function (_x240, _x241, _x242) {
        return _ref48.apply(this, arguments);
      };
    }();

    funs.KillInstance =
    /*#__PURE__*/
    function () {
      var _ref49 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee95(name, cxt, args) {
        var instanceID, form, info, paras, result;
        return regeneratorRuntime.wrap(function _callee95$(_context95) {
          while (1) {
            switch (_context95.prev = _context95.next) {
              case 0:
                instanceID = args[0].toString();

                if (instanceID == -1) {
                  form = cxt.form;
                  info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
                  instanceID = info.InstanceID;
                }

                paras = {
                  instanceID: instanceID,
                  cmd: "PureKillInstance",
                  service: "PureWebBPM"
                };
                _context95.next = 5;
                return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

              case 5:
                result = _context95.sent;
                _context95.next = 8;
                return viewReload(form, result, cxt);

              case 8:
              case "end":
                return _context95.stop();
            }
          }
        }, _callee95);
      }));

      return function (_x243, _x244, _x245) {
        return _ref49.apply(this, arguments);
      };
    }();

    funs.PauseInstance =
    /*#__PURE__*/
    function () {
      var _ref50 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee96(name, cxt, args) {
        var instanceID, userinfo, form, _info, data;

        return regeneratorRuntime.wrap(function _callee96$(_context96) {
          while (1) {
            switch (_context96.prev = _context96.next) {
              case 0:
                instanceID = _YIUIBase.YIUI.TypeConvertor.toLong(args[0]);
                userinfo = "";
                if (args.length > 2) userinfo = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);

                if (instanceID === -1) {
                  form = cxt.form;
                  _info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
                  instanceID = _info.InstanceID;
                } // if (instanceID == -1) {
                //     var fromParent = false;
                //     if (args.length > 1) {
                //         fromParent = YIUI.TypeConvertor.toBoolean(args[1]);
                //     }
                //     var form = cxt.form;
                //     if (fromParent) {
                //         var pFormID = form.pFormID;
                //         form = YIUI.FormStack.getForm(pFormID);
                //     }
                //     var info = form.getSysExpVals(YIUI.BPMConstants.WORKITEM_INFO);
                //     instanceID = info.InstanceID;
                //     YIUI.BPMService.pauseInstance(form, instanceID, userinfo).then(function(data){
                //         viewReload(form);
                //     });
                // } else {


                _context96.next = 6;
                return _YIUIBase.YIUI.BPMService.pauseInstance(cxt.form, instanceID, userinfo);

              case 6:
                data = _context96.sent;
                _context96.next = 9;
                return viewReload(cxt.form, null, cxt);

              case 9:
                return _context96.abrupt("return", true);

              case 10:
              case "end":
                return _context96.stop();
            }
          }
        }, _callee96);
      }));

      return function (_x246, _x247, _x248) {
        return _ref50.apply(this, arguments);
      };
    }();

    funs.Resume =
    /*#__PURE__*/
    function () {
      var _ref51 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee97(name, cxt, args) {
        var instanceID, userinfo, form, _info2, data;

        return regeneratorRuntime.wrap(function _callee97$(_context97) {
          while (1) {
            switch (_context97.prev = _context97.next) {
              case 0:
                instanceID = _YIUIBase.YIUI.TypeConvertor.toLong(args[0]);
                userinfo = "";
                if (args.length > 2) userinfo = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);

                if (instanceID === -1) {
                  form = cxt.form;
                  _info2 = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
                  instanceID = _info2.InstanceID;
                } // if (instanceID == -1) {
                //     var fromParent = false;
                //     if (args.length > 1) {
                //         fromParent = YIUI.TypeConvertor.toBoolean(args[1]);
                //     }
                //     var form = cxt.form;
                //     if (fromParent) {
                //         var pFormID = form.pFormID;
                //         form = YIUI.FormStack.getForm(pFormID);
                //     }
                //     var info = form.getSysExpVals(YIUI.BPMConstants.WORKITEM_INFO);
                //     instanceID = info.InstanceID;
                //     YIUI.BPMService.resume(form, instanceID, userinfo).then(function (data) {
                //         viewReload(form);
                //     });
                // } else {


                _context97.next = 6;
                return _YIUIBase.YIUI.BPMService.resume(cxt.form, instanceID, userinfo);

              case 6:
                data = _context97.sent;
                _context97.next = 9;
                return viewReload(cxt.form, null, cxt);

              case 9:
                return _context97.abrupt("return", true);

              case 10:
              case "end":
                return _context97.stop();
            }
          }
        }, _callee97);
      }));

      return function (_x249, _x250, _x251) {
        return _ref51.apply(this, arguments);
      };
    }();

    var viewReload =
    /*#__PURE__*/
    function () {
      var _ref52 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee98(form, data, context) {
        var originDocContainVerID, originVerID, originDoc, mtKey, mt;
        return regeneratorRuntime.wrap(function _callee98$(_context98) {
          while (1) {
            switch (_context98.prev = _context98.next) {
              case 0:
                // var container = form.getDefContainer();
                // if (container == null) {
                //     container = form.getContainer();
                // }
                // var tag = form.getSysExpVals(YIUI.BPMConstants.WORKITEM_VIEW);
                // // 如果这个工作项是代办页面打开的，提交工作项之后刷新代办页面
                // if (tag == YIUI.BPMConstants.WORKITEM_VIEW) {
                //     var pFormID = form.pFormID;
                //     var viewForm = YIUI.FormStack.getForm(pFormID);
                //     if (!viewForm || viewForm.type != YIUI.Form_Type.View) {
                //         return;
                //     }
                //     var old = viewForm.getOperationState();
                //     var loadParent = new YIUI.LoadOpt(viewForm);
                //     loadParent.doOpt();
                //     viewForm = YIUI.FormStack.getForm(pFormID);
                //     viewForm.setOperationState(old);
                //     viewForm.showDocument();
                // }
                // 获取载入前数据的版本号
                originDocContainVerID = false;
                originVerID = -1;
                originDoc = form.getDocument();

                if (originDoc) {
                  //MainTable
                  mtKey = form.mainTableKey;
                  mt = originDoc.getByKey(mtKey);

                  if (mt && mt.getRowCount() > 0) {
                    originVerID = mt.getByKey(_YIUIBase.YIUI.SystemField.VERID_SYS_KEY);
                    if (originVerID >= 0) originDocContainVerID = true;
                  }
                } // 原始数据无版本信息，那么就不要执行UpdateView


                if (originDocContainVerID) {
                  _context98.next = 6;
                  break;
                }

                return _context98.abrupt("return");

              case 6:
                if (!data) {
                  _context98.next = 17;
                  break;
                }

                form.setDocument(data);
                (0, _dispatchers["default"])().dispatch({
                  type: 'STOPEVENT'
                });
                _context98.prev = 9;
                _context98.next = 12;
                return form.showDocument();

              case 12:
                _context98.prev = 12;
                (0, _dispatchers["default"])().dispatch({
                  type: 'ENABLEEVENT'
                });
                return _context98.finish(12);

              case 15:
                _context98.next = 18;
                break;

              case 17:
                (0, _dispatchers["default"])().dispatch((0, _actions.reloadForm)(form.getUniqueId()));

              case 18:
                (0, _dispatchers["default"])().dispatch((0, _actions.workflowchange)());

              case 19:
              case "end":
                return _context98.stop();
            }
          }
        }, _callee98, null, [[9,, 12, 15]]);
      }));

      return function viewReload(_x252, _x253, _x254) {
        return _ref52.apply(this, arguments);
      };
    }();

    funs.RollbackToWorkitem =
    /*#__PURE__*/
    function () {
      var _ref53 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee99(name, cxt, args) {
        var form, wid, logicCheck, data, b;
        return regeneratorRuntime.wrap(function _callee99$(_context99) {
          while (1) {
            switch (_context99.prev = _context99.next) {
              case 0:
                form = cxt.form;
                wid = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);
                logicCheck = false;

                if (args.length > 1) {
                  logicCheck = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[1]);
                }

                _context99.next = 6;
                return _YIUIBase.YIUI.BPMService.rollbackToWorkitem(wid, logicCheck);

              case 6:
                data = _context99.sent;
                form.setSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO, null);
                form.hostUIStatusProxy = null;
                b = form.getDocument().getExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO);

                if (b != null) {
                  form.getDocument().putExpData(_YIUIBase.YIUI.BPMKeys.WORKITEM_INFO, null);
                }

                _context99.next = 13;
                return viewReload(form);

              case 13:
                return _context99.abrupt("return", true);

              case 14:
              case "end":
                return _context99.stop();
            }
          }
        }, _callee99);
      }));

      return function (_x255, _x256, _x257) {
        return _ref53.apply(this, arguments);
      };
    }();

    funs.AssignNextNodeParticipator = function (name, cxt, args) {
      var info = getActiveWorkitem(cxt, args);
      var role = false;

      if (args.length > 2) {
        role = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[2]);
      }

      var table = new _YIUICommon.DataDef.DataTable();

      var user_type = _YIUIBase.YIUI.DataUtil.dataType2JavaDataType(_YIUIBase.YIUI.DataType.STRING);

      table.addCol(_YIUIBase.YIUI.PPObject_Type.ColumnType, _YIUIBase.YIUI.DataType.STRING, user_type);
      table.addCol(_YIUIBase.YIUI.PPObject_Type.ColumnInfo, _YIUIBase.YIUI.DataType.STRING, user_type);
      var pp = new _YIUIBase.YIUI.PPObject(table);

      var opStr = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

      table.addRow();
      table.setByKey(_YIUIBase.YIUI.PPObject_Type.ColumnType, role ? _YIUIBase.YIUI.PPObject_Type.DicRole : _YIUIBase.YIUI.PPObject_Type.DicOperator);
      table.setByKey(_YIUIBase.YIUI.PPObject_Type.ColumnInfo, opStr);
      info.NextOpStr = _yesCommon.lodash.toJSON(pp.toJSON());
      return true;
    };

    funs.SetCustomKey = function (name, cxt, args) {
      var customKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]);

      var form = cxt.form;
      var info = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO);
      info.OperationKey = customKey;
      return true;
    };

    funs.BatchStateAction = function (name, cxt, args) {
      var form = cxt.form;

      var processKey = _YIUIBase.YIUI.TypeConvertor.toString(form.getPara(_YIUIBase.YIUI.BatchBPM.BPM_PROCESS_KEY));

      var actionNodeKey = _YIUIBase.YIUI.TypeConvertor.toString(form.getPara(_YIUIBase.YIUI.BatchBPM.BPM_ACTION_NODE_KEY));

      var lv = null;
      var mTblKey = form.mainTableKey;

      if (mTblKey) {
        lv = form.getListView(mTblKey);
      } else {
        lv = form.getListView(0);
      }

      var OIDList = lv.getFieldArray(form, _YIUIBase.YIUI.SystemField.OID_SYS_KEY);

      var result = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);

      var userInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

      var paras = {
        processKey: processKey,
        actionNodeKey: actionNodeKey,
        result: result,
        userInfo: userInfo,
        OIDList: _yesCommon.lodash.toJSON(OIDList),
        cmd: "PureBatchStateAction",
        service: "PureViewMap"
      };

      _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

      return true;
    };

    funs.GetProcessPath = function (name, cxt, args) {
      var OID = null;

      if (args.length > 0) {
        OID = _YIUIBase.YIUI.TypeConvertor.toInt(args[0]);
      } else {
        var form = cxt.form;

        if (form) {
          var doc = form.getDocument();

          if (doc) {
            OID = doc.getOID();
          }
        }
      }

      var paras = {
        OID: OID,
        cmd: "PureLoadProcessPath",
        service: "PureViewMap"
      };

      var rs = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

      return rs;
    };

    funs.RegisterAttachment = function (name, cxt, args) {
      var OID = _YIUIBase.YIUI.TypeConvertor.toLong(args[0]);

      var key = _YIUIBase.YIUI.TypeConvertor.toString(args[1]);

      var attachmentOID = _YIUIBase.YIUI.TypeConvertor.toLong(args[2]);

      var attachmentPara = "";
      if (args.length > 3) attachmentPara = _YIUIBase.YIUI.TypeConvertor.toString(args[3]);
      var attachmentInfo = "";
      if (args.length > 4) attachmentInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[4]);
      var paras = {
        OID: OID,
        Key: key,
        AttachmentOID: attachmentOID,
        AttachmentInfo: attachmentInfo,
        AttachmentPara: attachmentPara,
        cmd: "RegisterAttachment",
        service: "BPM"
      };

      var rs = _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras); //            var newForm = YIUI.FormBuilder.build(result);
      //            var container = form.getDefContainer();
      //            if (container == null) {
      //                container = form.getContainer();
      //            }
      //            newForm.setContainer(container);
      //            newForm.setSysExpVals(YIUI.BPMConstants.WORKITEM_VIEW, YIUI.BPMConstants.WORKITEM_VIEW);
      //            newForm.pFormID = form.formID;
      //            container.build(newForm);


      return true;
    };
  }
  {
    //Detail相关
    funs.OpenDetail = function (name, cxt, args) {
      var detailFormKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]),
          OID = _YIUIBase.YIUI.TypeConvertor.toString(args[1]),
          DOID = _YIUIBase.YIUI.TypeConvertor.toString(args[2]),
          form = cxt.form;

      if (DOID == undefined || DOID == "" || DOID == null) return;
      var paras = {
        cmd: "PureOpenDetail",
        OID: OID,
        DOID: DOID,
        detailFormKey: detailFormKey
      };

      var success = function success(jsonObj) {
        _YIUIBase.YIUI.FormBuilder.build(jsonObj, _YIUIBase.YIUI.FormTarget.MODAL, form.formID);
      };

      _YIUISvr["default"].SvrMgr.dealWithPureForm(paras, success);
    };

    funs.OpenDetailInGrid = function (name, cxt, args) {
      var detailFormKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]),
          OID = _YIUIBase.YIUI.TypeConvertor.toString(args[1]),
          DOID = _YIUIBase.YIUI.TypeConvertor.toString(args[2]),
          form = cxt.form,
          rowIndex = cxt.rowIndex;

      var paras = {
        cmd: "PureOpenDetail",
        OID: OID,
        DOID: DOID,
        inGrid: true,
        detailFormKey: detailFormKey
      };

      var success = function success(jsonObj) {
        var newForm = _YIUIBase.YIUI.FormBuilder.build(jsonObj, _YIUIBase.YIUI.FormTarget.MODAL, form.formID);

        newForm.getDocument().oid = OID;
        newForm.getDocument().clear();
        var pTable = form.getDocument().getByKey(newForm.refTableKey);
        var pGrid = form.getGridInfoByTableKey(newForm.refTableKey);
        pGrid = form.getComponent(pGrid.key);

        if (rowIndex == undefined) {
          rowIndex = pGrid.getFocusRowIndex();
        } else {
          pGrid.setFocusRowIndex(rowIndex);
        }

        var focusRow = pGrid.getRowDataAt(rowIndex);
        pTable.setByBkmk(focusRow.bookmark);
        var nTable = new _YIUICommon.DataDef.DataTable();

        _yesCommon.lodash.extend(true, nTable.cols, pTable.cols);

        nTable.key = pTable.key;
        nTable.addRow();

        for (var i = 0, len = nTable.cols.length; i < len; i++) {
          nTable.set(i, pTable.get(i));
        }

        nTable.batchUpdate();
        newForm.getDocument().add(newForm.refTableKey, nTable);
        newForm.showDocument();
      };

      _YIUISvr["default"].SvrMgr.dealWithPureForm(paras, success);
    };

    funs.SaveDetail = function (name, cxt, args) {
      var direct = false,
          form = cxt.form,
          doc = form.getDocument(),
          refTableKey = form.refTableKey,
          isStandalone = form.isStandalone();

      var pForm = _YIUIBase.YIUI.FormStack.getForm(form.pFormID);

      if (args.length > 0) {
        direct = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[0]);
      }

      var uiCheckOpt = new _YIUIBase.YIUI.UICheckOpt(form);
      if (!uiCheckOpt.doOpt()) return false;

      if (direct) {
        doc.docType = _YIUICommon.DataDef.D_ObjDtl;
        doc.setModified();

        var document = _YIUIBase.YIUI.DataUtil.toJSONDoc(doc);

        var params = {
          cmd: "PureSaveData",
          document: _yesCommon.lodash.toJSON(document),
          formKey: form.getFormKey()
        };

        var resultJson = _YIUISvr["default"].SvrMgr.dealWithPureData(params);

        _YIUIBase.YIUI.FormBuilder.diff(form, resultJson);

        if (!isStandalone) {
          pForm.reloadTable(refTableKey);
          pForm.showDocument();
        }
      } else {
        if (!isStandalone) {
          var pGrid = pForm.getGridInfoByTableKey(refTableKey);
          pGrid = pForm.getComponent(pGrid.key);
          var nTable = form.getDocument().getByKey(refTableKey),
              pTable = pForm.getDocument().getByKey(refTableKey),
              rowData;

          if (form.getOperationState() == _YIUIBase.YIUI.Form_OperationState.New) {
            pTable.addRow();
            var row,
                lastRowIndex = -1;

            for (var dlen = pGrid.getRowCount(), di = dlen - 1; di >= 0; di--) {
              row = pGrid.getRowDataAt(di);

              if (row.isDetail && row.bookmark != null) {
                lastRowIndex = di + 1;
                break;
              }
            }

            rowData = pGrid.addGridRow(null, lastRowIndex, false);
            rowData.bookmark = pTable.getBkmk();
          } else {
            rowData = pGrid.getRowDataAt(pGrid.getFocusRowIndex());
          }

          pTable.setByBkmk(rowData.bookmark);
          nTable.first();

          for (var i = 0, len = nTable.cols.length; i < len; i++) {
            pTable.set(i, nTable.get(i));
          }

          pGrid.showDetailRow(pGrid.getRowIndexByID(rowData.rowID), true);
        }
      }
    };

    funs.EditDetail = function (name, cxt, args) {
      var detailFormKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]),
          OID = _YIUIBase.YIUI.TypeConvertor.toString(args[1]),
          DOID = _YIUIBase.YIUI.TypeConvertor.toString(args[2]),
          form = cxt.form;

      if (DOID == undefined || DOID == "" || DOID == null) return;
      var paras = {
        cmd: "PureOpenDetail",
        OID: OID,
        DOID: DOID,
        doEdit: true,
        detailFormKey: detailFormKey
      };

      var success = function success(jsonObj) {
        _YIUIBase.YIUI.FormBuilder.build(jsonObj, _YIUIBase.YIUI.FormTarget.MODAL, form.formID);
      };

      _YIUISvr["default"].SvrMgr.dealWithPureForm(paras, success);
    };

    funs.EditDetailInGrid = function (name, cxt, args) {
      var detailFormKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]),
          OID = _YIUIBase.YIUI.TypeConvertor.toString(args[1]),
          DOID = _YIUIBase.YIUI.TypeConvertor.toString(args[2]),
          form = cxt.form,
          rowIndex = cxt.rowIndex;

      var paras = {
        cmd: "PureOpenDetail",
        OID: OID,
        DOID: DOID,
        inGrid: true,
        doEdit: true,
        detailFormKey: detailFormKey
      };

      var success = function success(jsonObj) {
        var newForm = _YIUIBase.YIUI.FormBuilder.build(jsonObj, _YIUIBase.YIUI.FormTarget.MODAL, form.formID);

        newForm.getDocument().oid = OID;
        newForm.getDocument().clear();
        var pTable = form.getDocument().getByKey(newForm.refTableKey);
        var pGrid = form.getGridInfoByTableKey(newForm.refTableKey);
        pGrid = form.getComponent(pGrid.key);

        if (rowIndex == undefined) {
          rowIndex = pGrid.getFocusRowIndex();
        } else {
          pGrid.setFocusRowIndex(rowIndex);
        }

        var focusRow = pGrid.getRowDataAt(rowIndex);
        pTable.setByBkmk(focusRow.bookmark);
        var nTable = new _YIUICommon.DataDef.DataTable();

        _yesCommon.lodash.extend(true, nTable.cols, pTable.cols);

        nTable.key = pTable.key;
        nTable.addRow();

        for (var i = 0, len = nTable.cols.length; i < len; i++) {
          nTable.set(i, pTable.get(i));
        }

        nTable.batchUpdate();
        newForm.getDocument().add(newForm.refTableKey, nTable);
        newForm.showDocument();
      };

      _YIUISvr["default"].SvrMgr.dealWithPureForm(paras, success);
    };

    funs.NewDetail = function (name, cxt, args) {
      var detailFormKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]),
          OID = _YIUIBase.YIUI.TypeConvertor.toString(args[1]),
          form = cxt.form,
          doc = form.getDocument();

      var paras = {
        cmd: "PureNewDetail",
        OID: OID,
        VERID: doc.verid,
        DVERID: doc.dverid,
        detailFormKey: detailFormKey
      };

      var success = function success(jsonObj) {
        _YIUIBase.YIUI.FormBuilder.build(jsonObj, _YIUIBase.YIUI.FormTarget.MODAL, form.formID);
      };

      _YIUISvr["default"].SvrMgr.dealWithPureForm(paras, success);
    };

    funs.NewDetailInGrid = function (name, cxt, args) {
      var detailFormKey = _YIUIBase.YIUI.TypeConvertor.toString(args[0]),
          OID = _YIUIBase.YIUI.TypeConvertor.toString(args[1]),
          form = cxt.form,
          doc = form.getDocument();

      var paras = {
        cmd: "PureNewDetail",
        OID: OID,
        VERID: doc.verid,
        DVERID: doc.dverid,
        inGrid: true,
        detailFormKey: detailFormKey
      };

      var success = function success(jsonObj) {
        _YIUIBase.YIUI.FormBuilder.build(jsonObj, _YIUIBase.YIUI.FormTarget.MODAL, form.formID);
      };

      _YIUISvr["default"].SvrMgr.dealWithPureForm(paras, success);
    };
  }
  {
    //BPMExtendFunction BPM扩展公式
    funs.TransferTask =
    /*#__PURE__*/
    function () {
      var _ref54 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee100(name, cxt, args) {
        var WID, form, wiInfo, operatorID, createRecord, userinfo, auditResult, srcOperator, transferType;
        return regeneratorRuntime.wrap(function _callee100$(_context100) {
          while (1) {
            switch (_context100.prev = _context100.next) {
              case 0:
                WID = parseFloat(args[0].toString()), form = cxt.form, wiInfo = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO), operatorID = parseFloat(args[1].toString()), createRecord = false, auditResult = -1, srcOperator = -1, transferType = -1;

                if (WID == -1) {
                  WID = wiInfo.WorkitemID;
                }

                if (args.length > 2) {
                  createRecord = args[2];
                }

                if (args.length > 3) {
                  userinfo = _YIUIBase.YIUI.TypeConvertor.toString(args[3]);
                }

                if (args.length > 4) {
                  auditResult = args[4];
                }

                if (args.length > 5) {
                  srcOperator = args[5];
                }

                if (args.length > 6) {
                  transferType = args[6];
                }

                _context100.next = 9;
                return _YIUIBase.YIUI.BPMService.transferTask(WID, operatorID, createRecord, userinfo, auditResult, srcOperator, transferType);

              case 9:
                _context100.next = 11;
                return viewReload(form);

              case 11:
              case "end":
                return _context100.stop();
            }
          }
        }, _callee100);
      }));

      return function (_x258, _x259, _x260) {
        return _ref54.apply(this, arguments);
      };
    }();

    funs.EndorseTask =
    /*#__PURE__*/
    function () {
      var _ref55 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee101(name, cxt, args) {
        var WID, form, wiInfo, operatorID, launchInfo, hide;
        return regeneratorRuntime.wrap(function _callee101$(_context101) {
          while (1) {
            switch (_context101.prev = _context101.next) {
              case 0:
                WID = parseFloat(args[0].toString()), form = cxt.form, wiInfo = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO), operatorID = parseFloat(args[1].toString()), launchInfo = "", hide = false;

                if (WID == -1) {
                  WID = wiInfo.WorkitemID;
                }

                if (args.length > 2) {
                  launchInfo = _YIUIBase.YIUI.TypeConvertor.toString(args[2]);
                }

                if (args.length > 3) {
                  hide = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[3]);
                }

                _context101.next = 6;
                return _YIUIBase.YIUI.BPMService.endorseTask(WID, operatorID, launchInfo, hide);

              case 6:
                _context101.next = 8;
                return viewReload(form);

              case 8:
              case "end":
                return _context101.stop();
            }
          }
        }, _callee101);
      }));

      return function (_x261, _x262, _x263) {
        return _ref55.apply(this, arguments);
      };
    }();

    funs.LaunchTask = function (name, cxt, args) {
      var WID = parseFloat(args[0].toString()),
          form = cxt.form,
          wiInfo = form.getSysExpVals(_YIUIBase.YIUI.BPMConstants.WORKITEM_INFO),
          nodeKey = args[1].toString(),
          operatorID = parseFloat(args[2].toString()),
          launchInfo = args[3].toString(),
          hideActiveWorkitem = args[4];
      var ppObject = {
        Type: 1,
        OperatorID: operatorID
      };

      if (WID == -1) {
        WID = wiInfo.WorkitemID;
      }

      _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, {
        service: "BPMExtend",
        cmd: "LaunchTask",
        WorkitemID: WID,
        NodeKey: nodeKey,
        LaunchInfo: launchInfo,
        HideActiveWorkitem: hideActiveWorkitem,
        PPObject: JSON.stringify(ppObject)
      });
    };

    funs.FillGrid = function (name, cxt, args) {
      var form = cxt.form,
          gridKey = args[0].toString(),
          fields = [],
          dataTable = args[1],
          grid = form.getComponent(gridKey);

      for (var i = 2, len = args.length; i < len; i++) {
        fields.push(args[i].toString());
      }

      var convertData = function convertData(cell, value) {
        var cellType = cell.edittype,
            result = value;

        if (cellType == "dict") {
          var itemKey = cell.itemKey;
          result = _YIUIBase.YIUI.UIUtil.convertDictValue(itemKey, cell, value);
        }

        return result;
      };

      dataTable.beforeFirst();

      while (dataTable.next()) {
        var index = grid.getLastDetailRowIndex();

        for (var j = 0, jLen = fields.length; j < jLen; j++) {
          var cellKey = fields[j],
              value = dataTable.get(j);
          var cell = grid.dataModel.colModel.cells[cellKey];
          value = convertData(cell, value);
          grid.setValueByKey(index, cellKey, value, true, true);
        }
      }
    };

    funs.FillGridData = function (name, cxt, args) {
      var form = cxt.form,
          dataTable = args[1],
          doc = form.getDocument(),
          clearOldData = false,
          gridKey = args[0].toString(),
          grid = form.getComponent(gridKey);

      if (args.length > 2) {
        clearOldData = _YIUIBase.YIUI.TypeConvertor.toBoolean(args[2]);
      }

      if (grid == null) {
        _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPONENT_NOT_EXISTS);
      }

      var tableKey = grid.tableKey,
          oldTable = doc.getByKey(tableKey);

      if (oldTable != null) {
        if (clearOldData) {
          _YIUIBase.YIUI.DataUtil.deleteAllRow(oldTable);
        }

        _YIUIBase.YIUI.DataUtil.append(dataTable, oldTable);

        oldTable.beforeFirst();
        grid.rootGroupBkmk = [];
        var gridLoader = new _YIUIBase.YIUI.ShowGridData(form, grid);
        gridLoader.load(); // grid.loadData();
      }
    };

    funs.LocaleString =
    /*#__PURE__*/
    function () {
      var _ref56 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee102(name, cxt, args) {
        var form, formKey, group, key, values, params, str;
        return regeneratorRuntime.wrap(function _callee102$(_context102) {
          while (1) {
            switch (_context102.prev = _context102.next) {
              case 0:
                form = cxt.form;
                formKey = form.getFormKey();
                group = args[0];
                key = args[1];
                values = [];
                params = {
                  service: "LocaleString",
                  formKey: formKey,
                  group: group,
                  key: key,
                  paras: _yesCommon.lodash.toJSON(values)
                };
                _context102.next = 8;
                return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, params);

              case 8:
                str = _context102.sent;

                if (!str) {
                  _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.NO_LOCALE_STRING_DEFINED);
                }

                return _context102.abrupt("return", str);

              case 11:
              case "end":
                return _context102.stop();
            }
          }
        }, _callee102);
      }));

      return function (_x264, _x265, _x266) {
        return _ref56.apply(this, arguments);
      };
    }();

    funs.LocaleParaFormat =
    /*#__PURE__*/
    function () {
      var _ref57 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee103(name, cxt, args) {
        var form, formKey, group, key, values, i, len, params, str;
        return regeneratorRuntime.wrap(function _callee103$(_context103) {
          while (1) {
            switch (_context103.prev = _context103.next) {
              case 0:
                form = cxt.form;
                formKey = form.getFormKey();
                group = args[0];
                key = args[1];
                values = [];

                for (i = 2, len = args.length; i < len; i++) {
                  values.push(args[i]);
                }

                params = {
                  service: "LocaleString",
                  formKey: formKey,
                  group: group,
                  key: key,
                  paras: _yesCommon.lodash.toJSON(values)
                };
                _context103.next = 9;
                return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, params);

              case 9:
                str = _context103.sent;

                if (!str) {
                  _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.NO_LOCALE_STRING_DEFINED);
                }

                return _context103.abrupt("return", str);

              case 12:
              case "end":
                return _context103.stop();
            }
          }
        }, _callee103);
      }));

      return function (_x267, _x268, _x269) {
        return _ref57.apply(this, arguments);
      };
    }();

    funs.LocaleFormat = funs.LocaleParaFormat;
  }
  return funs;
}();

var View = View || {};
exports.View = View;

(function () {
  View.FuncMap = new _YIUICommon.HashMap();
  Expr.regCluster(View.FuncMap, Expr.InnerFuns);
  Expr.regCluster(View.FuncMap, _YIUIBase.UI.BaseFuns);

  View.EvalEnv = function (form) {
    this.form = form;
    this.parser = new Expr.Parser(View.FuncMap);
  };

  _YIUICommon.Lang.impl(View.EvalEnv, {
    get: function get(cxt, id, scope, obj) {
      var form = cxt.form;
      var result = null;

      if (obj) {
        var heap = scope.getHeap();

        if (heap.contains(obj)) {
          var variable = heap.get(obj);
          var valueImpl = null;
          var para = scope.getPara(obj);

          if (para != null) {
            valueImpl = para;
          } else {
            if (variable instanceof _YIUICommon.DataDef.DataTable) {
              valueImpl = new _YIUIBase.YIUI.TblValImpl();
              scope.addPara(obj, valueImpl);
            } else if (variable instanceof _YIUICommon.DataDef.Document) {
              valueImpl = new _YIUIBase.YIUI.DocValImpl();
              scope.addPara(obj, valueImpl);
            } else if (_yesCommon.lodash.isObject(variable)) {
              valueImpl = new _YIUIBase.YIUI.JSONObjValImpl();
              scope.addPara(obj, valueImpl);
            } else if (_yesCommon.lodash.isArray(variable)) {
              valueImpl = new _YIUIBase.YIUI.JSONArrValImpl();
              scope.addPara(obj, valueImpl);
            }
          }

          result = valueImpl.getValue(variable, id);
        } else {
          result = _YIUIBase.YIUI.ExprUtil.getImplValue(form, id, cxt, obj);
        }
      } else {
        result = _YIUIBase.YIUI.ExprUtil.getImplValue(form, id, cxt);
      }

      return result;
    },
    set: function set(cxt, id, value, scope, obj) {
      var form = cxt.form;

      if (obj) {
        var heap = scope.getHeap();

        if (heap.contains(obj)) {
          var variable = heap.get(obj);
          var valueImpl = null;
          var para = scope.getPara(obj);

          if (para) {
            valueImpl = para;
          } else {
            if (variable instanceof _YIUICommon.DataDef.DataTable) {
              valueImpl = new _YIUIBase.YIUI.TblValImpl();
              scope.addPara(obj, valueImpl);
            } else if (variable instanceof _YIUICommon.DataDef.Document) {
              valueImpl = new _YIUIBase.YIUI.DocValImpl();
              scope.addPara(obj, valueImpl);
            } else if (_yesCommon.lodash.isObject(variable)) {
              valueImpl = new _YIUIBase.YIUI.JSONObjValImpl();
              scope.addPara(obj, valueImpl);
            }
          }

          valueImpl.setValue(variable, id, value);
        } else {
          _YIUIBase.YIUI.ExprUtil.setImplValue(form, id, value, cxt);
        }
      } else {
        _YIUIBase.YIUI.ExprUtil.setImplValue(form, id, value, cxt);
      }
    },
    checkMacro: function () {
      var _checkMacro = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee104(cxt, name) {
        var macro, form, paras;
        return regeneratorRuntime.wrap(function _callee104$(_context104) {
          while (1) {
            switch (_context104.prev = _context104.next) {
              case 0:
                form = cxt.form;

                if (!(form !== null && form !== undefined)) {
                  _context104.next = 10;
                  break;
                }

                macro = form.macroMap == undefined ? null : form.macroMap[name];

                if (!(macro == null)) {
                  _context104.next = 10;
                  break;
                }

                paras = {
                  service: "WebMetaService",
                  cmd: "GetMacro",
                  formKey: form.getFormKey(),
                  macroName: name
                };
                _context104.next = 7;
                return _YIUISvr["default"].Request.getSyncData(_YIUISvr["default"].SvrMgr.ServletURL, paras);

              case 7:
                macro = _context104.sent;

                if (!form.macroMap) {
                  form.macroMap = {};
                }

                form.macroMap[name] = macro;

              case 10:
                return _context104.abrupt("return", macro);

              case 11:
              case "end":
                return _context104.stop();
            }
          }
        }, _callee104);
      }));

      function checkMacro(_x270, _x271) {
        return _checkMacro.apply(this, arguments);
      }

      return checkMacro;
    }(),
    evalMacro: function () {
      var _evalMacro = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee105(cxt, scope, name, macro, args) {
        var newScope, argsList, heap, i;
        return regeneratorRuntime.wrap(function _callee105$(_context105) {
          while (1) {
            switch (_context105.prev = _context105.next) {
              case 0:
                newScope = new Expr.Scope(scope);
                argsList = macro.args;

                if (argsList !== null && argsList !== undefined && argsList.length > 0) {
                  heap = newScope.getHeap();

                  for (i = 0; i < argsList.length; i++) {
                    heap.put(argsList[i], args[i]);
                  }
                } // console.log(macro.content);


                _context105.next = 5;
                return this.parser.eval(this, _yesCommon.lodash.trim(macro.content), new Expr.Tree(), cxt, newScope);

              case 5:
                return _context105.abrupt("return", _context105.sent);

              case 6:
              case "end":
                return _context105.stop();
            }
          }
        }, _callee105, this);
      }));

      function evalMacro(_x272, _x273, _x274, _x275, _x276) {
        return _evalMacro.apply(this, arguments);
      }

      return evalMacro;
    }(),
    getLoop: function getLoop(cxt, name, loop, obj) {
      var objectLoop = null;

      switch (loop) {
        case 0:
          var key = obj.toString(),
              sepPos = key.indexOf(':'),
              para = null,
              comp;

          if (sepPos > 0) {
            para = key.substring(sepPos + 1);
            key = key.substring(0, sepPos);
          }

          comp = cxt.form.getComponent(key);

          if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.LISTVIEW) {
            objectLoop = new OBJLOOP.ListViewLoop(cxt, comp);
          } else if (comp.type == _YIUIBase.YIUI.CONTROLTYPE.GRID) {
            var includeEmptyRow = false;

            if ("empty" == para) {
              includeEmptyRow = true;
            }

            objectLoop = new OBJLOOP.GridLoop(cxt, comp, includeEmptyRow);
          }

          break;
      }

      return objectLoop;
    },
    resolveObject: function resolveObject(self, scope, obj) {
      var newCxt = null;
      var form = self.form;

      if (Expr.S_SELF == obj.toLowerCase()) {
        newCxt = self;
      } else if (Expr.S_PARENT == obj.toLowerCase()) {
        var parent = form.getParentForm();
        newCxt = {
          form: parent
        };
      } else {
        var comp = form.getComponent(obj);

        if (comp && comp.tagName == "stackcontainer") {
          var activeForm = comp.getActivePane();

          if (activeForm) {
            newCxt = activeForm.newCxt();
          } else {
            var viewCxt = {
              parent: form
            };
            newCxt = viewCxt;
          }
        } else {
          var v = scope.getHeap().get(obj);

          if (v != null) {
            newCxt = {
              obj: v
            };
          }
        }
      }

      return newCxt;
    },
    evalObject: function evalObject(cxt, obj, name, args) {
      return null;
    },
    evalFuncImpl: function evalFuncImpl(cxt, scope, obj, name) {
      var impl = null;
      var v = scope.getHeap().get(obj);

      if (v) {
        if (v instanceof _YIUICommon.DataDef.DataTable) {
          impl = View.TblFunImpl.get(name);
        } else if (v instanceof _YIUICommon.DataDef.Document) {
          impl = View.DocFunImpl.get(name);
        } else if (_yesCommon.lodash.isObject(v)) {
          impl = View.JSONObjFunImpl.get(name);
        } else if (_yesCommon.lodash.isArray(v)) {
          impl = View.JSONArrFunImpl.get(name);
        }
      }

      return impl;
    }
  });

  View.Parser = function (form) {
    this.form = form;
    this.parser = new Expr.Parser(View.FuncMap);
    this.env = new View.EvalEnv(form);
  };

  _YIUICommon.Lang.impl(View.Parser, {
    eval: function () {
      var _eval4 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee106(script, cxt, hack) {
        var tree, scope;
        return regeneratorRuntime.wrap(function _callee106$(_context106) {
          while (1) {
            switch (_context106.prev = _context106.next) {
              case 0:
                tree = new Expr.Tree();
                scope = new Expr.Scope();
                _context106.next = 4;
                return this.parser.eval(this.env, script, tree, cxt, scope);

              case 4:
                return _context106.abrupt("return", _context106.sent);

              case 5:
              case "end":
                return _context106.stop();
            }
          }
        }, _callee106, this);
      }));

      function _eval(_x277, _x278, _x279) {
        return _eval4.apply(this, arguments);
      }

      return _eval;
    }(),
    evalByTree: function () {
      var _evalByTree2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee107(tree, cxt, hack) {
        var scope;
        return regeneratorRuntime.wrap(function _callee107$(_context107) {
          while (1) {
            switch (_context107.prev = _context107.next) {
              case 0:
                scope = new Expr.Scope();
                _context107.next = 3;
                return this.parser.evalByTree(this.env, tree, cxt, scope);

              case 3:
                return _context107.abrupt("return", _context107.sent);

              case 4:
              case "end":
                return _context107.stop();
            }
          }
        }, _callee107, this);
      }));

      function evalByTree(_x280, _x281, _x282) {
        return _evalByTree2.apply(this, arguments);
      }

      return evalByTree;
    }(),
    getSyntaxTree: function getSyntaxTree(script) {
      var tree = new Expr.Tree();
      this.parser.parse(script, tree);
      tree.opti();
      return tree;
    }
  });

  View.Context = function (form) {
    this.form = form;
    this.cxt = new Object();
  };

  _YIUICommon.Lang.impl(View.Context, {
    setParent: function setParent(parent) {
      this.parent = parent;
    },
    getParent: function getParent() {
      return this.parent;
    },
    setRowIndex: function setRowIndex(rowIndex) {
      this.rowIndex = rowIndex;
    },
    setColIndex: function setColIndex(colIndex) {
      this.colIndex = colIndex;
    }
  });
})();

var OBJLOOP = OBJLOOP || {};

(function () {
  OBJLOOP.DataTableLoop = function (table) {
    this.table = table;
  };

  _YIUICommon.Lang.impl(OBJLOOP.DataTableLoop, {
    hasNext: function hasNext() {
      return this.table.getRowCount() > 0 && !this.table.isLast();
    },
    next: function next() {
      this.table.next();
    },
    clean: function clean() {}
  });

  OBJLOOP.GridLoop = function (cxt, grid, includeEmptyRow) {
    this.index = -1;
    this.context = cxt;
    this.includeEmptyRow = includeEmptyRow;
    this.key = grid.key;
    this.count = 0;

    if (this.includeEmptyRow) {
      this.count = grid.getRowCount();
    } else {
      for (var i = 0, tmpCount = grid.getRowCount(); i < tmpCount; i++) {
        var row = grid.getRowDataAt(i);

        if (!(row.isDetail && row.bookmark == undefined)) {
          this.count++;
        }
      }
    }
  };

  _YIUICommon.Lang.impl(OBJLOOP.GridLoop, {
    hasNext: function hasNext() {
      return this.count > 0 && this.index < this.count - 1;
    },
    next: function next() {
      ++this.index;
      this.context.key = this.key;
      this.context.rowIndex = this.index;
    },
    clean: function clean() {}
  });

  OBJLOOP.ListViewLoop = function (cxt, listView) {
    this.index = -1;
    this.context = cxt;
    this.key = listView.key;
    this.count = listView.data.length;
  };

  _YIUICommon.Lang.impl(OBJLOOP.ListViewLoop, {
    hasNext: function hasNext() {
      return this.count > 0 && this.index < this.count - 1;
    },
    next: function next() {
      ++this.index;
      this.context.key = this.key;
      this.context.rowIndex = this.index;
    },
    clean: function clean() {}
  });
})();

_YIUIBase.YIUI.UIUtil = function () {
  var Return = {
    getCompValue: function getCompValue(control, table) {
      var compValue = null;
      var value = null;
      var columnKey = control.columnKey;

      if (table.isValid()) {
        value = table.getByKey(columnKey);
      }

      if (value == null) {
        return value;
      }

      var type = control.type;
      var itemKey;

      switch (type) {
        case _YIUIBase.YIUI.CONTROLTYPE.DICT:
        case _YIUIBase.YIUI.CONTROLTYPE.COMPDICT:
        case _YIUIBase.YIUI.CONTROLTYPE.DYNAMICDICT:
          itemKey = control.itemKey;

          if (control.multiSelect) {
            if (_yesCommon.lodash.isEmptyObject(value)) {
              break;
            }

            if (type == _YIUIBase.YIUI.CONTROLTYPE.DYNAMICDICT) {
              itemKey = table.getByKey(columnKey + "ItemKey");
            }

            if (type == _YIUIBase.YIUI.CONTROLTYPE.COMPDICT) {
              _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPDICT_CANNOT_SET_MULTIDICT, control.key);
            }
          } else {
            if (parseInt(value) <= 0) {
              break;
            }

            if (type == _YIUIBase.YIUI.CONTROLTYPE.DYNAMICDICT) {
              itemKey = table.getByKey(columnKey + "ItemKey");

              if (itemKey == null || itemKey.length() == 0) {
                _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.DYNAMICDICT_ITEMKEY_NULL, control.key);
              }
            } else if (type == _YIUIBase.YIUI.CONTROLTYPE.COMPDICT) {
              itemKey = table.getByKey(columnKey + "ItemKey");

              if (itemKey == null || itemKey.length() == 0) {
                _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPDICT_ITEMKEY_NULL, control.key);
              }

              if (dict.isAllowMultiSelection()) {
                _YIUIBase.YIUI.ViewException.throwException(_YIUIBase.YIUI.ViewException.COMPDICT_CANNOT_SET_MULTIDICT, control.key);
              }
            }
          } //是否有预加载的caption字段


          var captionKey = columnKey + "_caption";
          var columnIndex = table.indexByKey(captionKey);
          compValue = convertDictValue(itemKey, dict, value);
          break;

        default:
          compValue = value;
      }

      return compValue;
    },
    convertDictValue: function convertDictValue(itemKey, dict, value) {
      var ret = null;

      if (dict.multiSelect) {
        var ids = value;

        if (ids.length > 0) {
          var oids = ids.split(",");

          if (oids.length > 0) {
            var list = [];
            var data = null;

            for (var i = 0, len = oids.length; i < len; i++) {
              data = {
                itemKey: itemKey,
                oid: oids[i]
              };
              list.add(data);
            }

            ret = list;
          }
        }
      } else {
        var oid = parseFloat(value);
        var data = {
          itemKey: itemKey,
          oid: oid
        };
        ret = data;
      }

      return ret;
    },
    format: function format(_format, args, startPos) {
      var result = "";
      var length = _format.length;

      var Para = function Para() {
        this.index = -1;

        this.setIndex = function (index) {
          this.index = index;
        };

        this.getIndex = function () {
          return this.index;
        };
      };

      var v = [];
      var l_i = 0;
      var i = 0;

      while (i < length) {
        var c = _format.charAt(i);

        if (c == '{') {
          var s_b = i;
          var s_e = -1; // 潜在的参数

          ++i;

          if (i < length) {
            c = _format.charAt(i);

            if (_yesCommon.lodash.isNumeric(c)) {
              while (_yesCommon.lodash.isNumeric(c) && i < length) {
                ++i;
                c = _format.charAt(i);
              }

              if (i < length) {
                if (c == '}') {
                  s_e = i;
                }

                ++i;
              }
            } else {
              ++i;
            }
          }

          if (s_e != -1) {
            // 生成之前的字符串
            if (l_i < s_b) {
              v.push(_format.substring(l_i, s_b));
            } // 找到一个参数


            var p = new Para();

            var s = _format.substring(s_b + 1, s_e);

            p.setIndex(parseInt(s));
            v.push(p);
            l_i = s_e + 1;
          }
        } else {
          ++i;
        }
      }

      if (l_i < i) {
        v.push(_format.substring(l_i));
      }

      var obj;

      for (var vi = 0, len = v.length; vi < len; vi++) {
        obj = v[vi];

        if (obj instanceof Para) {
          result += args[obj.getIndex() - 1 + startPos];
        } else {
          result += obj;
        }
      }

      return result;
    },

    /**
     * form: 被替换的form对象
     * formJson: 需要替换成的form的json序列化
     * cxt: 上下文环境
     */
    replaceForm: function replaceForm(form, formJson, cxt) {
      if (form.formKey != formJson.form.key) return form;
      var container = form.getContainer();
      var newForm = null;

      if (container) {
        var el = form.getRoot().el;
        var selLi = (0, _yesCommon.lodash)("li.aria-selected[aria-controls=" + el.attr("id") + "]");
        var ul = selLi.parent();
        var prevLi = selLi.prev();
        var nextLi = selLi.next();
        container.removeForm(form);
        formJson.form.formID = form.formID;
        newForm = _YIUIBase.YIUI.FormBuilder.build(formJson);
        newForm.pFormID = form.pFormID;
        newForm.entryKey = form.entryKey;
        newForm.initOperationState = form.initOperationState;
        container.isReplace = true;
        container.build(newForm);
        var newEl = newForm.getRoot().el;
        var newLi = (0, _yesCommon.lodash)("li[aria-controls=" + newEl.attr("id") + "]", ul);

        if (prevLi.length > 0) {
          prevLi.after(newLi);
        } else {
          nextLi.before(newLi);
        }
      } else {
        var parent = form.getRoot().el.parent();
        parent.empty();

        _YIUIBase.YIUI.FormStack.removeForm(form.formID);

        newForm = _YIUIBase.YIUI.FormBuilder.build(formJson);
        newForm.getRoot().render(parent);
      }

      newForm.target = form.target;

      if (newForm.target == 2) {
        (0, _yesCommon.lodash)("#" + form.formID).attr("id", newForm.formID);
      }

      newForm.dictCaption = form.dictCaption;

      var sysExpVals = _yesCommon.lodash.extend(form.sysExpVals, newForm.sysExpVals);

      newForm.sysExpVals = sysExpVals;
      cxt.form = newForm;
      return newForm;
    },
    isNull: function isNull(v) {
      return v == undefined || v == null;
    },
    getCompareType: function getCompareType(v1, v2) {
      var type = -1;

      if (this.isNull(v1)) {
        var temp = v2;
        v2 = v1;
        v1 = temp;
      }

      if (this.isNull(v1)) {
        type = _YIUICommon.DataType.NUMERIC;
      } else if (typeof v1 == "string") {
        if (_yesCommon.lodash.isNumeric(v1)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else {
          type = _YIUICommon.DataType.STRING;
        }
      } else if (typeof v1 == "number") {
        if (this.isNull(v2)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "string") {
          type = _YIUICommon.DataType.STRING;
        } else {
          type = _YIUICommon.DataType.NUMERIC;
        }
      } else if (typeof v1 == "boolean") {
        if (this.isNull(v2)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "number" || v2 instanceof _decimal["default"]) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "string") {
          type = _YIUICommon.DataType.STRING;
        } else if (typeof v2 == "boolean") {
          type = _YIUICommon.DataType.BOOLEAN;
        }
      } else if (v1 instanceof _decimal["default"]) {
        if (this.isNull(v2)) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "number" || v2 instanceof _decimal["default"]) {
          type = _YIUICommon.DataType.NUMERIC;
        } else if (typeof v2 == "string") {
          type = _YIUICommon.DataType.STRING;
        } else if (typeof v2 == "boolean") {
          type = _YIUICommon.DataType.BOOLEAN;
        }
      }

      return type;
    },
    dataType2JavaDataType: function dataType2JavaDataType(dataType) {
      switch (dataType) {
        case _YIUIBase.YIUI.DataType.LONG:
          return _YIUIBase.YIUI.JavaDataType.USER_LONG;

        case _YIUIBase.YIUI.DataType.BINARY:
          return _YIUIBase.YIUI.JavaDataType.USER_BINARY;

        case _YIUIBase.YIUI.DataType.BOOLEAN:
          return _YIUIBase.YIUI.JavaDataType.USER_BOOLEAN;

        case _YIUIBase.YIUI.DataType.DATE:
        case _YIUIBase.YIUI.DataType.DATETIME:
          return _YIUIBase.YIUI.JavaDataType.USER_DATETIME;

        case _YIUIBase.YIUI.DataType.DOUBLE:
        case _YIUIBase.YIUI.DataType.FLOAT:
        case _YIUIBase.YIUI.DataType.NUMERIC:
          return _YIUIBase.YIUI.JavaDataType.USER_NUMERIC;

        case _YIUIBase.YIUI.DataType.INT:
          return _YIUIBase.YIUI.JavaDataType.USER_INT;

        case _YIUIBase.YIUI.DataType.STRING:
        case _YIUIBase.YIUI.DataType.TEXT:
        case _YIUIBase.YIUI.DataType.FIXED_STRING:
          return _YIUIBase.YIUI.JavaDataType.USER_STRING;
      }

      return -1;
    },
    copyObject: function copyObject(obj) {
      return JSON.parse(JSON.stringify(obj));
    }
  };
  return Return;
}();

_YIUIBase.YIUI.BatchUtil = function () {
  var Return = {
    getViewSelOIDs: function getViewSelOIDs(form, defaultValue) {},
    getSelectOIDs: function getSelectOIDs(form, tblKey, oidKey, defaultValue) {
      var oids = new Array();
      var doc = form.getDocument();
      var table = doc.getByKey(tblKey);

      var s_Key = _YIUIBase.YIUI.DataUtil.getShadowTableKey(tblKey);

      var s_Tbl = doc.getByKey(s_Key); // 如果有影子表格,直接取影子表数据

      if (s_Tbl != null) {
        s_Tbl.beforeFirst();

        while (s_Tbl.next()) {
          var OID = s_Tbl.getByKey(oidKey);
          oids.push(OID);
        }
      } else {
        var lv = form.getListView(tblKey);
        var grid = form.getGridInfoByTableKey(tblKey);

        if (lv) {
          oids = lv.getFieldArray(form, oidKey);
        } else if (grid) {
          oids = grid.getFieldArray(form, oidKey);
        }
      }

      return oids;
    }
  };
  return Return;
}();